//==============================================================
//Written by: Philip A Covington, N8VB
//
//This software is licensed under the GNU General Public License
//==============================================================
//receiver.cs
//implements receiver functions for SDR
//==============================================================

/*
This file is part of a program that implements a Software-Defined Radio.

Copyright (C) 2007, 2008 Philip A Covington
Copyright (C) 2011-2012 Warren Pratt NR0V (wcpAGC code)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be reached by email at

p.covington@gmail.com
 * 
 * 
*/
using System;
using System.Runtime.InteropServices;
using System.Diagnostics;
using System.Runtime.Serialization;
using System.Threading;

namespace SharpDSPTx2._1
{	
	[Serializable()]
	public class Transmitter
	{
        [DllImport("kernel32.dll", CharSet = CharSet.Auto)]
        private static extern void OutputDebugString(string text);

		#region Private members

        internal DSPBufferTx txbuffer;        
        internal Filter filter;

		#endregion

		#region Constructor

		public Transmitter()
        {
            DSPStateTx state =  new DSPStateTx();
            
            txbuffer = new DSPBufferTx(state);
            
            // Filter
            filter = new Filter(ref txbuffer);
                        
		}

		#endregion
                

        #region Mutual Exclusion
        // use TxMutex.WaitOne() to claim, TxMutex.ReleaseMutex() to release
        private Mutex TxMutex = new Mutex();
        #endregion


        #region Public Properties

        public DSPStateTx DSPStateTxObj
        {
            get { return txbuffer.State; } 
        }
               
        public float FilterFrequencyLow
        {
            get { return filter.FilterFrequencyLow; }
            set 
            {
                TxMutex.WaitOne();
                filter.FilterFrequencyLow = value;
                TxMutex.ReleaseMutex();
            }
        }
        
        public float FilterFrequencyHigh
        {
            get { return filter.FilterFrequencyHigh; }
            set
            {
                TxMutex.WaitOne();
                filter.FilterFrequencyHigh = value;
                TxMutex.ReleaseMutex();
            }
        }
        

		private int iq_gain_value = 1;
		public int IQGainValue
		{
			get { return iq_gain_value; }
			set
			{
                TxMutex.WaitOne();
                iq_gain_value = value;
                TxMutex.ReleaseMutex();
            }
		}

		private int iq_phase_value = 0;
		public int IQPhaseValue
		{
			get { return iq_phase_value; }
			set
			{
                TxMutex.WaitOne();
                iq_phase_value = value;
                TxMutex.ReleaseMutex();
            }
		}
				
		private float samplerate = 48000F; // was 48000F  then 192000f changed by W3SZ
		public float SampleRate
		{
			get { return samplerate; }
			set
			{
                TxMutex.WaitOne();
                samplerate = value;
                txbuffer.State.DSPSampleRate = samplerate;
                TxMutex.ReleaseMutex();
            }
		}
								
		private WindowType_e window_type = WindowType_e.BLACKMANHARRIS_WINDOW;
		public WindowType_e WindowType
		{
			get { return window_type; }
			set
			{
                TxMutex.WaitOne();
                window_type = value;
                TxMutex.ReleaseMutex();
            }
		}
				
		private int block_size = 2048;
		public int BlockSize
		{
			get { return block_size; }
			set
			{
                TxMutex.WaitOne();
                block_size = value;
				fft_size = block_size * 2;
                TxMutex.ReleaseMutex();
            }
		}

		private int fft_size = 4096;
		public int FFTSize
		{
			get { return fft_size; }
			set
			{
                TxMutex.WaitOne();
                fft_size = value;
                TxMutex.ReleaseMutex();
            }
		}
        		
		private WindowType_e current_window = WindowType_e.BLACKMANHARRIS_WINDOW;
		public WindowType_e CurrentWindow
		{
			get { return current_window; }
			set
			{
                TxMutex.WaitOne();
                current_window = value;
                TxMutex.ReleaseMutex();
            }
		}
			
        
        #endregion
    }
}
