/* 
 * This software is derived from K.I.S.S. Konsole
 * K.I.S.S. Konsole for High Performance Software Defined Radio
 *
 * Developed from original code  Copyright 2006 (C) Phil Covington, N8VB
 * 
 * Copyright (C) 2009, 2010, 2011 Phil Harman, VK6APH
 * Copyright (C) 2009 David McQuate WA8YWQ
 * Copyright (C) 2009 Joe Martin K5SO
 * Copyright (C) 2009-2013 George Byrkit K9TRV
 * Copyright (C) 2009 Mark Amos W8XR
 * Copyright (C) 2011 Erik Anderson KE7YOA
 * Copyright (C) 2011-2012 Warren Pratt NR0V (wcpAGC code)
 * 
 * 
 * Copyright (C) 2009-2017 Roger Rehr, W3SZ
 * 
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * 
 */

/*

Below is summary by W3SZ

The program consists of two parts:
KISS KONSOLE SERVER: Server at radio location
MOVING FORMS TEST: Client running at operator location

MOVING FORMS TEST THREADS:

Form1: 
Primary GUI Thread for Main Controller Bar
Receive Audio Data Receiver Thread for Main Radio
Receive Audio Data Receiver Thread for Aux Radio
N1MM Communications Thread for Main Radio
N1MM Communications Thread for Aux Radio
Microphone Audio Sender Thread
CW Keying Sender Thread


50/144/222/432/903/1296/Micro/HF Radio Forms:
Primary Radio GUI Thread
Spectrum Data Receiver Thread
Power/SMeter/FFTSize/MOXSet Data Receiver Thread


Setup forms for 50/144/222/432/903/1296/Micro/HF radios:
Primary GUI Thread


Setup form for Main Controller bar (Form1):
Primary GUI Thread


KISS KONSOLE SERVER THREADS:

Form1:
Primary GUI Thread for Radio

Microphone Audio Data Receiver Thread (Data from Client)
CW Keying Data Receiver Thread (Data from Client)
Command Data Receiver Thread (Data from Client)
Power/SMeter/FFTSize/MOXSet Data Sender Thread (Data to Client)

EthernetDevice.cs:
Primary Thread
DataLoop Thread (Communicates with Metis)

HPSDRDevice.cs:
Primary Thread

USBDevice.cs:
Primary Thread
DataLoop Thread (Communicates with Ozy)

Wisdom.cs Form:
Primary Thread
Wisdom Thread

Setupform.cs Form:
Primary GUI Thread


See Code for details regarding SharpDSP, KISSKonsole.Setup, KISSAboutBox, DeviceTypeForm, DeviceChooserForm

*/

/*
 * Change log 
 * 
 *  1 June 2009 - Released as V1.0.0
 *  4 June 2009 - Allow keypad frequency entry with period for MHz
 *              - Changed frequency display to MHz.kHz Hz
 *  5 June 2009 - Improve Grid code and show all band edges in red
 *              - Fixed large memory size
 *  6 June 2009 - Make floats region independent (',' instead of '.')
 *              - Fixed bug in mouse frequency drag code.
 * 10 June 2009 - Added split display if both wide and narrow bandscopes being displayed
 * 11 June 2009 - Release as V1.0.1
 * 12 June 2009 - Add improved KKServer.csv file handling by George K9TRV
 *              - Change mouse cursor when over bandscope and when left mouse button down
 * 13 June 2009 - Read mouse speed and if necessary slow it down when drag tuning
 *              - S Meter test code from George K9TRV
 *              - Added mouse click tune
 *              - Working on mouse setting of the filter bandwidth
 * 14 June 2009 - fixed bug in drag tune rate - thanks Lyle KK7P for spotting!
 *              - Removed mouse sensitivity test since no longer needed
 *              - Added bandpass shift using mouse - WIP 
 *              - Added setting filter high and low frequencies with mouse -WIP
 * 15 June 2009 - Performance tuning by George K9TRV
 *              - More SMeter code by K9TRV
 *              - Limit bandpass shift to +/-9.999kHz
 *              - Limit filter drag to +/- 9.999kHz
 * 17 June 2009 - Textural SMeter working, fixed bug in click tune - both from George K9TRV
 * 20 June 2009 - Added Graphical S Meter
 *              - Standard calibrations added - will need a user control for fine calibration.
 * 21 June 2009 - Improvements to SMeter code, only draw background once - K9TRV
 *              - Added AGC hang time control.
 *              - Started contols for max, min and step settings for bandscope, still WIP
 * 22 June 2009 - Fixed bug in keyboard entry that cleared frequency display - K1RQG spotted this!
 *              - Added users settings for ANF and NR - K9TRV
 *              - Removed need to redraw bandscope backgound each update - K9TRV
 *              - Further work on max,min and step controls  - WIP
 * 24 June 2009 - Added FM and SAM modes - K9TRV
 *              - Completed bandscope controls, settings saved to KKServer.csv
 * 25 June 2009 - Improved bandscope drawing - K9TRV
 *              - Updated bandscope calibration description
 *              - Added bandwidth controls for SAM and FM
 *              - Increased max filter width in CW mode to 1kHz
 * 11 July 2009 - Send C&C data to Ozy if nothing received from EP6 to force it to send data
 *              - Setup now tool strip item so will open and close correctly - Harman Jnr
 *                good to see all those college $'s were well spent!
 *              - Release as V1.0.2
 * 12 July 2009 - Start of Waterfall display or how to use all your CPU in a few lines of code!
 * 13 July 2009 - Quick hack to correct click tune for CWL - needs more work 
 * 14 July 2009 - More Waterfall work - much less CPU - K9TRV
 *              - Added colours for the different waterfall levels
 * 16 July 2009 - Added seperate pictureboxs for each display - K9TRV
 *              - Added adjustable size for bandscope windows - K9TRV
 * 17 July 2009 - Added automatic setting for Waterfall low colour i.e. 'Waterfall AGC'
 * 18 July 2009 - Release as V1.0.3
 *              - Added squelch control for bandscope
 *              - Added squelch control for filter bandwidth
 *              - Added background colour for squelch control to show status
 * 23 July 2009 - Reduced CPU load by changing ADC Overload LED method - thanks Willi SM6OMH for spotting this!
 * 25 July 2009 - Added GUI for squelch level control and compenstated for filter BW
 * 26 July 2009 - Release as V1.0.4
 * 29 July 2009 - Merged Penelope code from Dave, WA8YWQ
 * 30 July 2009 - Added Hardware configuration options and selection for Penelope
 *              - Added 20dB Mic Gain option for Penelope
 *              - Added Hardware selection for Excalibur & save setting in KKServer.csv
 *              - Added 10MHz clock selection & save setting in KKServer.csv
 * 31 July 2009 - Added Tabs to Setup form - K9TRV
 *              - Added Tx filter high and low selection and save in KKServer.csv
 *              - Added Tx drive control and save in KKServer.csv
 *              - Added Tx gain per band to setup form and save values in KKServer.csv
 *  1 Aug  2009 - Added Full Duplex option and save setting in KKServer.csv
 *              - Added MOX buttton, mute receiver audio if not full duplex
 *              - Added Tune button and % power setting on setup form and save in KKServer.csv
 *                  TODO: Cancel MOX/TUN if active and Start clicked
 *  2 Aug  2009 - Added CW Pitch control to setup, filter settings and save values in KKServer.csv
 *              - Fixed bug when opening setup form - K9TRV
 *              - Started adding J6 output controls for Penelope
 *              - Added PTT from Atlas Bus
 *  3 Aug  2009 - PTT from Atlas immediately sent back via C&C C0 to minimise delays
 *              - Added Penny Open Collector control group enable/disable and save in KKServer.csv
 *              - More code for Penny Open Collector controls
 *  4 Aug  2009 - Yet more code for Penny Open Collector controls - who's bright idea was it to have 154? :-)
 *  6 Aug  2009 - Completed Penny Open Collect controls - K9TRV
 *              - Added Penny Open Collector controls to C&C data
 *              - Replaced Start check box with ON/OFF button 
 *              - Interlocked ON/OFF with MOX and TUN controls
 *  7 Aug  2009 - Added independent Duplex controls for Tx and Rx
 *              - Code review and improvements - K9TRV
 *  8 Aug  2009 - Release as V1.0.5
 *              - Start support for CW - add raised cosine profile for CW note
 *  9 Aug  2009 - Completed CW raised cosine profile
 *              - Added PPT sequencer controls
 * 10 Aug  2009 - Completed CW profile and PTT sequencer
 * 11 Aug  2009 - Fixed bug in Penny Open Collector code that had bits shifted
 *              - Renamed Mercury_send() and to_Mercury[] to Data_send and to_Device[] now we have Penny support
 *              - Tidy comments up
 * 15 Aug  2009 - Fixed bug that meant SSB was always USB
 *              - Changed "120m" to "12m"
 *              - Prevented PTT on AM and FM from latching
 *              - Prevent selection of options that are only relevant with Penelope fitted
 *              - Added display of software and FPGA code versions on Setup form
 *              - Check that Penny is present if selected
 *              - Only allow PTT, MOX and TUN if Penny selected
 * 16 Aug  2009 - Fixed long term dropped audio at 48kHz bug - K9TRV
 *              - Code tidy and improvements - K9TRV
 * 19 Aug  2009 - Moved to Ozy_Janus.rbf V1.6. C0[2] = DOT, C0[1] = Dash, C0[0] = PTT
 *              - Started coding for full Duplex operation
 * 20 Aug  2009 - Added state machine to Data_send() for Duplex code
 *              - Added dot and dash key signals and temporarily linked to PTT for testing CW code.
 *              - Tested Duplex code with Mercury V2.8, works fine, now need GUI to support it.
 * 21 Aug  2009 - Provide Method for I2C mic gain so can be toggled whilst Txing
 *              - Warn user if Penny selected but no 10MHz reference selected
 *              - Fixed the bug in the 'Start' routine when after loading Ozy returns a null string - K9TRV
 * 28 Oct  2009 - Test version for Ethernet using SharpPcap.
 * 29 Oct  2009 - Added sequence number check for data from PC.
 *  4 Nov  2009 - Added receive data flag so can indicate when we loose sync.
 * 30 Jan  2010 - Test Version for Ethernet UDP/IP.
 *
 *  ******** MAKE SURE THE ARP TABLE CONTAINS AN ENTRY FOR THE MAC ON Metis  *********************
 * 18 Mar  2010 - Changed UPD/IP receiver to poll mode and time out after 1 second   
 * 
 * 13 June 2010 - Added listener.ReceiveBufferSize = 0xFFFF;   // no lost frame counts at 192kHz with this setting
 * 29 June 2010 - Removed all USB code to make this an Ethernet only version.
 * 30 June 2010 - Reads PC IP address and use this with assigned port to listen on for Ozy data.
 *  5 July 2010 - Calls ARP_update.bat when starting to force ARP entry for Metis board
 * 20 Aug  2010 - Does Metis Discovery call when selecting Start and checks that board is present
 *              - Removed call to ARP batch file when loading form since Metis now supports ARP natively
 * 10 Nov  2010 - Added Mic gain support for Metis board.
 * 12 Nov  2010 - When doing discovery get Metis IP address from IP header rather than UDP/IP Payload
 *              - Added Metis start/stop sending data command to Start() and Stop()
 * 17 Nov  2010 - Added call to Metis_start_stop() in Form1_FormClosed
 *                Added Metis MAC address display to Setupform Ethernet page.
 *                It gets its value in MetisDiscovery()
 *                Changed all references to OzyII to Metis
 * 26 Nov  2010 - Fixed error in the way that sequence number was being sent to Metis.
 *              - Removed redundant code due to changes to how IP address assigned to Metis.
 * 06 Dec  2010 - V0.0.2 - G Byrkit reorged the code to get rid of static data members and initializers.  Moved init stuff to 'start',
 *                except kept getting local computer name and ethernet ports in Form1_Load, but update ethernet ports in 'Start()'
 * 08 Dec  2010 - Added more Console debug messages during Metis Discovery
 * 11 Dec  2010 - V0.0.3 - G Byrkit: switched to IPAddress.Any for the discovery listen and listener listen bind addresses.
 *                Seems to find Metis on either of my two NICs, regardless of which NIC is used.  Both APIPA and DHCP addressing tested.
 * 12 Dec  2010 - V0.0.4 - P Harman: Reset sequence_number in Start() to eliminate error message when restarting program.
 *              - fixed typo in MAC address from Metis to Console.
 * 14 Dec  2010 - V0.0.5 - G Byrkit: efficiency improvements.  Replace all the uses of .Checked with bools that are set when the
 *                  Checked_changed event fires.  Much more efficient than referencing control properties like .Checked.
 *                  Also eliminated many buffer allocations and array copy operations within Data_send.
 *                  Other changes to ComputeDisplay to remove array allocations each time in and to remove Point allocations each
 *                  time thru the function.
 * 20 Dec  2010 - V0.0.6 - G Byrkit: efficiency improvements.  make sure sequence_number++ is done as 'unchecked' to prevent overflow
 *                exception.  Use pntr++ rather than pntr + n or pntr + x + n in Data_send.  Fix error that Ozy_thread_started wasn't
 *                being set to false when thread was terminated.  Resulted in trying to kill thread extra times (when program exited)
 *                and trying to close sockets multiple times.
 * 21 Dec  2010 - V0.0.7 - G Byrkit: efficiency improvements.
 *                1) provide int enum replacements for BandText and ModeText, so that the switch statements don't have to compare strings
 *                2) hoist TXGain and MicOutGain out of both NESTED for() loops they were in.  calculation could be done once before the
 *                loop and not inside the inner loop for each data item!
 *  *                
 *  9 Jan  2011 - V0.0.8 - P.Harman: Added 'pump prime' send to Metis at Start to select clock source. Metis now responds to the Port
 *                address that the PC sends to. Discovery uses the same Port to receive and reply on.  Port is hard coded at 1024
 *                for now. Need to determine a free port and use that in future.  Removed all code relating to Discovery socket.
 *                
 * 15 Jan  2011 - V0.0.9  - P.Harman: Now uses just one .NET socket. Code automatically allocates a free port for Metis to reply to.
 * 16 Jan  2011 - V0.0.10 - P.Harman: Metis now always listens on port 1024, changed code to suite. 
 * 29 Jan  2011 - V0.0.11 - G Byrkit, K9TRV: Add all processing as in KISS USB.  Set Copyright date to modern, rationalize USB and Ethernet
 *                  versions.  PLEASE TEST CAREFULLY AND COMPLETELY!  (support for Hermes, PennyLane, advanced audio processing, etc.)
 *                  Also set x86 as default build
 * 30 Jan  2011 - V0.0.12 - P.Harman: remove 'Test' slider
 * 30 Jan  2011 - V0.0.13 - G Byrkit, K9TRV: fix some 'gotchas' that Phil found: Setup: checking PennyLane doesn't allow ExtCtrl stuff to
 *                  be selected.  On main form: mic gain and processor sliders need to be 'touched': code to give initial values missing
 *                  in Form1() (constructor), added.
 * 31 Jan  2011 - V0.0.14 - P.Harman, moved Tx Drive Level out of nested for() loop.
 * 31 Jan  2011 - V0.0.15 - G Byrkit: fixed Setup so that PennyLane selection doesn't leave transmit controls greyed out.  Moved freq
 *                              text box up off the spectrum, just like KISS USB.
 *  2 Feb  2011 - V0.0.16 - P.Harman, enable selection of Mic boost when using PennyLane. Added constructor for Processor Gain.
 *                          Renew clipper level when selecting. Added label to VOX level control. Only display Clip LED when PTTEnable true.
 *                          Add delay to Noise Gate. 
 *  4 Feb  2011 - V0.0.17 - Added Line In selection to Setup.cs. Included in I2C and To_Ozy(). Fixed bug where not able to select Penny10MHz clock.
 *  5 Feb  2011 - V0.0.18 - P.Harman - Altered sequence of testing for mic or line-in so that mic gain changes correctly.  Fixed bug that entered . in frequency
 *                                      from keypad.
 *  5 Feb  2011 - V1.0.0  - First release.                                    
 *  6 Feb  2011 - V1.0.1  - Fix 2 problems: SMeter on Vista and Win7, if font is NOT normal size, changes to 'too big' and causes display
 *                          errors in the text.  Also, the Freq display was set to 'RightToLeft', which causes eg 18.245000 to display as
 *                          '000 18.245', which is wrong.  Changed to LeftToRight, and set text to be right-justified.
 * 14 Feb  2011 - V1.0.2  - Added PTT Delay
 *  3 Mar  2011 - V1.0.3  - P.Harman - Altered sequence of testing for mic or line-in so that mic gain changes correctly.
 *  9 Apr  2011 - V1.0.4  - P.Harman - Added support for wide band Bandscope from Metis_V1.2 code
 *  3 Jun  2011 - V1.0.5  - G.Byrkit - made multi-metis detection and selection work.  Save last Metis/Hermes MAC in KKServer.csv file for
 *                          next run.  Change how detection works: rather than IPAddress.Any, it attempts discovery on each ethernet port
 *                          one at a time.  This works on G.Byrkit's system, wher IPAddress.Any method has distinct flaws: won't discover
 *                          on any port but the first, but USED to work in the past.  Either a driver or windows issue, so a method that
 *                          works reliably all the time is better.  Lays foundation for detecting Metis vs Hermes and choosing accordingly.
 *                          Also remove vestiges of 'choosing an ethernet adapter' in setup.
 * 30 Jul  2011           - Added Mic AGC on/off control. Added squarelaw to Mic, VOX and Noise gate controls.
 *                        - Fixed error in I&Q levels when using PennyLane
 *                        - Added sequence numbers for wide Bandscope data
 *						  - Corrected direction of VOX level control
 *	5 Aug  2011			  - Saved Mic AGC, Noise Gate and Level to KK.CVS
 *	9 Aug  2011 - V1.1.0  - G Byrkit (K9TRV) made first 'unified' version with USB and Ethernet support in one exe.  Split Ethernet code
 *	                        and most variables out of form1.cs.  Ethernet-specific went into EthernetDevice.cs.
 *	                        Generic went into HPSDRDevice.cs.  Made USBDevice.cs from the USB-specific code.  USBDevice.cs and EthernetDevice.cs
 *	                        are derived from HPDSRDevice.cs (class inheritance).
 * 15 Aug  2011 - V1.1.1  - G Byrkit (K9TRV) add 'short height' mode to the display tab of setup form. When not checked, main form is
 *                          800 pixels tall, when checked, main form is 580 pixels tall, to better support laptops and netbooks that
 *                          have 'short' screens (600 pixels high).
 * 17 Aug  2011 - V1.1.2  - G Byrkit (K9TRV) back out changes to support 'short' displays.  What worked on WinXP failed to work on
 *                          Windows 7!  The problem was how the main form sized itself on win7 when using the 125% or 150% larger fonts.
 *                          there was no problem on XP when using 'large fonts', however.  So this effort temporarily shelved.
 *                          But DID add support for 'Alex' and 'Apollo' in KKServer.csv and in the MainForm, via boolean variables.  Also,
 *                          in the data loop in USB and Ethernet code added decoding the values of AIN1-AIN6 (forward & reflected (reverse)
 *                          power, supply power volts) for use on Penelope, PennyLane, Apollo, Hermes and Alex.  These values are not yet
 *                          displayed.  Just decoded.
 * 27 Aug  2011 - V1.1.3  - G Byrkit (K9TRV) provisional Alexaires support added (antenna selection by band, atten by band, display of
 *                          forward and reverse power.
 * 28 Aug  2011 - V1.1.4  - G Byrkit (K9TRV) Wideband Spectrum display DID NOT WORK in USB (Ozy/Magister).  Fixed.
 * 28 Aug  2011 - V1.1.5  - G Byrkit (K9TRV) Waterfall display performance improvements from Erik Anderson KE7YOA
 * 01 Oct  2011 - V1.1.6  - G Byrkit (K9TRV) Phil's fixes to Alex incorporated.  Also, moved Process_data to HPSDRDevice.cs, along whith
 *                          the core of Data_send (called Data_send_core), which makes the bulk of the RX decoding and TX encoding all
 *                          common code.  Much easier to maintain!
 * 05 Jan  2012 - V1.1.7  - G Byrkit (K9TRV) Integrate Warren Pratt's (NR0V) AGC code. No 'graphical' AGC adjust yet, but show knee and
 *                          hang threshold on the panadapter (spectrum) display.  Provide to limited audience for review, correction, etc.
 * 14 Jan  2012 - V1.1.8  - G Byrkit (K9TRV) further work on NR0V Warren's AGC code.  Get Knee and Hang lines to display at correct location.
 *                          Fix problem with using LSB or other modes where filter_high is less than filter_low (like is negative as on LSB)
 *                          all of which caused NaN to be computed as log of a negative number is NOT defined.
 * 14 Jan  2012 - V1.1.9  - G Byrkit (K9TRV) AGC Knee and AGC Hang mouse drag implemented.  Sent out for testing.
 * 01 Apr  2012 - V1.1.10 - G Byrkit (K9TRV) fix in HPSDRDevice.cs in Data_send_core so that Penny OC outputs for TX are selected if
 *                          either PTT is pressed or MOX is pressed (MainForm.PTT || MainForm.MOX_set) is true.  Also in Process_data
 *                          make separate version checking and warning messages for Hermes to separate it from Mercury et al, as
 *                          firmware versions are different.
 * 01 May  2012 - V1.1.11 - G Byrkit (K9TRV) changed window type in SharpDSP for the wideband spectrum.  Was 'Hamming', now
 *                          'Blackman-Harris'.  The result is that the peaks are much sharper, according to Phil.  Other issues related
 *                          to Hermes handled in the code.  Most significantly, Phil VK6APH changed the version numbers that Hermes was
 *                          reporting, so that only the equivalent to 'Ozy version' is relevant.  That is, Penelope and Mercury versions
 *                          are NOT relevant.  So some code changed so that inappropriate versions are not reported.
 * 07 July 2012 - V1.1.12 - G Byrkit (K9TRV) bumped version number because of initozy11.bat file changes and addition of other
 *                          Ozy_janus.rbf file (.v18 thru .v22).  .v21 is the default file that is used.
 * 08 July 2012 - V1.1.13 - G Byrkit (K9TRV) Added testing whether Initozy11.bat is present in the current directory.  Provide a warning
 *                          and search for possible mis-named files that could be initozy11.bat.  Also, add a 'SkipVersionCheck' to the
 *                          main form's data, to KKServer.csv, and to the first tab of the setup form.  Also, add the new Metis version 18 to
 *                          the Ethernet CheckVersions function.
 * 15 July 2012 - V1.1.14 - G Byrkit (K9TRV) Implement 16k samples for Mercury v33, Metis v19, per VK6APH's (Phil's) request
 * 17 July 2012 - V1.1.15 - G Byrkit (K9TRV) Phil reports the above works!  Now, add a checkbox to Setup to allow attempting to connect
 *                          to the previous Ethernet device (quick connect) and only if that fails do enumeration/discovery of all
 *                          Metis/Hermes devices.  If unchecked, do the discovery always when the radio is started.
 * 18 July 2012 - V1.1.16 - G Byrkit (K9TRV) Ken N9VV requested TX bandwidth on setup increased from 5000 to 8000 max for 'wideband'
 *                          strange stuff...  Also looks like the ProcessorFilter bandwidth isn't being kept in sync with the TransmitFilter
 *                          bandwidth.  checking with Phil VK6APH on this...
 * 10 Aug  2012 - V1.1.17 - G Byrkit (K9TRV) convert to VS2010 and using .Net Framework 4.0.  Add a manifest that runs 'requireAdministrator'
 *                          rather than 'asInvoker'.  Update the USB (Ozy) utils to those built with VS2010 and having manifests that 'requireAdministrator'
 *                          Also have a manifest for InitOzy11.bat (InitOzy11.bat.manifest) that also requests 'requireAdministrator',
 *                          just in case it helps.  This (use of 'requireAdministrator') is intended to help those on Vista, Win7, and the
 *                          future Win8 which is released next week.
 * 24 Aug  2012 - V1.1.18 - G Byrkit (K9TRV) make an installer using WiX.  Also make KKServer.csv save into AppDataDir + "\OpenHPSDR\KISS Konsole\".  No
 *                          longer is KKServer.csv part of what's downloaded or distributed.  So it won't be overwritten by new versions of KK.
 *  9 Sep  2012 - V1.1.19 - G Byrkit (K9TRV) allow a command line argument of 'appdatadir' (case insensitive) followed by a relative or absolute path
 *                          that will be used as the path to where the KKServer.csv file will be stored.  If relative, the path will be under
 *                          AppDataDir + "\OpenHPSDR".  This will allow multiple instances of KK on the same system.  Otherwise, they would all share
 *                          the same KKServer.csv file.  Note that the installer will only allow 1 version to be 'installed' at any one time.  but that
 *                          doesn't prevent you from making copies of the executable folder.
 * 11 Oct  2012 - V1.1.20 - G Byrkit (K9TRV) update to account for new firmware versions for Ozy/Metis/Penny/Mercury/Hermes
 * 26 Oct  2012 - v1.1.21 - G Byrkit (K9TRV) Change to .Net Framework 4.0 CLIENT PROFILE so that it runs on more computers without extra software
 *                          Also change installer to reflect this.  Also add new versions of firmware, and note that Metis 1.9, Metis 2.0 and Ozy 2.3
 *                          all have various defects and mostly should NOT be used.  Add Ozy v2.4 to installer and initozy11.bat.  Ozy v2.1 still the default...
 * 31 Oct  2012 - v1.1.22 - G Byrkit (K9TRV) Phil advised me that there is NOT YET a Hermes V1.9.  Must have been one of those Hermes/Metis confusions.
 *                          So change the code and re-release so that KK will work with Hermes 1.8 without complaining.
 * 16 Nov  2012 - v1.1.23 - G Byrkit (K9TRV) Bill Diaz discovered a problem with fast-connect to previous device when the IP addr of the previous device
 *                          had changed (likely due to DHCP handing out a different IP address).  Fixed in EthernetDevice.cs.  Also, previous version
 *                          identified itself as 1.1.21 and not as 1.1.22 as it should have.
 * 15 Dec  2012 - v1.1.24 - G Byrkit (K9TRV) Accommodate Metis 2.2, which fixes an occasional hang in previous versions of Metis.
 * 1  Jan  2013 - v1.1.25 - G Byrkit (K9TRV) Accommodate Metis 2.3.  Change copyright dates to include 2013
 * 20 Jan  2013 - v1.1.26 - G Byrkit (K9TRV) Accommodate Metis 2.4.
 * 27 Jan  2013 - v1.1.27 - G Byrkit (K9TRV) Implement 384k spectrum size for Hermes only for now.  Also remove 'Unified wcpAGC' from the title bar
 *                          version string.
 * 26 May  2013 - v1.1.28 - G Byrkit (K9TRV) Allow 384k spectrum for Metis and Ozy as well.  Add support for Mercury 3.4, Penny 1.8, Metis 2.6, Ozy 2.5.
 *                          Ozy/Magister 2.5 set as default in InitOzy11.bat
 *                          
 *    
 * 
 * TODO:        - Save IQScale in KKServer.csv and set it accordingly at start
 *              - Investigate CPU usage when ANF/NR on at 192kHz sampling rate
 *              - Change USB buffer sizes as a function of sampling rate i.e. 
 *                    WriteBuf size = 1024 bytes for samples/block < 1024, 2048 otherwise
 *                    ReadBuf size = WriteBuf size * (1,2,4) for (48k, 96k, 192k) sample rates
 *              - Check 1Hz steps with mouse scroll is OK with other mice.
 *              - Review CPU usage of ANF and NR values 
 *              - Add  multi band EQ for the receiver (like the 10 band one in PowerSDR)
 *              - User calibration of frequency and amplitude
 *              - Add a second receiver within the current bandwidth
 *              - Fix click tune for CWL - need to allow for LSB filter and CW tone frequency
 *              
 *
 * 
 * 
 */

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Threading;
using System.Threading.Tasks;
using SharpDSP;
using SharpDSPTx;
using System.Diagnostics;   // use View > Output to see debug messages
using System.Globalization;
using System.Net;
using System.Net.NetworkInformation;  // so we don't have country issues with '.' and ',' in KKServer.csv files

using System.Timers;  //w3sz
using System.Net.Sockets; //w3sz
using System.Media; //w3sz
using System.Collections; //w3sz
using System.Linq; //w3sz
using System.Resources;  //w3sz
using System.Reflection;  //w3sz
using System.Numerics;  //w3sz
using System.Runtime.InteropServices; //w3sz
using FFTWSharp;  //w3sz
using System.Xml; //w3sz

using CSCore; //w3sz
using CSCore.Streams;
using CSCore.SoundOut;
using Microsoft.DirectX.DirectSound;
using System.Collections.Concurrent;
using FragLabs.Audio.Codecs;  //w3sz This Is Opus.

namespace KISS_Konsole
{
    public partial class Form1 : Form
    {


        const int OPUS_BANDWIDTH_NARROWBAND = 1101; /**< 4 kHz bandpass @hideinitializer*/
        const int OPUS_BANDWIDTH_MEDIUMBAND = 1102; /**< 6 kHz bandpass @hideinitializer*/
        const int OPUS_BANDWIDTH_WIDEBAND = 1103; /**< 8 kHz bandpass @hideinitializer*/
        const int OPUS_BANDWIDTH_SUPERWIDEBAND = 1104; /**<12 kHz bandpass @hideinitializer*/
        const int OPUS_BANDWIDTH_FULLBAND = 1105; /**<20 kHz bandpass @hideinitializer*/

        private object _waterfallLockObj = new object();


        [DllImport("User32.dll", CharSet = CharSet.Ansi, SetLastError = true, ExactSpelling = true)]
        private static extern bool MoveWindow(IntPtr hWnd, int x, int y, int w, int h, bool Repaint);

        //class includes CW Sidetone dot and dash events //w3sz

        public delegate void DotOrDashEventHandler(object sender, DotOrDashEventArgs e);

        public bool closeflag = false;

        public event DotOrDashEventHandler DotOrDashEvent;
        public IAsyncResult iar;
        public readonly byte[] udpbyteon = {1};
        public readonly byte[] udpbyteoff = {0};
        public int icw = 0;

        public class DotOrDashEventArgs : EventArgs
        {
            private bool _dot = false;
            private bool _dash = false;
            private IAsyncResult iasresult = null;

            public DotOrDashEventArgs(bool dot1, bool dash1)
            {
                _dot = dot1;
                _dash = dash1;
                iasresult = iar;

            }
            public bool dot1
            {
                get { return _dot; }
            }
            public bool dash1
            {
                get { return _dash; }
            }
            public IAsyncResult iar
            {
                get { return iasresult; }
            }
        }
            

        private void OnDotOrDashEvent()
        {
            if (DotOrDashEvent != null)
            {
                EventHandler<DotOrDashEventArgs> handler = HPSDRDevice_DotOrDashEvent;
                handler.BeginInvoke(this, new DotOrDashEventArgs(GlobalVar.dot, GlobalVar.dash), EndAsyncHandler, handler);
            }
        }

        void EndAsyncHandler(IAsyncResult result)
        {

        }

        void HPSDRDevice_DotOrDashEvent(object sender, DotOrDashEventArgs e)
        {

            if ((GlobalVar.dot == true) || (GlobalVar.dash == true))
            {
                icw = 0;
                if (checkBox1.Checked)
                {
                   sidetone_volume.Volume = sidetonevol;
                }
                else sidetone_volume.Volume = 0.0f;
                }

            else if ((GlobalVar.dot == false) & (GlobalVar.dash == false))
            {
                if (icw < 10)
                {
                   icw = icw + 1;
                }

                try
                {
                    sidetone_volume.Volume = 0.0f;
                }                  
	            catch (Exception ex)
	            {
	                Console.WriteLine(ex.ToString());
	            }
            }
        }


        //end of this w3sz section...

        // create a delegate for the text display since may be called from another thread
        public string Ozy_version = null;  // holds version of Ozy code loaded into FX2 or Metis

        public string numberOfIPAdapters;
        public string Network_interfaces = null;  // holds a list with the description of each Network Adapter
        public int adapterSelected = 1;           // from Setup form, the number of the Network Adapter to use

        //put Sharp_DSP Stuff here
        
        //FFTSize is W3SZ variable for spectrum and waterfall, has nothing to do with
        //SharpDSP which is for audio
        public static int FFTSize = Properties.Settings.Default.FFTSize;
        public int OldDSPFFTSize = FFTSize;
        public string VHFBand = Properties.Settings.Default.VHFBand;
        public decimal alpha = Properties.Settings.Default.alpha;

        public bool doWideBand = false;
        public bool Allow16kWidebandSamples = false;
  
        public static int rbufSize = 2048; // A buffer of 2048 bytes holds 1024 16-bit samples  // w3sz was 2048 // increased to 8192 and then to 16384 to 32768 to improve filter skirts
        public static int iqsize = rbufSize / 2; 
        public static int outbufferSize = iqsize;  //DSPBlockSize in SharpDSP also increased from 1048 to 4096 to 8192
        public static int txbufSize = 2048; // A buffer of 2048 bytes holds 1024 16-bit samples  // w3sz was 2048 // increased to 8192 and then to 16384 to 32768 to improve filter skirts
        public static int txiqsize = txbufSize / 2;
        public static int txoutbufferSize = txiqsize;  

        // no longer a const int, which can be used at compile time, but a static int, which can be
        // used at runtime, but not thru the object reference, only thru the classname reference.

        public static int EP4BufSize = 8192;         // EP4 data is 4096 16-bit samples
        public float[] FullBandwidthPowerSpectrumData = new float[EP4BufSize];
        public float[] PowerSpectrumData = new float[rbufSize];

        public byte[] frequency = new byte[4];             // hold Penny and Mercury frequency when in simplex mode and Penny when in Duplex   
        public byte[] duplex_frequency = new byte[4];      // holds Mercury frequency when in Duplex mode  
        public byte[] split_tx_frequency_bytes = new byte[4];      // holds Tx frequency when in Split mode

        public SharpDSP2._1.DSPState state;         // public so that the Setup form can use it
        public SharpDSP2._1.DSPBuffer SignalBuffer;
        public SharpDSP2._1.Receiver rcvr;          // public so that the Setup form can use it 

        public SharpDSPTx2._1.DSPStateTx txstate;         // public so that the Setup form can use it
        public SharpDSPTx2._1.DSPBufferTx txSignalBuffer;
        public SharpDSPTx2._1.Transmitter tx;

        //w3sz set up a second Signal Buffer to handle PowerSpectrum to make the large FFT Spectrum and Waterfall displays  
        // w3sz public so that other forms can use it

        public Complex[] SignalBuffer2 = new Complex[FFTSize];
      //  float[] PowerSpectrum = new float[FFTSize];

        int step_size; // the frequency step using the mouse scroll wheel
        public bool read_ready; // set when we have read 1024 samples from USB port
        int[] I_sample = new int[iqsize]; // holds the I samples from Ozy
        int[] Q_sample = new int[iqsize]; // holds the Q samples from Ozy

        public OperMode CurrentMode;
        public string ModeText;
        int previous_frequency_value;
        public int SampleRate;
        public int CWPitch = 700;   // used to calculate filter center frequency, mouse tuning, etc.

        // the following contain the last used VFO frequency on each band
        int set_frequency_160, set_frequency_80, set_frequency_40, set_frequency_30, set_frequency_20;
        int set_frequency_17, set_frequency_15, set_frequency_12, set_frequency_10, set_frequency_6, set_frequency_GC;
        // the following hold the last state of the preamp on each band
        bool Preamp_160, Preamp_80, Preamp_40, Preamp_30, Preamp_20, Preamp_17, Preamp_15, Preamp_12, Preamp_10;
        bool Preamp_6, Preamp_GC;

        string BandText; // holds the previous band when we change bands so we can restore the frequency we were on
        public bool PTT = false;           // true when PTT on Atlas is active
        public bool LineIn = false;        // set when line input on Penelope or PennyLane selected

        public int Merc_version;           // Version number of Mercury FPGA code
        public int Penny_version;          // Version number of Penelope FPGA code
        public int Ozy_FPGA_version;       // Version number of Ozy FPGA code

        public bool SkipVersionCheck = false;

        public int rate;            // sets bandscope update rate in F.P.S.
        public int Rate
        {
            get { return rate; }
            set
            {
                rate = value;
                if (rate <= 0)
                    timer1.Interval = 2000;
                else
                    timer1.Interval = 1000 / rate;  //timer value from 1000 to 50 i.e. 1 second to 20mS
            }
        }
        
        public int IQScale = 6;     // sets default display scale of I and Q signals

        public int GridMax = 0;     // sets maximum value of Bandscope display
        public int GridMin = -160;  // sets minimum value of Bandscope display
        public int GridStep = 20;   // sets distance between grid lines on bandscope 

        public int WaterfallStep = 0; // sets the distance in pixels the line moves per Waterfall scan

        int delta;                  // difference between the current and previous mouse position on the bandscope
        int QuickMemory;            // holds current Quick Memory frequency
        string QuickMemoryBand;     // holds the current Quick Memory band
        public string Your_call;           // holds the users call sign
        string KeypadFrequency = null;     // holds the frequency entered using the keypad
        int keycount = 0;           // holds number of key presses of the  keypad
        bool ShowCross = false;     // when set show cross at current mouse location
        int MousePositionX;         // as it says
        int MousePositionY;         // you guessed it!
        int FilterLeft;             // left hand location of filter on bandscope
        int FilterRight;            // Hmm... now let me see...
        bool FrequencyDrag = true;  // true when we can drag tune with the mouse
        bool BandwidthShift = true; // true when can drag shift the filter
        bool HighFilter = false;    // true when we can drag high filter frequency
        bool LowFilter = false;     // true when we can drag low filter frequency
        public bool WaterfallAGC = false;  // true when Waterfall AGC is selected on the Setup form
        float max_signal = -200.0f;      // maximum value of nPS in the array, used by  bandscope squelch
        int filter_max_signal = -200;  // maximum signal through the current filter, used by the filter, squelch
        int pixels_per_GridStep;     // how many pixels a Grid Step represents in the narrow bandscope 
        public bool IsSynced = false;      // true when we have sync from Ozy
        public bool IsInSequence = true;    // true when we have received frames in order
        public bool ADCOverload = false;     // true when ADC is overloaded
        public bool KK_on = false;           // true when KK is running
        int VOXHang = 0;

        int squelch_y_value = 0;                // Y location of the squelch threshold line 
        Rectangle squelch = new Rectangle();    // the little rectangle at the LH end of the squelch line.
        bool SquelchDrag = false;               // true when you are dragging the squelch level 

        public bool Hermes = false;         // true if Hermes is selected
        public bool PennyLane = false;      // true if PennyLane is selected
        public bool PenneyPresent = false;  // true if a Penelope card is fitted or not
        public bool Alex = false;           // true if Alex is present
        public bool Apollo = false;         // true if Apollo is present
        public bool MicGain20dB = false;    // true if 20dB mic gain is active on Penny or PennyLane
        public bool Penelope10MHz = false;  // true if 10MHz reference comes from Penny or PennyLane
        public bool Atlas10MHz = false;     // true if 10MHz reference comes from Atlas or Excalibur
        public bool Mercury10MHz = false;   // true if 10MHz reference comes from Mercury
        public bool Excalibur = false;      // true if Excalibur card fitted
        public float TxFilterHigh = 3500f;  // Transmitter high filter setting
        public float TxFilterLow = 300f;    // Transmitter low filter setting
        public float DriveGain = 0.50f;     // Determines the drive level to Penelope
        public static int Gain160m = 50;           // Determines gain for Penelope on 160m
        public static int Gain80m = 50;            // ditto for 80m etc
        public static int Gain60m = 50;
        public static int Gain40m = 50;
        public static int Gain30m = 50;
        public static int Gain20m = 50;
        public static int Gain17m = 50;
        public static int Gain15m = 50;
        public static int Gain12m = 50;
        public static int Gain10m = 50;
        public static int Gain6m = 50;
        public static int BandGain = 50;
        public static int Gain50 = 50;
        public static int Gain144 = 50;
        public static int Gain222 = 50;
        public static int Gain432 = 50;
        public static int Gain903 = 50;
        public static int Gain1296 = 50;
        public static int Gain2304 = 50;
        public static int Gain3456 = 50;
        public static int Gain5760 = 50;
        public static int Gain10368 = 50;
        public static int Gain24192 = 50;// Determines gain for Penelope, set by what ever band selected

        public bool Duplex = false;         // Selects full duplex operation
        public bool OnlyTxOnPTT = true;     // part of full duplex, only Tx when PTT set
        public bool MOX_set = false;               // set when PTT active
        public bool TUN_set = false;               // set when Tune active
        public int TuneLevel = 50;          // sets Tune power as a % of Drive setting
        public bool previous_OnlyTxOnPTT;          // hold previous Only Tx on PTT start so we can restore 
        // Penny Open Collector settings 
        public bool PennyOC = false;        // when true enables Penny OC settings
        public int Penny160mTxOC = 0;
        public int Penny160mRxOC = 0;
        public int Penny80mTxOC = 0;
        public int Penny80mRxOC = 0;
        public int Penny60mTxOC = 0;
        public int Penny60mRxOC = 0;
        public int Penny40mTxOC = 0;
        public int Penny40mRxOC = 0;
        public int Penny30mTxOC = 0;
        public int Penny30mRxOC = 0;
        public int Penny20mTxOC = 0;
        public int Penny20mRxOC = 0;
        public int Penny17mTxOC = 0;
        public int Penny17mRxOC = 0;
        public int Penny15mTxOC = 0;
        public int Penny15mRxOC = 0;
        public int Penny12mTxOC = 0;
        public int Penny12mRxOC = 0;
        public int Penny10mTxOC = 0;
        public int Penny10mRxOC = 0;
        public int Penny6mTxOC = 0;
        public int Penny6mRxOC = 0;

        // the default alex state has TX antenna set to '1' and RX antenna set to '1'
        const int DefaultAlexState = 65;

        public int Alex160mState = DefaultAlexState;
        public int Alex80mState = DefaultAlexState;
        public int Alex60mState = DefaultAlexState;
        public int Alex40mState = DefaultAlexState;
        public int Alex30mState = DefaultAlexState;
        public int Alex20mState = DefaultAlexState;
        public int Alex17mState = DefaultAlexState;
        public int Alex15mState = DefaultAlexState;
        public int Alex12mState = DefaultAlexState;
        public int Alex10mState = DefaultAlexState;
        public int Alex6mState = DefaultAlexState;
        public int AlexGCState = DefaultAlexState;

        // amount of attenuation currently selected on Alex or equiv (like on Hermes/Apollo)
        public double AlexAtten = 0.0;

        public int FM_deviation = 2400;    // sets the +/- FM deviation limits

        public string Metis_IP_address = "";             // Start() copies above value here, so it won't change until re-starting.
        public string MetisMAC = "";

        // PTT Sequencer
        public int DelayRF = 0; // number of mS to delay  RF after PTT activated
        public int DelayPTT = 0; // number of mS to delay release of PTT after RF stops

        public bool PTTEnable = false;

        Point MouseOld = new Point(); // keep location of mouse for drag tuning 

        public SetupForm Setup_form = null;  // get reference to the Setup form.

        NumberFormatInfo nfi = NumberFormatInfo.InvariantInfo;  // so we are region independent in terms of ',' and '.' for floats

        // these are used by the SMeter 
        int SMeterValue = 0;          // this gets passed to the SMeter code 
        Font meterFont = new Font(FontFamily.GenericSansSerif, 7);  // use smaller font for the meter

        Pen YellowPen = Pens.Yellow;
        Pen RedPen = Pens.Red;
        Pen WhitePen = Pens.White;
        Pen ChartreusePen = Pens.Chartreuse;
        Pen dotPen = (Pen)Pens.DarkOrange.Clone();

        public KKMethod KKDevice = KKMethod.Ethernet;

        public bool DoFastEthernetConnect = false;
        public string EthernetHostIPAddress = "";

        public string app_data_path = "";
        public string KKCSVName = "";

        public static string w3szendpoint = Properties.Settings.Default.w3szendpoint; //w3sz
        static public int server2port = Properties.Settings.Default.server2port; //6000 for spectrum send to client
        static public int server3port = Properties.Settings.Default.server3port; //7000 send pwr/S meter data to client
        static public int server4port = Properties.Settings.Default.server4port; //8000 commands from client
        static public int server6port = Properties.Settings.Default.server6port; //10000 for main receive audio send to client
        static public int server7port = Properties.Settings.Default.server7port; //11000 for mic audio from client
        static public int server8port = Properties.Settings.Default.server8port; //12000 for CW keying from client
        static public int server9port = Properties.Settings.Default.server9port; //13000 for aux receive audio send to client

        IPEndPoint localpt2 = new IPEndPoint(IPAddress.Parse(w3szendpoint), server2port);  //w3sz spectrum to client
        UdpClient udpServer2 = new UdpClient();  //w3sz channel 1 spectrum

        static public UdpClient udpServer6 = new UdpClient(); //w3sz 
        static public IPEndPoint localpt6 = new IPEndPoint(IPAddress.Parse(Form1.w3szendpoint), server6port);  //w3sz send receive audio to client

        public UdpClient udpServer7 = new UdpClient(); //w3sz 
        public IPEndPoint localpt7 = new IPEndPoint(IPAddress.Parse(Form1.w3szendpoint), server7port);  //w3sz mic audio from client

        public UdpClient udpServer8 = new UdpClient(); //w3sz 
        public IPEndPoint localpt8 = new IPEndPoint(IPAddress.Parse(Form1.w3szendpoint), server8port);  //w3sz CW keying from client
  
        static public UdpClient udpServer9 = new UdpClient(); //w3sz 
        static public IPEndPoint localpt9 = new IPEndPoint(IPAddress.Parse(Form1.w3szendpoint), server9port);  //w3sz send receive audio to client


        static Random _r = new Random();  //w3sz

        public bool DSPchanged = false;
        public BackgroundWorker backgroundWorker1 = new BackgroundWorker();
        public BackgroundWorker backgroundWorker2 = new BackgroundWorker();
        public static string WisdomName;
        public static bool WisdomComplete = false;
        public float sidetonevol;

        public bool EnableBandData;
        public Int32 LPT_Addr;
        public string mwband;
        public bool chkClient;
        public bool chkEtherAudio;
        public bool chkLocalAudio;
        public bool EtherAudioMain;
        public bool EtherAudioAux;
        public bool voice;
        public bool voice1;
        public static int txbufmsec; //tx audio buffer length expressed as msec
        public static int txbuflen;  //tx audio buffer length expressed as number of samples
        public bool formShown = false;

        public Form1(string appDataDir)
        {
            //Control.CheckForIllegalCrossThreadCalls = false;  // leave on so we catch these
            InitializeComponent();

            this.Shown += new System.EventHandler(this.Form1_Shown);
        

            txbufmsec = Properties.Settings.Default.txbufmsec;
            txbuflen = txbufmsec * 48;
            Console.WriteLine("Tx buffer is " + txbuflen);
            alphaUpDown.Value = alpha;
            DisplayFrequencyB(Properties.Settings.Default.split_tx_freq);
            comboBox1.Text = Properties.Settings.Default.DCWAtten.ToString();
            splitBox.Checked = false;
            fagcKneecomboBox.Text = Properties.Settings.Default.fagcKnee.ToString();
            fAGCLimitcomboBox2.Text = Properties.Settings.Default.fagcLimit.ToString();

            DotOrDashEvent += new DotOrDashEventHandler(HPSDRDevice_DotOrDashEvent);

            rcvr = new SharpDSP2._1.Receiver();
            SharpDSP2._1.PowerSpectrumSignal.ps_event += new SharpDSP2._1.PowerSpectrumSignal.PSpectrumEventHandler(PowerSpectrumSignal_ps_event);
            SharpDSP2._1.OutbandPowerSpectrumSignal.ps_event += new SharpDSP2._1.OutbandPowerSpectrumSignal.PSpectrumEventHandler(OutbandPowerSpectrumSignal_ps_event);

            rcvr.Alpha = alpha;
            rcvr.freqAGCKnee = Properties.Settings.Default.fagcKnee;
            rcvr.freqAGCLimit = Properties.Settings.Default.fagcLimit;
            rcvr.freqAGCType = AGCSpeed.Text;
            rcvr.freqOutputAGCType = AGCSpeed.Text ;

            rcvr.freqNoAGCMult = Properties.Settings.Default.noAGCMult;
            rcvr.freqAGCMult = Properties.Settings.Default.AGCMult;
            

            getDCW();
         
            state = rcvr.DSPStateObj;       // fetch state that Receiver constructor created
            state.DSPBlockSize = iqsize;     // put in some initial values

            tx = new SharpDSPTx2._1.Transmitter();
            txstate = tx.DSPStateTxObj;
            txstate.DSPBlockSize = txiqsize;
            
            // determine where a default appDataDir would be located
            app_data_path = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) + "\\OpenHPSDR\\KISS Konsole\\";

            // see if the non-default appDataDir is provided, and if so, whether it is valid or can be created.
            if ((appDataDir != null) && (appDataDir.Length > 0))
            {
                bool relativePath = true;
                try
                {
                    relativePath = !Path.IsPathRooted(appDataDir);
                    if (relativePath)
                    {
                        appDataDir = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) + "\\OpenHPSDR\\" + appDataDir;
                        appDataDir = Path.GetFullPath(appDataDir) + "\\";
                    }
                    try
                    {
                        if (!Directory.Exists(appDataDir))
                        {
                            Directory.CreateDirectory(appDataDir);
                        }
                        app_data_path = appDataDir;
                    }                  
		            catch (Exception ex)
		            {
		                Console.WriteLine(ex.ToString());
		            }
                }                  
	            catch (Exception ex)
	            {
	                Console.WriteLine(ex.ToString());
	            }

            }

            if (!Directory.Exists(app_data_path))
            {
                Directory.CreateDirectory(app_data_path);
            }

            WisdomName = app_data_path + "Wisdom";

            if (!File.Exists(WisdomName))   //w3sz if wisdom file doesn't exist, create it
            {
                Console.WriteLine("Will create new Wisdom file at {0}", app_data_path.ToString());
                Wisdom wisdom;
                wisdom = new Wisdom();
                Console.WriteLine("Created new Wisdom file at {0}", app_data_path.ToString());
            }
            else
            {
                WisdomComplete = true;
                Console.WriteLine("Wisdom file exists at {0}", app_data_path.ToString());
            }

            // now read the lines in the config file.  Warn if action above didn't create one!
            // This allows KKServer.csv to be no longer part of the SVN tree.  This means that the KKServer.csv
            // file won't get mangled/updated by SVN when someone updates the code!


            // Load the previous radio settings from the KKServer.csv file and allocate values.
            // First check that the file exists
            KKCSVName = app_data_path + "KKServer.csv";
            if (!File.Exists(KKCSVName))   //if file doesn't exist, create it
            {
                CreateKKCSV(KKCSVName);
            }

            // now read the lines in the config file.  Warn if action above didn't create one!
            // This allows KKServer.csv to be no longer part of the SVN tree.  This means that the KKServer.csv
            // file won't get mangled/updated by SVN when someone updates the code!
            if (File.Exists(KKCSVName))   //if file exists open it
            {
                ReadKKCSV(KKCSVName);

                BandSelect.Text = BandText;    // set  band to start on 
                Frequency_change();                 // check we are in band, update frequency and display 
                ChangeMode();                       // Set Bandwidth slider value based on Mode in use
                rcvr.SampleRate = SampleRate;   // set the previously saved sample rate
                QuickMemory = set_frequency.Value;  // set a quick frequency value
                QuickMemoryBand = BandSelect.Text;  // set a quick frequency band

                // Force the Form1 Controls to the values retrieved from KKServer.csv
                SetVolume(this, EventArgs.Empty);           // force Volume control to accept previous saved value
                AGCTrackBar_Scroll(this, EventArgs.Empty);  // force AGC Threshold to accept previous saved value
                Filter_squelch_CheckedChanged(this, EventArgs.Empty); // force Squelch to accept previous saved value
                Squelch_setting.Text = (Squelch_level.Value / 10).ToString();
                DriveLevel_Scroll(this, EventArgs.Empty);   // force Drive contol to accept value from KKServer.csv
                chkClipper_CheckedChanged(this, EventArgs.Empty); // force Processor control to update
                MicrophoneGain_Scroll(this, EventArgs.Empty);
                NoiseGateLevel_Scroll(this, EventArgs.Empty);

                Zoom_Track_Bar_Scroll(this, EventArgs.Empty);

                rcvr.SampleRate = SampleRate;   // set the previously saved sample rate
                AdjustFilterByMode();
                tx.SampleRate = SampleRate; 
                
                // set the bandwidth since dependent on sample rate

                ModeText = Mode.Text;
                if((ModeText == "USB") || (ModeText == "LSB") || (ModeText == "FM") || (ModeText == "AM"))
                {
                    voice = true;
                }
                else
                {voice = false;}

                rcvr.LOFrequency = 0.0f;
                //rcvr.BinauralMode = true;             // currently has no effect
                rcvr.PowerSpectrumUpdateRate = 50;      // Set maximum PowerSpectrum update rate
                //rcvr.PowerSpectrumCorrection = 1f;
                FilterBox.Text = Properties.Settings.Default.FilterType;
                FilterChange();
                rcvr.WindowType = SharpDSP2._1.WindowType_e.HANNING_WINDOW;  //12/20/2016 was BLACKMAN3_WINDOW
                tx.WindowType = SharpDSPTx2._1.WindowType_e.BLACKMAN3_WINDOW;
                // set up the squelch control 
                Squelch_level.Minimum = GridMin * 10; Squelch_level.Maximum = GridMax * 10;
                Squelch_setting.Text = (Squelch_level.Value / 10).ToString();
                
                previous_OnlyTxOnPTT = OnlyTxOnPTT;

                SignalBuffer = new SharpDSP2._1.DSPBuffer(state);
                txSignalBuffer = new SharpDSPTx2._1.DSPBufferTx(txstate);

                testbytes();  //added by w3sz just a reminder of how to do conversions to/from byte array

            }
            else
            {
                MessageBox.Show("Can't find KKServer.csv, should be in the same directory as KISS Konsole.exe?", "File Error");
            }

            GlobalVar.FFTSize = FFTSize;
            GlobalVar.DSPBlockSize = FFTSize / 2;
            GlobalVar.PSBlockSize = GlobalVar.DSPBlockSize;

            Zoom100.Checked = true;
            Zoom100_CheckedChanged(this, EventArgs.Empty);

            backgroundWorker1.WorkerReportsProgress = true;
            backgroundWorker1.WorkerSupportsCancellation = true;
            backgroundWorker1.DoWork += new DoWorkEventHandler(backgroundWorker1_DoWork);
            backgroundWorker1.ProgressChanged += new ProgressChangedEventHandler(backgroundWorker1_ProgressChanged);
            backgroundWorker1.RunWorkerCompleted += new RunWorkerCompletedEventHandler(backgroundWorker1_RunWorkerCompleted);
            if (chkClient == true)
            { 
            try
            {   //Spectrum send to client
                udpServer2.ExclusiveAddressUse = false;
                udpServer2.Client.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);
                udpServer2.Client.Connect(localpt2); //w3sz changed from .Bind Jan 2024

                //Main Receive Audio send to client
                udpServer6.ExclusiveAddressUse = false;
                udpServer6.Client.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);
                udpServer6.Client.Connect(localpt6); //w3sz changed from .Bind Jan 2024

                //Mic Audio receive from client
                udpServer7.ExclusiveAddressUse = false;
                udpServer7.Client.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);
                udpServer7.Client.Connect(localpt7); //w3sz changed from .Bind Jan 2024

                //CW Key receive from client
                udpServer8.ExclusiveAddressUse = false;
                udpServer8.Client.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);
                udpServer8.Client.Connect(localpt8); //w3sz changed from .Bind Jan 2024

                //Aux Receive Audio send to client
                udpServer9.ExclusiveAddressUse = false;
                udpServer9.Client.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);
                udpServer9.Client.Connect(localpt9); //w3sz changed from .Bind Jan 2024

                    Console.WriteLine("UDP SERVERS SET UP SUCCESSFULLY");

            }                  
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }




            //;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

            // start getting mic audio data from client
            ThreadPool.QueueUserWorkItem(delegate //w3sz
            {
                if (chkClient == true)
                {
                    UdpClient udpServer7 = new UdpClient(server7port); //w3sz
                    IPEndPoint inEndPoint7 = new IPEndPoint(IPAddress.Parse(w3szendpoint), server7port); //w3sz
     
                    OpusDecoder _decoder;
                    _decoder = OpusDecoder.Create(48000, 1);
                    _decoder.ForwardErrorCorrection = true;

                    while (true) //w3sz
                    {
                            byte[] buffer7 = udpServer7.Receive(ref inEndPoint7);
                            int len;
                            len = buffer7.Length;
                        if (true) //.Settings.Default.opusTxComp || (digiChk.Checked && !Properties.Settings.Default.opusTxComp))
                        {
                            try
                            {
                                if (buffer7.Length > 0)
                                {
                                    byte[] buffer8 = _decoder.Decode(buffer7, buffer7.Length, out len);
                                    testmicprovider.Write(buffer8, 0, len); //buffer for decompressed mic audio to WaveOut
                                    testmicprovider2.Write(buffer8, 0, len); //buffer for decompressed mic audio to Tx Processor
                                }
                            }
                                              
				            catch (Exception ex)
				            {
				                Console.WriteLine(ex.ToString());
				            }
                        }
                        else
                        {
                            try
                            {
                                if (buffer7.Length > 0)
                                {
                                    testmicprovider.Write(buffer7, 0, len); //buffer for mic audio to WaveOut
                                    testmicprovider2.Write(buffer7, 0, len); //buffer for mic audio to Tx Processor
                                }
                            }
                                              
				            catch (Exception ex)
				            {
				                Console.WriteLine(ex.ToString());
				            }
                        }
                    }
                }
            });


            //;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
            // start getting CW data from client
            ThreadPool.QueueUserWorkItem(delegate //w3sz
            {
                if (chkClient == true)
                {
                    UdpClient udpServer8 = new UdpClient(server8port); //w3sz
                    IPEndPoint inEndPoint8 = new IPEndPoint(IPAddress.Parse(w3szendpoint), server8port); //w3sz

                    while (true) //w3sz
                    {
                       try
                    {
                        byte[] buffer = udpServer8.Receive(ref localpt8); //w3sz
                        if (buffer[0] == 0x01)
                        {
                            if (GlobalVar.dot == false)
                            {
                                GlobalVar.dot = true;
                                button1.BackColor = Color.Red;
                   //                  Console.WriteLine("Key is ON");
                            }
                        }
                        else if(buffer[0] == 0x00)
                        {
                            GlobalVar.dot = false;
                            button1.BackColor = SystemColors.Control;
               //             Console.WriteLine("Key is OFF");
                            
                        }
                        
                    }                                      
		            catch (Exception ex)
		            {
		                Console.WriteLine(ex.ToString());
		            }
                }

                    }
                
            });
            //;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

            SharpDSP2._1.CPX[] outbuffer = new SharpDSP2._1.CPX[Form1.outbufferSize];     // allocate a buffer for DSP output
            SharpDSP2._1.CPX[] IQ_data_matrix = new SharpDSP2._1.CPX[Form1.outbufferSize];

            if (_encoder != null)
            {
                _encoder.Dispose();
            }

            //;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
            // finish processing Receive Audio Data from radio and send to client computer
            ThreadPool.QueueUserWorkItem(delegate //w3sz
            {
                int jj = 0;
                SharpDSP2._1.CPX IQ_data;


                //Below for Opus

                if (_encoder != null)
                {
                    _encoder.Dispose();
                }
                _encoder = OpusEncoder.Create(48000, 1, FragLabs.Audio.Codecs.Opus.Application.Voip);
                _encoder.Bitrate = Properties.Settings.Default.voiceBitrate;  // 24576;  //  16384;  //  32768;  // 65536; // 131072;  // 65536; // 32768; // 24576;  // 16384; //was 8192
                _encoder.Bandwidth = Properties.Settings.Default.voiceBandwidth;
                _segmentFrames = 960; //was 960
                _bytesPerSegment = _encoder.FrameByteCount(_segmentFrames);
                mainrec_data_queue = new ConcurrentQueue<byte[]>();
                auxrec_data_queue = new ConcurrentQueue<byte[]>();
                //End section for Opus

                while (ReceiveSend) //w3sz
                {
                    try
                    {
                        for (int ii = 0; ii < outbufferSize;)
                        {
                            if (HPSDRDevice.ProcessedIQ_queue.TryDequeue(out IQ_data))
                            {
                                IQ_data_matrix[ii] = (SharpDSP2._1.CPX)IQ_data;
                                ii++;
                            }
                        }
                        rcvr.DoDSPProcess(ref IQ_data_matrix, ref outbuffer);
                        int SampleSpacing = SampleRate / 48000;
                        for (int sample = 0; sample < outbufferSize; sample += SampleSpacing)
                        {

                            if (jj < AudioLength)
                            {
                                AudioByte[jj++] = (byte)(((int)(scaleOut * outbuffer[sample].real)) & 0xff);
                                AudioByte[jj++] = (byte)(((int)(scaleOut * outbuffer[sample].real)) >> 8);
                            }

                            else
                            {
                                Array.Copy(AudioByte, FinalAudioData, AudioLength);
                                if (chkLocalAudio && Form1.receiveProvider2 != null)
                                {
                                    receiveProvider2.Write(AudioByte, 0, AudioLength);

                                }
                                jj = 0;

                                if ((chkEtherAudio) && (EtherAudioMain || EtherAudioAux))
                                {
                                    if (Properties.Settings.Default.opusComp)
                                    {
                                        try
                                        {
                                            byte[] soundBuffer = new byte[AudioLength + _notEncodedBuffer.Length];
                                            for (int i = 0; i < _notEncodedBuffer.Length; i++)
                                                soundBuffer[i] = _notEncodedBuffer[i];
                                            for (int i = 0; i < AudioLength; i++)
                                                soundBuffer[i + _notEncodedBuffer.Length] = FinalAudioData[i];

                                            int byteCap = _bytesPerSegment;
                                            int segmentCount = (int)Math.Floor((decimal)soundBuffer.Length / byteCap);
                                            int segmentsEnd = segmentCount * byteCap;
                                            int notEncodedCount = soundBuffer.Length - segmentsEnd;
                                            _notEncodedBuffer = new byte[notEncodedCount];
                                            for (int i = 0; i < notEncodedCount; i++)
                                            {
                                                _notEncodedBuffer[i] = soundBuffer[segmentsEnd + i];
                                            }

                                            for (int i = 0; i < segmentCount; i++)
                                            {
                                                byte[] segment = new byte[byteCap];
                                                for (int j = 0; j < segment.Length; j++)
                                                    segment[j] = soundBuffer[(i * byteCap) + j];
                                                int len;
                                                buff = _encoder.Encode(segment, segment.Length, out len);
                                                if (len > 0)
                                                {
                                                    buff2 = new byte[len];
                                                    System.Buffer.BlockCopy(buff, 0, buff2, 0, len);
                                                    if (EtherAudioMain)
                                                    {
                                                        Form1.udpServer6.Send(buff2, len, Form1.localpt6);
                                                    }
                                                    else if (EtherAudioAux)
                                                    {
                                                        Form1.udpServer9.Send(buff2, len, Form1.localpt9); //w3sz 
                                                    }
                                                }
                                            }
                                        }                  
							            catch (Exception ex)
							            {
							                Console.WriteLine(ex.ToString());
							            }
                                    }
                                    else
                                    {
                                        try
                                        {
                                            if (EtherAudioMain)
                                            {
                                                Form1.udpServer6.Send(AudioByte, AudioLength, Form1.localpt6);
                                            }
                                            else if (EtherAudioAux)
                                            {
                                                Form1.udpServer9.Send(AudioByte,AudioLength , Form1.localpt9); //w3sz 
                                            }
                                        }                  
							            catch (Exception ex)
							            {
							                Console.WriteLine(ex.ToString());
							            }

                                    }
                                }
                            }
                        }
                    }                                      
		            catch (Exception ex)
		            {
		                Console.WriteLine(ex.ToString());
		            }

                }

            });



            // start getting data from UI
            ThreadPool.QueueUserWorkItem(delegate //w3sz
            {
                if(chkClient==true)
                { 
                UdpClient udpServer4 = new UdpClient(server4port); //w3sz
                IPEndPoint inEndPoint4 = new IPEndPoint(IPAddress.Parse(w3szendpoint), server4port); //w3sz
                while (closeflag == false) //w3sz
                {
                    try
                    {
                        byte[] buffer4 = udpServer4.Receive(ref inEndPoint4); //w3sz 
                        string sbuffer4 = Encoding.UTF8.GetString(buffer4, 0, buffer4.Length);

                            if (sbuffer4.StartsWith("band"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    BandSelect.Text = sbuffer4;
                                });
                            }
                            else if (sbuffer4.StartsWith("MRON"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    if (sbuffer4 == "0") EtherAudioMain = false;
                                    else EtherAudioMain = true;
                                });
                            }
                            else if (sbuffer4.StartsWith("LRON"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    if (sbuffer4 == "0") EtherAudioAux = false;
                                    else EtherAudioAux = true;
                                });
                            }
                            else if (sbuffer4.StartsWith("mode"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
									 if ((ModeText == "USB") || (ModeText == "LSB") || (ModeText == "FM") || (ModeText == "AM") || (ModeText == "SAM") || (ModeText == "CWU") || (ModeText == "CWL"))
									 {
										Mode.Text = sbuffer4;
										ModeText = Mode.Text;
										ChangeMode();
									 }
                                });
                            }
                            else if (sbuffer4.StartsWith("rvol"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    rcvr.VolumeRight = float.Parse(sbuffer4, System.Globalization.CultureInfo.InvariantCulture);
                                    VolumeTrackBar.Value = (int)(rcvr.VolumeRight * VolumeTrackBar.Maximum);
                                });
                            }
                            else if (sbuffer4.StartsWith("lvol"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    rcvr.VolumeLeft = float.Parse(sbuffer4, System.Globalization.CultureInfo.InvariantCulture);
                                    VolumeTrackBar.Value = (int)(rcvr.VolumeLeft * VolumeTrackBar.Maximum);
                                });
                            }
                            else if (sbuffer4.StartsWith("AGCM"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    rcvr.AGCMaximumGainDB = double.Parse(sbuffer4, System.Globalization.CultureInfo.InvariantCulture);
                                    UpdateMaxGainAndSlider(rcvr.AGCMaximumGainDB);
                                });
                            }
                            else if (sbuffer4.StartsWith("Driv"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    Drive.Text = sbuffer4;
                                    DriveLevel.Value = int.Parse(Drive.Text);
                                // apply square law to drive level control
                                DriveGain = (float)Math.Pow(DriveLevel.Value, 2) / 10000;  // DriveGain is float from 0 to 1.0
                                set_frequency.Focus(); // return focus to the frequency slider
                            });
                            }
                            else if (sbuffer4.StartsWith("freq"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    if ((int.Parse(sbuffer4) > set_frequency.Minimum) & (int.Parse(sbuffer4) < set_frequency.Maximum))
                                    {
                                        set_frequency.Value = int.Parse(sbuffer4);
                                        DisplayFrequency(set_frequency.Value);
                                    }
                                });
                            }


                            else if (sbuffer4.StartsWith("sque"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    Squelch_setting.Text = sbuffer4;
                                    try
                                    {
                                        Squelch_level.Value = int.Parse(Squelch_setting.Text) * 10;
                                    }                                                      
						            catch (Exception ex)
						            {
						                Console.WriteLine(ex.ToString());
						            }
                                });
                            }
                            else if (sbuffer4.StartsWith("sqOn"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    bandscopeSquelch = bool.Parse(sbuffer4);
                                    Bandscope_squelch.Checked = bandscopeSquelch;
                                });
                            }
                            else if (sbuffer4.StartsWith("sqfi"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    filterSquelch = bool.Parse(sbuffer4);
                                    Filter_squelch.Checked = filterSquelch;
                                });
                            }
                            else if (sbuffer4.StartsWith("step"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    stepSize.Text = sbuffer4;
                                });
                            }
                            else if (sbuffer4.StartsWith("AGCS"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    AGCSpeed.Text = sbuffer4;
                                });
                            }
                            else if (sbuffer4.StartsWith("NN10"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    ANF.Checked = bool.Parse(sbuffer4);
                                });
                            }
                            else if (sbuffer4.StartsWith("NR11"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    NR.Checked = bool.Parse(sbuffer4);
                                });
                            }
                            else if (sbuffer4.StartsWith("NB10"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    NB1.Checked = bool.Parse(sbuffer4);
                                });
                            }
                            else if (sbuffer4.StartsWith("NB20"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    NB2.Checked = bool.Parse(sbuffer4);
                                });
                            }
                            else if (sbuffer4.StartsWith("pamp"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    Preamp.Checked = bool.Parse(sbuffer4);
                                });
                            }
                            else if (sbuffer4.StartsWith("bass"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    chkBassCut.Checked = bool.Parse(sbuffer4);
                                });
                            }
                            else if (sbuffer4.StartsWith("vox1"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    chkVOX.Checked = bool.Parse(sbuffer4);
                                });
                            }
                            else if (sbuffer4.StartsWith("clip"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    chkClipper.Checked = bool.Parse(sbuffer4);
                                });
                            }
                            else if (sbuffer4.StartsWith("NGck"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    chkNoiseGate.Checked = bool.Parse(sbuffer4);
                                });
                            }
                            else if (sbuffer4.StartsWith("PrGV"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    ProcessorGain.Value = int.Parse(sbuffer4);
                                    ProcessorGainChange();
                                });
                            }
                            else if (sbuffer4.StartsWith("VOXV"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    VOXLevel.Value = int.Parse(sbuffer4);
                                    VOXLevelChange();
                                });
                            }
                            else if (sbuffer4.StartsWith("McGV"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    MicrophoneGain.Value = int.Parse(sbuffer4); // apply square law to gain control and allow max 20dB of gain 
                                                                                //  MicGain = (float)Math.Pow(MicrophoneGain.Value, 2) / 1000.00f;
                                MicGain = (float)Math.Pow(MicrophoneGain.Value, 2) / 1000.00f;
                                // adjust mic gain level here W3SZ 4-28-2016
                            });
                            }
                            else if (sbuffer4.StartsWith("NGLV"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    NoiseGateLevel.Value = int.Parse(sbuffer4);
                                });
                            }
                            else if (sbuffer4.StartsWith("FiWi"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    labelFilterWidth.Text = sbuffer4;
                                    BandwidthTrackBar.Value = int.Parse(sbuffer4);
                                    BandwidthTrackBar_Scroll(this, EventArgs.Empty);
                                });
                            }
                            else if (sbuffer4.StartsWith("SaRt"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    SampleRate = int.Parse(sbuffer4);
                                    rcvr.SampleRate = SampleRate;
                                    tx.SampleRate = SampleRate;
                                });

                            }
                            else if (sbuffer4.StartsWith("z100"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    Zoom100.Checked = bool.Parse(sbuffer4);
                                });
                            }
                            else if (sbuffer4.StartsWith("z075"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    Zoom75.Checked = bool.Parse(sbuffer4);
                                });
                            }
                            else if (sbuffer4.StartsWith("z050"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    Zoom50.Checked = bool.Parse(sbuffer4);
                                });
                            }
                            else if (sbuffer4.StartsWith("z025"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    Zoom25.Checked = bool.Parse(sbuffer4);
                                });
                            }
                            else if (sbuffer4.StartsWith("ztra"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    Zoom_Track_Bar.Value = int.Parse(sbuffer4);
                                    Zoom_Track_Bar_Scroll(this, EventArgs.Empty);
                                });
                            }
                            else if (sbuffer4.StartsWith("FFTS"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    FFTSize = int.Parse(sbuffer4);
                                    GlobalVar.FFTSize = FFTSize;
                                    Properties.Settings.Default.FFTSize = GlobalVar.FFTSize;
                                    Properties.Settings.Default.Save();
                                    if (OldDSPFFTSize != GlobalVar.FFTSize)
                                    {
                                    //     OnOffButton_Click(this, EventArgs.Empty);
                                    DSPchanged = true;
                                        OnOffButton.BackColor = SystemColors.Control;
                                        OnOffButton.Text = "OFF";
                                        PTT = false;                                // force PTT off
                                    MOX_set = false;                            // force MOX off
                                    previous_OnlyTxOnPTT = OnlyTxOnPTT;         // save this status so we can restore it
                                    OnlyTxOnPTT = true;                         // force always PTT off 
                                    TUN.BackColor = SystemColors.Control;
                                        if (TUN_set)
                                        {
                                            DriveLevel.Value = PreviousDrive;            // restore the previous drive level
                                        DriveLevel_Scroll(this, EventArgs.Empty);   // force drive level update as TUN is on 
                                    }
                                        TUN_set = false;                            // force Tune off
                                    MOX.BackColor = SystemColors.Control;
                                        if (ourDevice != null)
                                        {
                                            ourDevice.Stop();
                                            ourDevice = null;
                                        }
                                        OnlyTxOnPTT = previous_OnlyTxOnPTT;         // restore the previous status 
                                    KK_on = false;                              // set this last so PTT off gets sent
                                    set_frequency.Focus(); // set focus to the frequency slider
                                    MessageBox.Show(String.Format("Client has reset FFT Size to {0} Please close down and restart program.", Convert.ToString(FFTSize)));
                                    }

                                });
                            }
                            else if (sbuffer4.StartsWith("tune"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    TUN_Click(this, EventArgs.Empty);
                                });
                            }
                            else if (sbuffer4.StartsWith("moxo"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                string notmox = sbuffer4.ToString();
                                Console.WriteLine("Received from UI: " + notmox);
                                bool bnotmox = Convert.ToBoolean(notmox);
                                //this is 'reverse' logic to work with Toggle MOX method
                                if (bnotmox == false) MOX_set = false;
                                else if (bnotmox == true) MOX_set = true;
                                Invoke((MethodInvoker)delegate
                                {
                                    MOX_Click(this, EventArgs.Empty);
                                });
                            }
                            else if (sbuffer4.StartsWith("ga50"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    Gain50 = int.Parse(sbuffer4);
                                });
                            }
                            else if (sbuffer4.StartsWith("ga14"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    Gain144 = int.Parse(sbuffer4);
                                });
                            }
                            else if (sbuffer4.StartsWith("ga22"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    Gain222 = int.Parse(sbuffer4);
                                });
                            }
                            else if (sbuffer4.StartsWith("ga43"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    Gain432 = int.Parse(sbuffer4);
                                });
                            }
                            else if (sbuffer4.StartsWith("ga90"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    Gain903 = int.Parse(sbuffer4);
                                });
                            }
                            else if (sbuffer4.StartsWith("g129"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    Gain1296 = int.Parse(sbuffer4);
                                });
                            }
                            else if (sbuffer4.StartsWith("gai2"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    Gain2304 = int.Parse(sbuffer4);
                                });
                            }
                            else if (sbuffer4.StartsWith("gai3"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    Gain3456 = int.Parse(sbuffer4);
                                });
                            }
                            else if (sbuffer4.StartsWith("gai5"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    Gain5760 = int.Parse(sbuffer4);
                                });
                            }
                            else if (sbuffer4.StartsWith("gai1"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    Gain10368 = int.Parse(sbuffer4);
                                });
                            }
                            else if (sbuffer4.StartsWith("ga24"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    Gain24192 = int.Parse(sbuffer4);
                                });
                            }
                            else if (sbuffer4.StartsWith("g160"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    Gain160m = int.Parse(sbuffer4);
                                });
                            }
                            else if (sbuffer4.StartsWith("ga80"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    Gain80m = int.Parse(sbuffer4);
                                });
                            }
                            else if (sbuffer4.StartsWith("ga40"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    Gain40m = int.Parse(sbuffer4);
                                });
                            }
                            else if (sbuffer4.StartsWith("ga30"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    Gain30m = int.Parse(sbuffer4);
                                });
                            }
                            else if (sbuffer4.StartsWith("ga20"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    Gain20m = int.Parse(sbuffer4);
                                });
                            }
                            else if (sbuffer4.StartsWith("ga17"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    Gain17m = int.Parse(sbuffer4);
                                });
                            }
                            else if (sbuffer4.StartsWith("ga15"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    Gain15m = int.Parse(sbuffer4);
                                });
                            }
                            else if (sbuffer4.StartsWith("ga12"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    Gain12m = int.Parse(sbuffer4);
                                });
                            }
                            else if (sbuffer4.StartsWith("ga10"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    Gain10m = int.Parse(sbuffer4);
                                });
                            }
                            else if (sbuffer4.StartsWith("ga06"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    Gain6m = int.Parse(sbuffer4);
                                });
                            }

                            else if (sbuffer4.StartsWith("CWPI"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    CWPitch = int.Parse(sbuffer4);
                                    BandwidthTrackBar_Scroll(this, EventArgs.Empty);
                                    Console.WriteLine("OKOKOKOKOK");
                                });
                            }

                            else if (sbuffer4.StartsWith("splt"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    if (sbuffer4.ToString().Contains("rue"))
                                    {
                                        splitBox.Checked = true;
                                    }
                                    else if (sbuffer4.ToString().Contains("alse"))
                                    {
                                        splitBox.Checked = false;
                                    }
                                });
                            }

                            else if (sbuffer4.StartsWith("freB"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    if ((int.Parse(sbuffer4) > set_frequency.Minimum) & (int.Parse(sbuffer4) < set_frequency.Maximum))
                                    {
                                        Properties.Settings.Default.split_tx_freq = int.Parse(sbuffer4);
                                        Properties.Settings.Default.Save();
                                        DisplayFrequencyB(Properties.Settings.Default.split_tx_freq);
                                    }
                                });
                            }

                            else if (sbuffer4.StartsWith("Opus"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    Properties.Settings.Default.opusComp = Convert.ToBoolean(sbuffer4.ToString());
                                    Properties.Settings.Default.Save();
                             });
                            }


                            else if (sbuffer4.StartsWith("TxOp"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    Properties.Settings.Default.opusTxComp = Convert.ToBoolean(sbuffer4.ToString());
                                    Properties.Settings.Default.Save();
                                });
                            }



                            else if (sbuffer4.StartsWith("MMIC"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    Properties.Settings.Default.mainMicOn = Convert.ToBoolean(sbuffer4.ToString());
                                    Properties.Settings.Default.Save();
                                });
                            }
                            else if (sbuffer4.StartsWith("AMIC"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    Properties.Settings.Default.auxMicOn = Convert.ToBoolean(sbuffer4.ToString());
                                    Properties.Settings.Default.Save();
                                });
                            }




                            else if (sbuffer4.StartsWith("AtoB"))
                            {
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    AtoBBut_Click(this, EventArgs.Empty);
                                });
                            }

                            else if (sbuffer4.StartsWith("BtoA"))
                            {
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    BtoABut_Click(this, EventArgs.Empty);
                                });
                            }

                            else if (sbuffer4.StartsWith("VoBR"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Properties.Settings.Default.voiceBitrate = int.Parse(sbuffer4);
                                if (!digiChk.Checked)
                                {
                                    if (Properties.Settings.Default.voiceCodecIsVoip == true)
                                    {
                                        _encoder.Bitrate = Properties.Settings.Default.voiceBitrate;
                                        _encoder.Bandwidth = Properties.Settings.Default.voiceBandwidth;
                                    }
                                    else if (Properties.Settings.Default.voiceCodecIsVoip == false)
                                    {
                                        _encoder.Bitrate = Properties.Settings.Default.voiceBitrate;
                                        _encoder.Bandwidth = Properties.Settings.Default.voiceBandwidth;
                                    }
                                }
                            }
                            else if (sbuffer4.StartsWith("VoCo"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                if (sbuffer4 == "Voip")
                                {
                                    Properties.Settings.Default.voiceCodecIsVoip = true;
                                    if (!digiChk.Checked)
                                    {
                                        _encoder.Bitrate = Properties.Settings.Default.voiceBitrate;
                                        _encoder.Bandwidth = Properties.Settings.Default.voiceBandwidth;
                                    }
                                }
                                else if(sbuffer4 == "Audio")
                                {
                                    Properties.Settings.Default.voiceCodecIsVoip = false;
                                    if (!digiChk.Checked)
                                    {
                                        _encoder.Bitrate = Properties.Settings.Default.voiceBitrate;
                                        _encoder.Bandwidth = Properties.Settings.Default.voiceBandwidth;
                                    }
                                }
                             }
                            

                            else if (sbuffer4.StartsWith("DaBR"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Properties.Settings.Default.dataBitrate = int.Parse(sbuffer4);
                                if (digiChk.Checked)
                                {
                                    if (Properties.Settings.Default.dataCodecIsVoip ==true)
                                    {
                                        _encoder.Bitrate = Properties.Settings.Default.dataBitrate;
                                        _encoder.Bandwidth = Properties.Settings.Default.dataBandwidth;
                                    }
                                    else if (Properties.Settings.Default.dataCodecIsVoip == false)
                                    {
                                        _encoder.Bitrate = Properties.Settings.Default.dataBitrate;
                                        _encoder.Bandwidth = Properties.Settings.Default.dataBandwidth;
                                    }
                                }
                            }
                            else if (sbuffer4.StartsWith("DaCo"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                if (sbuffer4 == "Voip")
                                {
                                    Properties.Settings.Default.dataCodecIsVoip = true;
                                    if (digiChk.Checked)
                                    {
                                        _encoder.Bitrate = Properties.Settings.Default.dataBitrate;
                                        _encoder.Bandwidth = Properties.Settings.Default.dataBandwidth;
                                    }
                                }
                                else if (sbuffer4 == "Audio")
                                {
                                    Properties.Settings.Default.dataCodecIsVoip = false;
                                    if (digiChk.Checked)
                                    {
                                        _encoder.Bitrate = Properties.Settings.Default.dataBitrate;
                                        _encoder.Bandwidth = Properties.Settings.Default.dataBandwidth;
                                    }
                                }
                            }
                            else if (sbuffer4.StartsWith("DiOn"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: DigiChk is " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    if (sbuffer4 == "False")
                                    {
                                        digiChk.Checked = false;
                                    }
                                    else
                                    {
                                        digiChk.Checked = true;
                                    }
                                });
                            }

                            else if (sbuffer4.StartsWith("VoBW"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Properties.Settings.Default.voiceBandwidth = int.Parse(sbuffer4);
                                if (!digiChk.Checked)
                                {
                                    if (Properties.Settings.Default.voiceCodecIsVoip == true)
                                    {
                                        _encoder.Bitrate = Properties.Settings.Default.voiceBitrate;
                                        _encoder.Bandwidth = Properties.Settings.Default.voiceBandwidth;
                                    }
                                    else if (Properties.Settings.Default.voiceCodecIsVoip == false)
                                    {
                                        _encoder.Bitrate = Properties.Settings.Default.voiceBitrate;
                                        _encoder.Bandwidth = Properties.Settings.Default.voiceBandwidth;
                                    }
                                }
                            }
                            else if (sbuffer4.StartsWith("DaBW"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: " + sbuffer4.ToString());
                                Properties.Settings.Default.dataBandwidth = int.Parse(sbuffer4);
                                if (digiChk.Checked)
                                {
                                    if (Properties.Settings.Default.dataCodecIsVoip == true)
                                    {
                                        _encoder.Bitrate = Properties.Settings.Default.dataBitrate;
                                        _encoder.Bandwidth = Properties.Settings.Default.dataBandwidth;
                                    }
                                    else if (Properties.Settings.Default.dataCodecIsVoip == false)
                                    {
                                        _encoder.Bitrate = Properties.Settings.Default.dataBitrate;
                                        _encoder.Bandwidth = Properties.Settings.Default.dataBandwidth;
                                    }
                                }
                            }

                            else if (sbuffer4.StartsWith("MWBA"))
                            {
                                sbuffer4 = sbuffer4.Remove(0, 4);
                                Console.WriteLine("Received from UI: mwband is " + sbuffer4.ToString());
                                Invoke((MethodInvoker)delegate
                                {
                                    mwband = sbuffer4.ToString();
                                });

                                //        A B C D
                                //  PIN   2 7 8 9
                                // BAND  
                                // 2304   0 1 1 0   00000110  96
                                // 3456   1 1 1 0   10000110  97
                                // 5760   0 0 0 1   00000001  128
                                // 10368  1 0 0 1   10000001  129
                                // 24192  0 1 0 1   00000101  160

                                int dataout;
                                switch (mwband)
                                {
                                    case "2304":
                                        {
                                            dataout = 96;
                                            break;
                                        }
                                    case "3456":
                                        {
                                            dataout = 97;
                                            break;
                                        }
                                    case "5760":
                                        {
                                            dataout = 128;
                                            break;
                                        }
                                    case "10368":
                                        {
                                            dataout = 129;
                                            break;
                                        }
                                    case "24192":
                                        {
                                            dataout = 160;
                                            break;
                                        }
                                    default:
                                        {
                                            dataout = 0;
                                            Console.WriteLine("ERROR: mwband = 0");
                                            break;
                                        }

                                }  //end switch
                                PortControl.Output(LPT_Addr, dataout);

                            }  //end else if for MWBA
                    }  //end try

                    catch (Exception e)
                    {
                        Console.WriteLine("Exception occurred in command receiver");
                        Console.WriteLine(e.ToString());
                    }
                }  //end while
                }
            });  //end threadpool queue workitem delegate


            label1.BackColor = Color.White; //w3sz

            Debug.Indent();   // Indent Debug messages to make them easier to see


            // make the pen for the squelch line on the spectrum display have 'dot' properties
            dotPen.DashStyle = System.Drawing.Drawing2D.DashStyle.Dot;

            // draw 
            pictureBox2.Paint += new PaintEventHandler(pictureBox2_Paint); // SMeter event handler
            pictureBox2.Height = 40;
            pictureBox2.Width = 140;

            // set the size of the various graphics windows so the scaling the dpi does not cause an error
            splitContainer1.Height = 520;
            splitContainer1.Width = 1024;
            splitContainer1.SplitterDistance = 518;
            pictureBoxSpectrum.Height = 256;
            pictureBoxSpectrum.Width = 1024;
            pictureBoxWaterfall.Height = 260;
            pictureBoxWaterfall.Width = 1024;

            VolumeTrackBar.Minimum = 0;
            VolumeTrackBar.Maximum = 100;

        }

        class PortControl // Import dll to project
        {
          //  [DllImport("inpout32.dll", EntryPoint = "Out32")]  //this was for 32 bit
            [DllImport("inpoutx64.dll", EntryPoint = "Out32")]  //this is for 64 bit
            public static extern void Output(int address, int value); // decimal
        }

        private void Form1_Shown(object sender, EventArgs e)
        {

        }

const string formatter = "{0,16}{1,20}";
        // Convert an int argument to a byte array and display it. 
        public static void GetBytesInt32(int argument)
        {
            byte[] byteArray = BitConverter.GetBytes(argument);
            Console.WriteLine(formatter, argument,
                BitConverter.ToString(byteArray));
        }

        //below is to either read DCWindowFile or create it
        void getDCW()
        {
            float[] DCW = new float[iqsize + 1];
            string DCWFile = "";
            switch (Properties.Settings.Default.DCWAtten)
            {
                case 20:
                    DCWFile = "DCWindow20";
                    break;
                case 40:
                    DCWFile = "DCWindow40";
                    break;
                case 60:
                    DCWFile = "DCWindow60";
                    break;
                case 80:
                    DCWFile = "DCWindow80";
                    break;
                case 100:
                    DCWFile = "DCWindow100";
                    break;
                case 120:
                    DCWFile = "DCWindow120";
                    break;
                case 140:
                    DCWFile = "DCWindow140";
                    break;
            }
                    BinaryReader br;
            try
            {
                br = new BinaryReader(new FileStream(DCWFile, FileMode.Open));
                for (int i = 0; i < iqsize; i++)
                {
                    DCW[i] = br.ReadSingle();
                }
                br.Close();
                try
                {
                    rcvr.DCWindow = DCW;
                }                                  
	            catch (Exception ex)
	            {
	                Console.WriteLine(ex.ToString());
	            }
            }
            catch (IOException e)
            {
                Console.WriteLine(e.ToString() + "Readfile failed");
                try
                {
                    rcvr.DCWindow = calcDolphChebyshev(iqsize + 1);
                }
                catch(Exception e1)
                {
                    Console.WriteLine(e1.ToString());
                }
            }

        }

        public static bool ReceiveSend = true;
        

        //below is for calcuating DolphChebyshev filter
        public float[] calcDolphChebyshev(int size)
        {
            float[] window = new float[size];
            int ii, nn;
            double M, n, sum = 0, max = 0;
            double tg = Math.Pow(10, Properties.Settings.Default.DCWAtten / 20);
            double x0 = Math.Cosh((1.0 / (size - 1)) * Acosh(tg));
            M = (size - 1) / 2;
            if (size % 2 == 0) M = M + 0.5;
            for (nn = 0; nn < ((size / 2) + 1); nn++)
            {
                n = nn - M;
                sum = 0;
                for (ii = 1; ii < M; ii++)
                {
                    sum += cheby_poly(size - 1, x0 * Math.Cos(Math.PI * ii / size)) * Math.Cos(2.0 * n * Math.PI * ii / size);
                }
                window[nn] = (float)(tg + 2 * sum);
                window[size - nn - 1] = window[nn];
                if (window[nn] > max) max = window[nn];
            }
            for (nn = 0; nn < size; nn++) window[nn] /= (float)max;

            string DCWFile = "";
            switch (Properties.Settings.Default.DCWAtten)
            {
                case 20:
                    DCWFile = "DCWindow20";
                    break;
                case 40:
                    DCWFile = "DCWindow40";
                    break;
                case 60:
                    DCWFile = "DCWindow60";
                    break;
                case 80:
                    DCWFile = "DCWindow80";
                    break;
                case 100:
                    DCWFile = "DCWindow100";
                    break;
                case 120:
                    DCWFile = "DCWindow120";
                    break;
                case 140:
                    DCWFile = "DCWindow140";
                    break;

            }

            BinaryWriter bw;
            try
            {
                bw = new BinaryWriter(new FileStream(DCWFile, FileMode.Create));
                for (int i = 0; i < window.Length; i++)
                {
                    bw.Write(window[i]);
                }
                bw.Close();
            }
            catch (IOException e)
            {
                Console.WriteLine(e.ToString() + "WriteFile failed");
            }
            return window;
        }

        double cheby_poly(int n, double x)
        {
            double res;
            if (Math.Abs(x) <= 1) res = Math.Cos(n * Math.Acos(x));
            else res = Math.Cosh(n * Acosh(x));
            return res;
        }

        public static double Acosh(double x)
        {
            if (x < 1.0) throw new ArithmeticException("range exception");
            return Math.Log(x + Math.Sqrt(x * x - 1));
        }

        public static void testbytes()
        {
            Console.WriteLine(
                "This example of the BitConverter.GetBytes( int ) " +
                "\nmethod generates the following output.\n");
            Console.WriteLine(formatter, "int", "byte array");
            Console.WriteLine(formatter, "---", "----------");

            // Convert int values and display the results.
            GetBytesInt32(0);
            GetBytesInt32(15);
            GetBytesInt32(-15);
            GetBytesInt32(0x100000);
            GetBytesInt32(-0x100000);
            GetBytesInt32(1000000000);
            GetBytesInt32(-1000000000);
            GetBytesInt32(int.MinValue);
            GetBytesInt32(int.MaxValue);
        }
        // end w3sz test section

        // in-band data (EP6)
        void PowerSpectrumSignal_ps_event(object source, SharpDSP2._1.PowerSpectrumSignal.PSpectrumEvent e)
        {
#if false
            Array.Copy(e.buffer, PowerSpectrumData, rbufSize);
#else
            // G Byrkit, K9TRV: just assign the buffer.  Save the copy time
            PowerSpectrumData = e.buffer;
#endif
        }

        // wideband (full spectrum) data (EP4)
        void OutbandPowerSpectrumSignal_ps_event(object source, SharpDSP2._1.OutbandPowerSpectrumSignal.PSpectrumEvent e)
        {
#if false
            Array.Copy(e.buffer, FullBandwidthPowerSpectrumData, EP4BufSize);
#else
            // G Byrkit, K9TRV: just assign the buffer.  Save the copy time
            FullBandwidthPowerSpectrumData = e.buffer;
#endif
        }

        public void UpdateFormTitle()
        {
            //         this.Text = "HPSDR K.I.S.S. Konsole - " + version + " - " + Your_call;  // text on top of form 
        }

        private void Form1_Load(object sender, EventArgs e)
        {

            this.MaximumSize = new Size(1050, 800);
            bool Result = MoveWindow(this.Handle, this.Left, this.Top, 1050, 800, true);
            UpdateFormTitle();

            this.StartPosition = FormStartPosition.CenterScreen;  // TODO save last position and restore from KKServer.csv

      //      this.Height = 800;
        }
        
        int waterfall_low_threshold = -120;
        public int WaterfallLowThreshold
        {
            get { return waterfall_low_threshold; }
            set { waterfall_low_threshold = value; }
        }

        int waterfall_high_threshold = -40;
        public int WaterfallHighThreshold
        {
            get { return waterfall_high_threshold; }
            set { waterfall_high_threshold = value; }
        }
        
        Point[] bandDraw;
        Point[] WaterfallDraw;
        public byte[] W3SZ_Y_byte; //w3sz
        public byte[] W3SZ_Y_byte_long; //w3sz
        public int xWidth = 1024;  //w3sz
        public int FFTLowLimit;  //w3sz
        public int FFTHiLimit;  //w3sz
        public int FFTSpan;
        public double FFTRadioPerCent;
        public double FFTPerCent;
        public double FFTPixDispRatio;
        public double FreqSpan = 192000;   //starts with sampling rate 192000 and no zoom
        public int FFTPixDispRatioInteger;

        // computed during one ComputeDisplay block, and used in the next.
        // May be set to WaterfallLowThreshold if not doing waterfall AGC.
        // This value, and not waterfall_low_threshold, is used in the calculations in ComputeDisplay
        float waterfallPreviousMinValue = 0.0f;

        float yposhton;

        private void ComputeDisplay(float[] PowerSpectrum)
        {
            int xmax = pictureBoxSpectrum.Width; // max number of pixels across the screen

            int BSHeight = pictureBoxSpectrum.Height;
            int WFHeight = pictureBoxWaterfall.Height;
            double lines = (float)(GridMax - GridMin) / (float)GridStep;
            lines = Math.Ceiling((double)lines); // if lines is a fraction round up to next integer 

            int ymax = BSHeight; // maximum y value

            int yscale = IQScale;  // held on Setup form, sets X axis gain when displaying I and Q signals

            float GainOffset = (PreampOn) ? -20.0f : 0.0f;  // if the preamp is on then remove 20dB from the signal level
            if (Alex)
            {
                GainOffset += (float)AlexAtten;
            }

            xWidth = xmax;  //spectrum and waterfall display width

            float waterfall_minimum = 0.0f;  // reset waterfall_minimum value in nPS array 
            // set to just above black for now for testing but make a user setting and save in KKServer.csv
            int waterfall_low_color_R = 1; int waterfall_low_color_G = 1; int waterfall_low_color_B = 1;
            int R = 0, G = 0, B = 0;
            float range;
            float off_set;
            float overall_percent; // value from 0.0 to 1.0 where 1.0 is high and 0.0 is low.
            float local_percent;

            max_signal = -200;

            // reset maximum value of nPS in the array

            if ((bandDraw == null) || (bandDraw.Length != xWidth))  //  12/20/2016 was IDraw
            {
                // only create these arrays (and populate them!) if not created, or if xWidth changes...

                bandDraw = null;
                WaterfallDraw = null;
                W3SZ_Y_byte = null;
                W3SZ_Y_byte_long = null;
                bandDraw = new Point[xWidth];
                WaterfallDraw = new Point[xWidth];
                W3SZ_Y_byte = new byte[4]; //this is output of spectrum data by W3SZ
                W3SZ_Y_byte_long = new byte[4 * xWidth];

                // populate the arrays.  Only do this when the size (xWidth) changes, to minimize memory allocation
                for (int i = 0; i < xWidth; ++i)
                {
                    bandDraw[i] = new Point();
                    WaterfallDraw[i] = new Point();
                    for (int ij = 0; ij < 4; ++ij)  //w3sz
                    {
                        W3SZ_Y_byte_long[ij + 4 * i] = new byte();
                    }
                }
            }

            // advance to the next waterfall line and count it
                NextWaterfallLine();
                if (!WaterfallAGC)  // if selected then set the low threshold automatically
                    waterfallPreviousMinValue = WaterfallLowThreshold;
            

            byte[] rawData = new byte[xWidth * 4];
            int bytePos = 0;
            int logGain = 85;  // see below as to why this value TODO: allow user to calibrate this value


            for (int xpos = 0; xpos < xWidth; xpos++)
            {
                float SampleMax = 0.0f;

                #region How scaling the bandscope works
                /*
                 * We need to scale the bandscope display depending on the max,min and step values
                 * the uses has selected on the Setup Form.
                 * We know the number of pixels that each grid line represents since this is 
                 * calculated in DrawGrid() as vertical_grid_spacing.
                 * 
                 * Hence we can calculate the number of pixels/dB of vertical resolution from
                 * 
                 *  pixels_per_dB = vertical_grid_spacing/GridStep.
                 *  
                 * We know the signal level at the top of the bandscope since this is held in GridMax.
                 * 
                 * We need to know what each sample represents in terms of dBs.
                 * To do this we feed a known signal into Mercury with the preamp off. 
                 * 
                 * With this signal as input we  determine the maximum signal in the array and note the value.
                 * The signal processing to this point has been:
                 * 
                 *       Po =  (FFT(Pin) * G)  
                 * 
                 * where G = the overall gain from the antenna socket to here
                 *     Pin = the power level at the antenns socket in dBm and 
                 *     Po  = the  level we wish  to display on the bandscope
                 *     FFT = the fast fourier transform of the input signal 
                 *     
                 * We want the bandscope to the scaled in dBm so we take log and multiply by 10 i.e.
                 * 
                 *     10 log Po = 10log (FFT(Pin) * G)
                 *  
                 *  or 10 log Po = 10log (FFT(Pin) + 10Log G 
                 * 
                 * Since the relations between signal input and (FFT(Pin) is linear 
                 * if we know Pin and measure Po we can calculate 10log G.
                 * 
                 * For example, with my Mercury board, 0dBm gives a peak value of 85 (This value will be calibrated in future releases)
                 *
                 *  hence  85 = 0 + 10log G 
                 * 
                 * We can now calculate the dB value of every sample.  We need to map these to the screen.
                 * 
                 * For example, if we assume the top of the screen is 0dBm then we can calculate each samples
                 * pixel position as follows 
                 * 
                 *  yintPS = -(nPS - 85)* pixels_per_db  or -(nPS  - 85) * vertical_grid_spacing/GridStep
                 *  
                 * For values of GridMax other than 0dBm we need need to subtract this value, hence
                 * 
                 *  yintPS = -(nPS - 85 - GridMax) * vertical_grid_spacing / GridStep;
                 *  
                 * We do exactly the same calculations for the wide bandscope - in this case 10log G = 56
                 * 
                 */
                #endregion

                // w3sz this section determines how many bins there are per pixel, and takes the highest value
                // bin in each group of bins associated with a pixel as the value to be displayed for that pixel

                int j;
                float nPS = -60000.0f;
                float nPSA = -60000.0f;
                for (j = 0; j < FFTPixDispRatioInteger; j++)
                {
                    nPSA = PowerSpectrum[j + (int)(FFTLowLimit + xpos * FFTPixDispRatio)] + GainOffset;
                    if (nPSA > nPS)
                    {
                        nPS = nPSA;
                    }
                }

                // Display FullBandwidthPowerSpectrumData on the same grid.
                // FullBandwidthPowerSpectrumData are a block of 4096 contiguous undecimated samples taken at 122.88 Msps.
                // PowerSpectrumData comes from the decimated samples.
                // Decimation results in processing gain, so, while the two spectra
                // have the same scale, when plotting, their offsets differ.
                // Since we have more data points than horizontal pixels in our graph,
                // search for & display the largest signal in each group of four.

                SampleMax = -999.0f;

                float nFBPS = (SampleMax + GainOffset);
                
                float yintPS = -(nPS - logGain - GridMax) * (float)(BSHeight / lines) / GridStep;  // see explanation above 

                // Draw the sample point by adding a point to the respective point lists
                // when we've exited the for loop, use DrawCurve for each that we are drawing
               
                    if (nPS != -60000) //added if 12/20/2016
                    {
                        bandDraw[xpos].X = xpos;
                        bandDraw[xpos].Y = (int)yintPS;
                    }
                    else
                    {
                        if(xpos > 0)
                        {
                            bandDraw[xpos].X = xpos;
                            bandDraw[xpos].Y = bandDraw[xpos-1].Y;
                        }
                    }
                    yposhton = (nPS + 500);  //w3sz host order to network order

                    W3SZ_Y_byte = BitConverter.GetBytes(yposhton);
                    W3SZ_Y_byte_long[4 * xpos] = W3SZ_Y_byte[0];
                    W3SZ_Y_byte_long[1 + 4 * xpos] = W3SZ_Y_byte[1];
                    W3SZ_Y_byte_long[2 + 4 * xpos] = W3SZ_Y_byte[2];
                    W3SZ_Y_byte_long[3 + 4 * xpos] = W3SZ_Y_byte[3];
                

                // this should be moved to one-time processing.  It's not needed to be done for each block of data being displayed.
                
                    // This code is based on the waterfall in PowerSDR 

                    // from the above explanation we know the dBm value of each sample is  nPS - logGain
                    float waterfall_data = nPS - logGain;

                    // select a colour for each pixel depending on its dBm value
                    if (waterfall_data <= waterfallPreviousMinValue)
                    {
                        R = waterfall_low_color_R;
                        G = waterfall_low_color_G;
                        B = waterfall_low_color_B;
                    }
                    else if (waterfall_data >= waterfall_high_threshold)
                    {
                        R = 192;  // looks like a mauve/purple sort of colour 
                        G = 124;
                        B = 255;
                    }
                    else // value is between low and high  
                    {
                        range = waterfall_high_threshold - waterfallPreviousMinValue;
                        off_set = waterfall_data - waterfallPreviousMinValue;
                        overall_percent = off_set / range; // value from 0.0 to 1.0 where 1.0 is high and 0.0 is low.

                        if (overall_percent < (float)2 / 9) // background to blue
                        {
                            local_percent = overall_percent / ((float)2 / 9);
                            R = (int)((1.0 - local_percent) * waterfall_low_color_R);
                            G = (int)((1.0 - local_percent) * waterfall_low_color_G);
                            B = (int)(waterfall_low_color_B + local_percent * (255 - waterfall_low_color_B));
                        }
                        else if (overall_percent < (float)3 / 9) // blue to blue-green
                        {
                            local_percent = (overall_percent - (float)2 / 9) / ((float)1 / 9);
                            R = 0;
                            G = (int)(local_percent * 255);
                            B = 255;
                        }
                        else if (overall_percent < (float)4 / 9) // blue-green to green
                        {
                            local_percent = (overall_percent - (float)3 / 9) / ((float)1 / 9);
                            R = 0;
                            G = 255;
                            B = (int)((1.0 - local_percent) * 255);
                        }
                        else if (overall_percent < (float)5 / 9) // green to red-green
                        {
                            local_percent = (overall_percent - (float)4 / 9) / ((float)1 / 9);
                            R = (int)(local_percent * 255);
                            G = 255;
                            B = 0;
                        }
                        else if (overall_percent < (float)7 / 9) // red-green to red
                        {
                            local_percent = (overall_percent - (float)5 / 9) / ((float)2 / 9);
                            R = 255;
                            G = (int)((1.0 - local_percent) * 255);
                            B = 0;
                        }
                        else if (overall_percent < (float)8 / 9) // red to red-blue
                        {
                            local_percent = (overall_percent - (float)7 / 9) / ((float)1 / 9);
                            R = 255;
                            G = 0;
                            B = (int)(local_percent * 255);
                        }
                        else // red-blue to purple end
                        {
                            local_percent = (overall_percent - (float)8 / 9) / ((float)1 / 9);
                            R = (int)((0.75 + 0.25 * (1.0 - local_percent)) * 255);
                            G = (int)(local_percent * 255 * 0.5);
                            B = 255;
                        }
                    }

                    // Find the lowest signal level in the block of 1024 samples. If the automatic check box
                    // is selected then use this value for waterfall_low_threshold next time through.
                    if (waterfall_minimum > waterfall_data)
                        waterfall_minimum = waterfall_data;

                    rawData[bytePos++] = (byte)B;
                    rawData[bytePos++] = (byte)G;
                    rawData[bytePos++] = (byte)R;
                    rawData[bytePos++] = 255;
                // end if doing waterfall display

                // Find the maximum signal level in the block of 1024 samples for use in the wideband squelch
                if (bandscopeSquelch && xpos > 0 && max_signal < (nPS - logGain)) //TODO: sample 0 is always 85 - why?
                {
                    max_signal = (nPS - logGain);
                }
                
            } // end for each x data point

            if (chkClient == true)
            {
                udpServer2.Send(W3SZ_Y_byte_long, xWidth * 4, localpt2);  //w3sz  
            }
            if (label1.BackColor == Color.Black) label1.BackColor = Color.Red;
            else label1.BackColor = Color.Black;

            // copy the rawData buffer to the Bitmap, the fast way.  This uses Bitmap.LockBits and Marshal.Copy.
            try
            {
                lock (_waterfallLockObj)
                { 
                    Rectangle drawRect = new Rectangle(0, currentWaterfallLine, xWidth, 1);
                    System.Drawing.Imaging.BitmapData rawBmpData = waterfallBitmap.LockBits(drawRect,
                    System.Drawing.Imaging.ImageLockMode.WriteOnly, waterfallBitmap.PixelFormat);
                    System.Runtime.InteropServices.Marshal.Copy(rawData, 0, rawBmpData.Scan0, 4 * xWidth);
                    waterfallBitmap.UnlockBits(rawBmpData);

                    rawData = null;

                    // Now store the waterfall_minumum value, so that it is available for the next packet to use
                    // Provide  some 'history' (smoothing) in the WaterfallAGC i.e. 80% of the previous value and 20% of the current
                    waterfallPreviousMinValue = ((waterfallPreviousMinValue * 8) + (waterfall_minimum * 2)) / 10.0f;
                }
            }
            catch (Exception e)
            {
                Console.WriteLine(e.ToString());
                ClearWaterfall();
            }
        }  //end of ComputeDisplay

        private int currentWaterfallLine = 0;
        private int numWaterfallLines = 0;

        private void ClearWaterfall()
        {
            currentWaterfallLine = 0;
            numWaterfallLines = 0;
        }

        Int32[] blackness = null;

        private void NextWaterfallLine()
        {
            try
            {
                lock (_waterfallLockObj)
                {
                    if (--currentWaterfallLine < 0)
                    {
                        currentWaterfallLine = waterfallBitmap.Height - 1;
                    }
                    if (numWaterfallLines < waterfallBitmap.Height)
                    {
                        ++numWaterfallLines;
                    }
                }
            }
            catch (Exception e)
            {
                Console.WriteLine(e.ToString());
                ClearWaterfall();
            }

            if (numWaterfallLines == 1)
            {
#if true
                try
                {
                    lock (_waterfallLockObj)
                    {
                        // about to make first use!  set all to black!
                        int width = waterfallBitmap.Width;
                        int height = waterfallBitmap.Height;
                        int length = width * height;

                        if (blackness == null)
                        {
                            // first create a row describing "black"
                            blackness = new Int32[length];
                            int idx;
                            for (idx = 0; idx < length; idx++)
                            {
                                blackness[idx] = unchecked((Int32)0xFF000000); // R=0, G=0, B=0, A=255
                            }
                        }

                        // now gain raw access to the bitmap
                        Rectangle drawRect = new Rectangle(0, 0, width, height);//w3sz was 0,0
                        System.Drawing.Imaging.BitmapData rawBmpData = waterfallBitmap.LockBits(drawRect,
                        System.Drawing.Imaging.ImageLockMode.WriteOnly, waterfallBitmap.PixelFormat);

                        System.Runtime.InteropServices.Marshal.Copy(blackness, 0, rawBmpData.Scan0, length);

                        // we're done, no more need for raw access
                        waterfallBitmap.UnlockBits(rawBmpData);
                    }
                }
                catch (Exception e)
                {
                    Console.WriteLine(e.ToString());
                    ClearWaterfall();
                }
#else
                waterfallBitmap = new Bitmap(1024, 256, System.Drawing.Imaging.PixelFormat.Format32bppPArgb);
#endif
            }
        }

        private bool agcHangEnabled = false;
        public bool AGCHangEnabled
        {
            get { return agcHangEnabled; }
            set
            {
                agcHangEnabled = value;
                rcvr.AGCHangEnable = value;
                if (SetupFormValid())
                {
                    Setup_form.AGCHangEnabled = value;
                }
            }
        }

        private Rectangle AGCHang = new Rectangle();
        private bool AGCHangDrag = false;
        private int AGCHang_y_value = 0;

        private Rectangle AGCKnee = new Rectangle();
        private bool AGCKneeDrag = false;
        private int AGCKnee_y_value = 0;

        private static SolidBrush pana_text_brush = new SolidBrush(Color.Khaki);
        private static Font pana_font = new Font("Tahoma", 7F, FontStyle.Regular, GraphicsUnit.Point, ((byte)(0)));

        private int dBToPixel(double dB)
        {
            return (int)((GridMax - dB) * pixels_per_GridStep / GridStep);
        }

        private double PixelToDb(int y)
        {
            double dB = (double)y * GridStep / pixels_per_GridStep;

            return GridMax - dB;
        }

        float cal_offset = 0.0f;

        private void DrawSpectrumGrid(Graphics g)
        {
            #region How DrawGrid works

            //w3sz I modified this when I added zooming
            //I replaced SampleRate with  Convert.ToInt32(FreqSpan).  Freqspan is just FFTPerCent * SampleRate
            //I had to put in traps for when FreqSpan has not yet been defined and is thus zero
            //For very severe zoom, the center frequency starts to shift rightwards.  I don't know if this will be a practical problem.

            /* 
             * We want to display a grid spaced 5/10/20kHz apart horizontally depending on the sampling rate.
             * We also want to display a vertical grid spaced zdB apart. We also want to indicate the 
             * current filter width on the display.
             * 
             * The screen is 1024 pixels wide so filter width =  bandwidth / sampling_rate
             * 
             * To display the grid, first calculate the vertical grid spacing. We want to display from +/-XdB to -YdBm in ZdB steps 
             * which means we need 160/20 = 8 horizontal lines.
             * 
             * We know the frequency at the left hand edge of the bandscope since it will be the tuned frequency less 
             * (sampling rate/2).  This will be displayed at the top of the first vertical grid line. We want the vertical 
             * grid to have spacings of 5/10/20kHz depending on the sampling rate. We also want the frequency markings to
             * be in these increments. So we need to find the first location from the left hand side of the screen that we
             * can divide by 5/10/20 and get no remainder.  We use the % function to do this. 
             * 
             * Since the minimium grid size is 5kHz we are going to have to test in finer increments than this such that
             * frequency/5 = 0 remainder.  So we use 100Hz increments - this gives a nice smooth display when we drag it.
             * 
             * Once we know the first frequency from the left hand edge of the bandscope we can convert this to pixels. 
             * We know the bandscope is 1024 pixels wide and the frequency width = sampling rate. So we can calculate the x 
             * location of the first vertical bar from  
             * 
             *  pixel_indent = Hz offset from LHS * width of bandscope/sample rate
             *  
             * We can now format the frequency text and display the lines. 
             * 
             * 
             */

            #endregion

            // get the height of our panel
            int BSHeight = pictureBoxSpectrum.Height;

            // calculate the number of horizontal lines and the distance between them
            double lines = (float)(GridMax - GridMin) / (float)GridStep;
            lines = Math.Ceiling((double)lines); // if lines is a fraction round up to next integer 
            int x_locate = 0;
            string displayFreq;
            int display_frequency = 0;
            int offset = 15; // how many pixels from the top to start drawing the vertical lines

            int pixel_spacing;
            float pixels_per_100Hz;
            int grid_spacing;

            // determine vertical grid spacing
            int vertical_grid_spacing = (int)(BSHeight / lines);
            pixels_per_GridStep = vertical_grid_spacing;

            // determine horizontal grid spacing

            if (FreqSpan != 0)
            {
                if (FreqSpan >= 129000)
                {
                    grid_spacing = 200;
                    pixel_spacing = Convert.ToInt32(pictureBoxSpectrum.Width * grid_spacing * 100 / FreqSpan);  // how many pixels this spacing represents
                    pixels_per_100Hz = Convert.ToSingle(pictureBoxSpectrum.Width * 100.0f / FreqSpan); // how many pixels per 100Hz across the screen 
                }
                else if (FreqSpan >= 86000)
                {
                    grid_spacing = 150;
                    pixel_spacing = Convert.ToInt32(pictureBoxSpectrum.Width * grid_spacing * 100 / FreqSpan);  // how many pixels this spacing represents
                    pixels_per_100Hz = Convert.ToSingle(pictureBoxSpectrum.Width * 100.0f / FreqSpan); // how many pixels per 100Hz across the screen 
                }
                else if (FreqSpan >= 43000)
                {
                    grid_spacing = 100;
                    pixel_spacing = Convert.ToInt32(pictureBoxSpectrum.Width * grid_spacing * 100 / FreqSpan);  // how many pixels this spacing represents
                    pixels_per_100Hz = Convert.ToSingle(pictureBoxSpectrum.Width * 100.0f / FreqSpan); // how many pixels per 100Hz across the screen 
                }
                else if (FreqSpan >= 17200)
                {
                    grid_spacing = 50;
                    pixel_spacing = Convert.ToInt32(pictureBoxSpectrum.Width * grid_spacing * 100 / FreqSpan);  // how many pixels this spacing represents
                    pixels_per_100Hz = Convert.ToSingle(pictureBoxSpectrum.Width * 100.0f / FreqSpan); // how many pixels per 100Hz across the screen 
                }
                else if (FreqSpan >= 8600)
                {
                    grid_spacing = 20;
                    pixel_spacing = Convert.ToInt32(pictureBoxSpectrum.Width * grid_spacing * 100 / FreqSpan);  // how many pixels this spacing represents
                    pixels_per_100Hz = Convert.ToSingle(pictureBoxSpectrum.Width * 100.0f / FreqSpan); // how many pixels per 100Hz across the screen 
                }
                else
                {
                    grid_spacing = 10;
                    pixel_spacing = Convert.ToInt32(pictureBoxSpectrum.Width * grid_spacing * 100 / FreqSpan);  // how many pixels this spacing represents
                    pixels_per_100Hz = Convert.ToSingle(pictureBoxSpectrum.Width * 100.0f / FreqSpan); // how many pixels per 100Hz across the screen 
                }
            }
            else
            {
                grid_spacing = SampleRate / 960;  // gives 50/100/200 x 100Hz
                pixel_spacing = pictureBoxSpectrum.Width * grid_spacing * 100 / SampleRate;  // how many pixels this spacing represents
                pixels_per_100Hz = pictureBoxSpectrum.Width * 100.0f / SampleRate; // how many pixels per 100Hz across the screen 
            }
            // Draw the background color
                g.FillRectangle(Brushes.DarkGreen, pictureBoxSpectrum.ClientRectangle);

            Font font = new Font(FontFamily.GenericMonospace, 12);
            
                int left_edge_frequency;

                // Show the  filter bandwidth on the screen 
                // The screen is 1024 pixels wide so filter width =  bandwidth / sampling_rate
                if (FreqSpan != 0)
                {
                    FilterLeft = Convert.ToInt32((pictureBoxSpectrum.Width) / 2 + (int)(pictureBoxSpectrum.Width * Math.Min(rcvr.FilterFrequencyLow, rcvr.FilterFrequencyHigh) / FreqSpan));
                    FilterRight = Convert.ToInt32((pictureBoxSpectrum.Width) / 2 + (int)(pictureBoxSpectrum.Width * Math.Max(rcvr.FilterFrequencyLow, rcvr.FilterFrequencyHigh) / FreqSpan));
                }
                else
                {
                    FilterLeft = (pictureBoxSpectrum.Width) / 2 + (int)(pictureBoxSpectrum.Width * Math.Min(rcvr.FilterFrequencyLow, rcvr.FilterFrequencyHigh) / SampleRate);
                    FilterRight = (pictureBoxSpectrum.Width) / 2 + (int)(pictureBoxSpectrum.Width * Math.Max(rcvr.FilterFrequencyLow, rcvr.FilterFrequencyHigh) / SampleRate);

                }
                int Width = FilterRight - FilterLeft;

                // Draw the filter bandwidth in a different colour
                g.FillRectangle(Brushes.SeaGreen, FilterLeft, 0, Width, BSHeight);

                // truncate tuned frequency to 100Hz digits 
                int truncate_freq = (int)(set_frequency.Value / 100d);

                // the frequency at the left edge of the screen will be the tuned frequency - sample rate/(2 x 100Hz)
                if (FreqSpan != 0)
                {
                    left_edge_frequency = Convert.ToInt32(truncate_freq - (FreqSpan / 200d));
                }
                else
                {
                    left_edge_frequency = Convert.ToInt32(truncate_freq - (SampleRate / 200d));
                }
                // we now step in from the left edge frequency until we have a frequency that divides by 
                // the grid spacing with no remainder. This will be the first frequency we display
                for (display_frequency = left_edge_frequency; display_frequency < truncate_freq - 12; display_frequency++)
                {
                    if ((display_frequency % grid_spacing) == 0)
                        break;
                }

                // how many pixels is this from the left edge of the screen
                float indent_pixels = (display_frequency - left_edge_frequency) * pixels_per_100Hz;

                // draw the vertical grid lines with their associated frequency
                for (float i = indent_pixels; i < pictureBoxSpectrum.Width; i += pixels_per_100Hz)
                {
                    if (display_frequency % grid_spacing == 0)  // are we on a grid boundry
                    {
                        // draw the vertical lines
                        g.DrawLine(Pens.DarkGray, i, offset, i, BSHeight); // offset stops the line going through the text

                        displayFreq = String.Format("{0:0.000}", (float)display_frequency / 10000);
                        if (display_frequency < 100000)  // so the decimal point in the frequency aligns with the grid line - perfectionist!
                            x_locate = (int)i - 17;
                        else
                            x_locate = (int)i - 26;
                        // draw the text above each line
                        g.DrawString(displayFreq, font, Brushes.White, x_locate, 0);
                    }
                    // highlight band edge in Red
                    if (display_frequency == set_frequency.Minimum / 100 || display_frequency == set_frequency.Maximum / 100)
                        g.DrawLine(Pens.Red, i, offset, i, BSHeight);

                    display_frequency++; // increment frequency in 100s of Hz 
                }

                // draw a single vertical line in the middle of the screen
                g.DrawLine(Pens.Yellow, pictureBoxSpectrum.Width / 2, offset, pictureBoxSpectrum.Width / 2, BSHeight);

                // Draw the horizontal lines and signal calibration 
                // First draw the horizontal lines so they don't cover the calibration text 
                // Only draw them for the narrow bandscope
                for (int k = 0; k < BSHeight; k += vertical_grid_spacing)
                {
                    g.DrawLine(Pens.DarkGray, 0, k, pictureBoxSpectrum.Width, k);
                }

                // The dBm value for the top of the screen is held in Grid.Max, we don't indicate this but we do the next line
                // This will be Grid.Max - Grid.Step
                int textSpacing = vertical_grid_spacing;
                for (int x = GridMax - GridStep; x > GridMin; x -= GridStep)
                {
                    string level = x.ToString() + "dBm";
                    g.DrawString(level, font, Brushes.White, 0, textSpacing - 9);
                    textSpacing += vertical_grid_spacing;
                }

                // now do the AGC stuff
                AGCKnee_y_value = 0;
                AGCHang_y_value = 0;
                string agc = "";
                double hang = AGCHangLevel;
                double thresh = AGCThreshDB;
                int agc_fixed_gain = (int)AGCFixedGainDB;
                int x1 = 40;
                int x2 = pictureBoxSpectrum.Width - 40;
                int x3 = 50;
                float GainOffset = (PreampOn) ? 0.0f : 20.0f;  // if the preamp is on then remove 20dB from the signal level
                if (Alex)
                {
                    GainOffset += (float)AlexAtten;
                }

                if (rcvr.AGCMode == SharpDSP2._1.AGCType_e.agcOff)
                {
                    cal_offset = 20.0f;
                    AGCKnee_y_value = dBToPixel(-(float)agc_fixed_gain);
                    // Debug.WriteLine("agcknee_y_D:" + agcknee_y_value);
                    agc = "-F";
                }
                else
                {
                    // Warren NR0V reports that this offset change is necessary
                    cal_offset = -63.0f + GainOffset;

                    // do AGC Hang stuff, but only if AGC is on
                    AGCKnee_y_value = dBToPixel((float)thresh + cal_offset);
                    if (agcHangEnabled)
                    {
                        AGCHang_y_value = dBToPixel((float)hang + cal_offset);
                        if (AGCHang_y_value < 0)
                            AGCHang_y_value = 0;
                        else if (AGCHang_y_value >= pictureBoxSpectrum.Height)
                            AGCHang_y_value = pictureBoxSpectrum.Height - 1;

                        AGCHang.Height = 10; AGCHang.Width = 10; AGCHang.X = 50;
                        AGCHang.Y = AGCHang_y_value - (AGCHang.Height / 2);
                        g.FillRectangle(Brushes.Yellow, AGCHang);

                        using (Pen p = new Pen(Color.Yellow))
                        {
                            p.DashStyle = DashStyle.Dot;
                            g.DrawLine(p, x3, AGCHang_y_value, x2, AGCHang_y_value);
                            g.DrawString("-H", pana_font, pana_text_brush, AGCHang.X + AGCHang.Width, AGCHang.Y - AGCHang.Height);
                        }
                    }
                    agc = "-G";
                }

                if (AGCKnee_y_value < 0)
                    AGCKnee_y_value = 0;
                else if (AGCKnee_y_value >= pictureBoxSpectrum.Height)
                    AGCKnee_y_value = pictureBoxSpectrum.Height - 1;

                // do AGC Knee stuff.  IF AGC is off, this draws the 'agc fixed gain' value
                AGCKnee.Height = 10; AGCKnee.Width = 10; AGCKnee.X = 40;
                AGCKnee.Y = AGCKnee_y_value - (AGCKnee.Height / 2);
                g.FillRectangle(Brushes.YellowGreen, AGCKnee);

                using (Pen p = new Pen(Color.YellowGreen))
                {
                    p.DashStyle = DashStyle.Dot;
                    g.DrawLine(p, x1, AGCKnee_y_value, x2, AGCKnee_y_value);
                    g.DrawString(agc, pana_font, pana_text_brush, AGCKnee.X + AGCKnee.Width, AGCKnee.Y - AGCKnee.Height);
                }
         

          
            // draw cross if required, only used on narrow bandscope at the moment. 
            if (ShowCross)
            {
                g.DrawLine(Pens.Yellow, MousePositionX, 0, MousePositionX, BSHeight); // draw cross vertical line
                if (MousePositionY < BSHeight)  // with both displays check we are within the screen boundries
                    g.DrawLine(Pens.Yellow, 0, MousePositionY, pictureBoxSpectrum.Width, MousePositionY); // draw cross horizontal line
            }
        }

        private void RefreshSpectrumScope(Graphics g)
        {
            if (bandDraw != null) g.DrawCurve(WhitePen, bandDraw);

            // set the range of the squelch contol to the max and min bandscope values x10 to give high resolution
            Squelch_level.Minimum = GridMin * 10; Squelch_level.Maximum = GridMax * 10;

            if (filterSquelch || bandscopeSquelch)
            {
                squelch_y_value = (int)(-((Squelch_level.Value / 10) - (GridMax)) * pixels_per_GridStep / GridStep);

                // draw dotted line to represent squelch threshold
                g.DrawLine(dotPen, 0, squelch_y_value, pictureBoxSpectrum.Width - 1, squelch_y_value);

                // draw the little rectangle at the left side of the squelch threshold line
                squelch.Height = 10; squelch.Width = 10; squelch.X = 0;
                squelch.Y = squelch_y_value - squelch.Height / 2; // make it a square
                g.FillRectangle(Brushes.DarkOrange, squelch);   // fill the rectangle with colour
            }
        }


        // format changed to PArgb (rgb values pre-multiplied by A.  Since A is 255 (max), regular RGB counts as 'pre-multiplied'.
        private Bitmap waterfallBitmap = new Bitmap(1024, 256, System.Drawing.Imaging.PixelFormat.Format32bppPArgb);

        private void RefreshWaterfall(Graphics g)
        {
            
                try
            {
                lock (_waterfallLockObj)
                {
                    int height = pictureBoxWaterfall.Height;

                    // display the waterfall bitmap, if we have one
                    // display it in two parts: from the currentWaterfallLine to the end of the array,
                    // 
                    if (waterfallBitmap != null)
                    {
                        int firstLine = currentWaterfallLine;
                        int firstSegmentLength = waterfallBitmap.Height - currentWaterfallLine;
                        Rectangle destRect = new Rectangle(0, 0, 1024, firstSegmentLength);

                        // Create rectangle for source image.  This is the first part, from currentWaterfallLine to the end of the bitmap
                        Rectangle srcRect = new Rectangle(0, currentWaterfallLine, 1024, firstSegmentLength);
                        GraphicsUnit units = GraphicsUnit.Pixel;

                        // Draw image to screen, part 1.
                        g.DrawImage(waterfallBitmap, destRect, srcRect, units);
                        // draw rest of the image (which may be blank/black) which is the FIRST PART of the bitmap, to AFTER where
                        // the first part was drawn

                        destRect.Y = firstSegmentLength;
                        destRect.Height = currentWaterfallLine;
                        srcRect.Y = 0;
                        srcRect.Height = currentWaterfallLine;
                        g.DrawImage(waterfallBitmap, destRect, srcRect, units);
                    }
                }
            }
            catch (Exception e)
            {
                Console.WriteLine(e.ToString());
                ClearWaterfall();
            }
        }

        // this is called when we exit the program - terminate the Ethernet connection and save the 
        // current state of the Radio in the KKServer.csv file.
        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            if(MOX_set || TUN_set || PTT)
            {
                MOX_set = false;
                TUN_set = false;
                PTT = false;
            }

            if (ourDevice != null)
            {
                ourDevice.Stop();
                ourDevice.Close();
            }
            StopAudio();

            // save the last frequency used 
            switch (BandSelect.Text)
            {
                case "160m": set_frequency_160 = set_frequency.Value; break;
                case "80m": set_frequency_80 = set_frequency.Value; break;
                case "40m": set_frequency_40 = set_frequency.Value; break;
                case "30m": set_frequency_30 = set_frequency.Value; break;
                case "20m": set_frequency_20 = set_frequency.Value; break;
                case "17m": set_frequency_17 = set_frequency.Value; break;
                case "15m": set_frequency_15 = set_frequency.Value; break;
                case "12m": set_frequency_12 = set_frequency.Value; break;
                case "10m": set_frequency_10 = set_frequency.Value; break;
                case "6m": set_frequency_6 = set_frequency.Value; break;
                case "GC": set_frequency_GC = set_frequency.Value; break;
            }
            // save program settings in KKServer.csv file
            WriteKKCSV(KKCSVName);
     //       if (chkClient == true)
            {
                udpServer2.Close();
            }
            closeflag = true;
            
            Environment.Exit(0);
        }

        private void SetSMeterGraph(double value)
        {
            // Calibrate SMeter. S9 = -73dBm = 70 on scale hence add 140
            int sValue = (int)value + 140;
            if (sValue > 140)       // check the SMeter value is within the range we can display so we don't damage it
                sValue = 140;
            else if (sValue < 0)
                sValue = 0;

            SMeterValue = (int)sValue;
            pictureBox2.Invalidate();
        }

        private string SMeterText(double avgValue, double instantaneousValue)
        {
            if (PreampOn)  // Compensate for 20dB of Preamp gain
            {
                avgValue -= 20;
                instantaneousValue -= 20;
            }
            if (Alex)
            {
                avgValue += AlexAtten;
                instantaneousValue += AlexAtten;
            }

            // Calibrate readings. With 1kHz bandwidth noise floor is -115dBm/-135dBm  preamp on/off
            // Raw values are -45 with preamp off so need to add -70
            avgValue += -70.0f;
            instantaneousValue += -70.0f;
            // set the max signal level for use by the Squelch control
            filter_max_signal = (int)avgValue;

            // set the bargraph (meter)
            SetSMeterGraph(instantaneousValue);

            // could also compensate with a 'global correction' factor
            return String.Format(nfi, "SMeter: Avg {0:F1} : Inst {1:F1}", avgValue, instantaneousValue);
        }

        private string SMeterText(SharpDSP2._1.SignalMeter s)
        {
            return SMeterText(s.AvgValue, s.InstValue);
        }

        private void SetSMeterText(string text)
        {
            labelSMeter.Text = text;
        }

        void DrawSMeterBackground(Graphics sg)
        {
            sg.FillRectangle(Brushes.Black, 0, 0, pictureBox2.Width, pictureBox2.Height);
            float MeterHeigth = pictureBox2.Height - 3;
            float MeterWidth = pictureBox2.Width - 5;
            pictureBox2.BackColor = Color.Black;
            Brush whiteBrush = Brushes.White;
            Brush redBrush = Brushes.Red;
            Pen whitePen = new Pen(Color.White, 2);
            Pen narrowWhitePen = new Pen(Color.White, 1);
            Pen redPen = new Pen(Color.Red, 2);
            Pen narrowRedPen = new Pen(Color.Red, 1);
            float MarkerSpacing = 14; // Meter.Width / 10.0f;
            string marker = null;

            // Draw the white items
            for (float x = 5, y = 12, z = 1; x < pictureBox2.Width / 2; x += MarkerSpacing, y += MarkerSpacing, z += 2)
            {
                marker = z.ToString();
                sg.DrawLine(narrowWhitePen, x, MeterHeigth, x, 0.9f * MeterHeigth); //small ticks
                sg.DrawLine(whitePen, y, MeterHeigth, y, 0.8f * MeterHeigth); //big ticks
                sg.DrawString(marker, meterFont, whiteBrush, y - 5, 0.45f * MeterHeigth);
            }

            // Draw the red items
            MarkerSpacing = 22; // Meter.Width / 6.5f;  // wider spacing for these itesm 

            for (float x = 67 + MarkerSpacing / 2, y = 67 + MarkerSpacing, z = 0; x < pictureBox2.Width; x += MarkerSpacing, y += MarkerSpacing, z++)
            {
                if (z == 0) marker = "+20";
                else if (z == 1) marker = "+40";
                else if (z == 2) marker = "+60";
                sg.DrawLine(narrowRedPen, x, MeterHeigth, x, 0.9f * MeterHeigth); //small ticks
                sg.DrawLine(redPen, y, MeterHeigth, y, 0.8f * MeterHeigth); //big ticks
                sg.DrawString(marker, meterFont, redBrush, y - 10, 0.45f * MeterHeigth);
            }

            // Draw horizontal lines to finish off
            sg.DrawLine(whitePen, 0, MeterHeigth, MeterWidth / 2, MeterHeigth);
            sg.DrawLine(redPen, 1 + MeterWidth / 2, MeterHeigth, MeterWidth - 1, MeterHeigth);
        }

        Image SMeterBackgroundImage;

        // Draw the SMeter and display the signal level
        void pictureBox2_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
        {
            Graphics g = e.Graphics;

            // We only need to draw the SMeter backrough once, then use it as an image 
            // saves CPU cycles

            if (SMeterBackgroundImage == null)
            {
                SMeterBackgroundImage = new Bitmap(pictureBox2.Width, pictureBox2.Height);
                Graphics sg = Graphics.FromImage(SMeterBackgroundImage);

                DrawSMeterBackground(sg);

                pictureBox2.BackgroundImage = SMeterBackgroundImage;
            }
            else
            {
            }

            // now draw SMeter value as a vertical bar
            // scale below S9 is 7 pixels per marker and 11 above so need to scale accordingly
            Pen yellowPen = new Pen(Color.Yellow, 2);
            SMeterValue = (int)((float)SMeterValue * 1.15f) - 10;
            if (SMeterValue > 68)  // scale changes at the S9 level
                SMeterValue = (int)((float)SMeterValue * 0.98f);
            g.DrawLine(yellowPen, SMeterValue, 0, SMeterValue, pictureBox2.Height);
        }

        // Slider control sets transceiver center frequency
        private void set_frequency_Scroll_1(object sender, EventArgs e)
        {
            // when the frequency slider has focus the mouse scroll wheel
            // can be used to increment the frequency by +/- step_size.

            // the scroll bar can also be moved with the arrow and Up/Down keys 
            // make the scroll bar large and small step sizes a function of the step size 
            set_frequency.LargeChange = ((step_size + 1) * 100) - step_size;
            set_frequency.SmallChange = (step_size + 1);

            // check we are still within the band and update the tune and display frequencies
            Frequency_change();

            UpdateGraphs();
        }

        // select the step size that the mouse scroll wheel does
        private void stepSize_SelectedIndexChanged(object sender, EventArgs e)
        {
            // since we have alreade taken a step use (step size - 1)
            // TODO: this works OK for the mousewheel on my mouse but check this is universal
            switch (stepSize.Text)
            {
                case "1Hz": step_size = 0; break;
                case "10Hz": step_size = 9; break;
                case "100Hz": step_size = 99; break;
                case "1kHz": step_size = 999; break;
            }
            set_frequency.Focus(); // return focus to the frequency slider
        }

        public void UpdateBandGain()
        {
            switch (BandSelect.Text)
            {
                case "160m":
                    BandGain = Gain160m; // set the band gain
                    break;
                case "80m":
                    BandGain = Gain80m; // set the band gain 
                    break;
                case "40m":
                    BandGain = Gain40m; // set the band gain 
                    break;
                case "30m":
                    BandGain = Gain30m; // set the band gain 
                    break;
                case "20m":
                    BandGain = Gain20m; // set the band gain 
                    break;
                case "17m":
                    BandGain = Gain17m; // set the band gain 
                    break;
                case "15m":
                    BandGain = Gain15m; // set the band gain 
                    break;
                case "12m":
                    BandGain = Gain12m; // set the band gain 
                    break;
                case "10m":
                    switch (VHFBand)
                    {
                        case "50":
                            BandGain = Gain50;
                            break;
                        case "144":
                            BandGain = Gain144;
                            break;
                        case "222":
                            BandGain = Gain222;
                            break;
                        case "432":
                            BandGain = Gain432;
                            break;
                        case "903":
                            BandGain = Gain903;
                            break;
                        case "1296":
                            BandGain = Gain1296;
                            break;
                        case "2304":
                            BandGain = Gain2304;
                            break;
                        case "3456":
                            BandGain = Gain3456;
                            break;
                        case "5760":
                            BandGain = Gain5760;
                            break;
                        case "10368":
                            BandGain = Gain10368;
                            break;
                        case "24192":
                            BandGain = Gain24192;
                            break;
                        case "HF":
                            BandGain = Gain10m;
                            break;
                    }
                    break;
                case "6m":
                    BandGain = Gain6m; // set the band gain 
                    break;
            }
        }

        public OpBand CurrentBand;

        public void BandSelect_SelectedIndexChanged(object sender, EventArgs e)
        {
            // when changing bands store the last frequency used on that band so we can return to it later
            switch (BandText)
            {
                case "160m": set_frequency_160 = set_frequency.Value; break;
                case "80m": set_frequency_80 = set_frequency.Value; break;
                case "40m": set_frequency_40 = set_frequency.Value; break;
                case "30m": set_frequency_30 = set_frequency.Value; break;
                case "20m": set_frequency_20 = set_frequency.Value; break;
                case "17m": set_frequency_17 = set_frequency.Value; break;
                case "15m": set_frequency_15 = set_frequency.Value; break;
                case "12m": set_frequency_12 = set_frequency.Value; break;
                case "10m": set_frequency_10 = set_frequency.Value; break;
                case "6m": set_frequency_6 = set_frequency.Value; break;
                case "GC": set_frequency_GC = set_frequency.Value; break;
            }

            // set the band edges depending on the band selected. Restore the last used frequency from KKServer.csv
            switch (BandSelect.Text)
            {
                case "160m":
                    CurrentBand = OpBand.M160;
                    set_frequency.Minimum = 1800000;  // low band limit
                    set_frequency.Maximum = 2000000;  // high band limit
                        try
                    { 
                        set_frequency.Value = set_frequency_160;
                    }
                    catch
                    {
                        set_frequency.Value = set_frequency.Minimum;
                    }
                    if (Mode.Text == "USB")
                    {
                        Mode.Text = "LSB";                  // select the appropriate sideband
                        ChangeMode();  // reset the Bandwidth slider
                    }
                    Preamp.Checked = Preamp_160;
                    BandGain = Gain160m; // set the band gain 
                    break;

                case "80m":
                    CurrentBand = OpBand.M80;
                    set_frequency.Minimum = 3500000;
                    set_frequency.Maximum = 4000000;
                        try
                    { 
                        set_frequency.Value = set_frequency_80;
                    }
                    catch
                    {
                        set_frequency.Value = set_frequency.Minimum;
                    }
                    if (Mode.Text == "USB")
                    {
                        Mode.Text = "LSB";                  // select the appropriate sideband
                        ChangeMode();  // reset the Bandwidth slider
                    }
                    Preamp.Checked = Preamp_80;
                    BandGain = Gain80m;
                    break;

                case "40m":
                    CurrentBand = OpBand.M40;
                    set_frequency.Minimum = 7000000;
                    set_frequency.Maximum = 7300000;
                        try
                    { 
                        set_frequency.Value = set_frequency_40;
                    }
                    catch
                    {
                        set_frequency.Value = set_frequency.Minimum;
                    }
                    if (Mode.Text == "USB")
                    {
                        Mode.Text = "LSB";                  // select the appropriate sideband
                        ChangeMode();  // reset the Bandwidth slider
                    }
                    Preamp.Checked = Preamp_40;
                    BandGain = Gain40m;
                    break;

                case "30m":
                    CurrentBand = OpBand.M30;
                    set_frequency.Minimum = 10100000;
                    set_frequency.Maximum = 10150000;
                        try
                    { 
                        set_frequency.Value = set_frequency_30;
                    }
                    catch
                    {
                        set_frequency.Value = set_frequency.Minimum;
                    }
                    if (Mode.Text == "LSB")
                    {
                        Mode.Text = "USB";                  // select the appropriate sideband
                        ChangeMode();  // reset the Bandwidth slider
                    }
                    Preamp.Checked = Preamp_30;
                    BandGain = Gain30m;
                    break;

                case "20m":
                    CurrentBand = OpBand.M20;
                    set_frequency.Minimum = 14000000;
                    set_frequency.Maximum = 14350000;
                        try
                    { 
                        set_frequency.Value = set_frequency_20;
                    }
                    catch
                    {
                        set_frequency.Value = set_frequency.Minimum;
                    }
                    if (Mode.Text == "LSB")
                    {
                        Mode.Text = "USB";                  // select the appropriate sideband
                        ChangeMode();  // reset the Bandwidth slider
                    }
                    Preamp.Checked = Preamp_20;
                    BandGain = Gain20m;
                    break;

                case "17m":
                    CurrentBand = OpBand.M17;
                    set_frequency.Minimum = 18068000;
                    set_frequency.Maximum = 18168000;
                        try
                    { 
                        set_frequency.Value = set_frequency_17;
                    }
                    catch
                    {
                        set_frequency.Value = set_frequency.Minimum;
                    }
                    if (Mode.Text == "LSB")
                    {
                        Mode.Text = "USB";                  // select the appropriate sideband
                        ChangeMode();  // reset the Bandwidth slider
                    }
                    Preamp.Checked = Preamp_17;
                    BandGain = Gain17m;
                    break;

                case "15m":
                    CurrentBand = OpBand.M15;
                    set_frequency.Minimum = 21000000;
                    set_frequency.Maximum = 21450000;
                        try
                    { 
                        set_frequency.Value = set_frequency_15;
                    }
                    catch
                    {
                        set_frequency.Value = set_frequency.Minimum;
                    }
                    if (Mode.Text == "LSB")
                    {
                        Mode.Text = "USB";                  // select the appropriate sideband
                        ChangeMode();  // reset the Bandwidth slider
                    }
                    Preamp.Checked = Preamp_15;
                    BandGain = Gain15m;
                    break;

                case "12m":
                    CurrentBand = OpBand.M12;
                    set_frequency.Minimum = 24880000;
                    set_frequency.Maximum = 24980000;
                        try
                    { 
                        set_frequency.Value = set_frequency_12;
                    }
                    catch
                    {
                        set_frequency.Value = set_frequency.Minimum;
                    }
                    if (Mode.Text == "LSB")
                    {
                        Mode.Text = "USB";                  // select the appropriate sideband
                        ChangeMode();  // reset the Bandwidth slider
                    }
                    Preamp.Checked = Preamp_12;
                    BandGain = Gain12m;
                    break;

                case "10m":
                    CurrentBand = OpBand.M10;
                    set_frequency.Minimum = 26000000;
                    set_frequency.Maximum = 32000000;
                        try
                    { 
                        set_frequency.Value = set_frequency_10;
                    }
                    catch
                    {
                        set_frequency.Value = set_frequency.Minimum;
                    }
                    if (Mode.Text == "LSB")
                    {
                        Mode.Text = "USB";                  // select the appropriate sideband
                        ChangeMode();  // reset the Bandwidth slider
                    }
                    Preamp.Checked = Preamp_10;
                    BandGain = Gain10m;
                    break;

                case "6m":
                    CurrentBand = OpBand.M6;
                    set_frequency.Minimum = 50000000;
                    set_frequency.Maximum = 55000000;
                        try
                    { 
                        set_frequency.Value = set_frequency_6;
                    }
                    catch
                    {
                        set_frequency.Value = set_frequency.Minimum;
                    }
                    if (Mode.Text == "LSB")
                    {
                        Mode.Text = "USB";                  // select the appropriate sideband
                        ChangeMode();  // reset the Bandwidth slider
                    }
                    Preamp.Checked = Preamp_6;
                    BandGain = Gain6m;
                    break;

                case "GC":
                    CurrentBand = OpBand.GC;
                    set_frequency.Minimum = 0;
                    set_frequency.Maximum = 55000000;
                        try
                    { 
                        set_frequency.Value = set_frequency_GC;
                    }
                    catch
                    {
                        set_frequency.Value = 10000000;
                    }
                    Preamp.Checked = Preamp_GC;
                    BandGain = 0;
                    break;

                default:
                    CurrentBand = OpBand.M20;
                    set_frequency.Minimum = 14000000;
                    set_frequency.Maximum = 14350000;
                    break;
            }

            BandText = BandSelect.Text; // save the band we are currently on as we leave 
            Frequency_change(); // update frequency and display
            set_frequency.Focus(); // return focus to the frequency slider
        }

        private void Mode_SelectedIndexChanged(object sender, EventArgs e)
        {
            ChangeMode(); // select filter setting based on mode
            set_frequency.Focus(); // return focus to the frequency slider
        }

        private void ChangeMode() // ChangeMode() is called by Mode/BandSelect_SelectedIndexChanged()
        {
            ModeText = Mode.Text;
            if ((ModeText == "USB") || (ModeText == "LSB") || (ModeText == "FM") || (ModeText == "AM"))
            {
                voice = true;
            }
            else if((ModeText == "CWU") || (ModeText == "CWL"))
            { voice = false; }
			else {return;}



            if (rcvr.WindowType != SharpDSP2._1.WindowType_e.KAISER_WINDOW)
            {
                BandwidthCWvalue = BandwidthTrackBar.Value;
            }
            else if (rcvr.WindowType == SharpDSP2._1.WindowType_e.KAISER_WINDOW)
            {
                BandwidthCWvalue = BandwidthTrackBar.Value / 2;
            }

            switch (Mode.Text)
            {
                case "AM":  // this is 'regular' AM, not Synchronous AM (SharpDSP2._1.DSPMode_e.SAM)
                    CurrentMode = OperMode.AM;
                    state.DSPMode = SharpDSP2._1.DSPMode_e.AM;
                    txstate.DSPMode = SharpDSPTx2._1.DSPMode_e.AM;
                    BandwidthTrackBar.Minimum = 3000;
                    BandwidthTrackBar.Maximum = 12000;
                    rcvr.FilterFrequencyHigh = BandwidthTrackBar.Value;
                    rcvr.FilterFrequencyLow = -BandwidthTrackBar.Value;
                    tx.FilterFrequencyHigh = BandwidthTrackBar.Value;
                    tx.FilterFrequencyLow = -BandwidthTrackBar.Value;
                    break;

                case "FM":
                    CurrentMode = OperMode.FM;
                    rcvr.SetFMMode();
                    state.DSPMode = SharpDSP2._1.DSPMode_e.FMN;
                    txstate.DSPMode = SharpDSPTx2._1.DSPMode_e.FMN;
                    BandwidthTrackBar.Minimum = 3000;
                    BandwidthTrackBar.Maximum = 12000;
                    rcvr.FilterFrequencyHigh = BandwidthTrackBar.Value;
                    rcvr.FilterFrequencyLow = -BandwidthTrackBar.Value;
                    tx.FilterFrequencyHigh = BandwidthTrackBar.Value;
                    tx.FilterFrequencyLow = -BandwidthTrackBar.Value;
                    break;

                case "SAM":
                    CurrentMode = OperMode.SAM;
                    rcvr.SetSAMMode();
                    state.DSPMode = SharpDSP2._1.DSPMode_e.SAM;
                    txstate.DSPMode = SharpDSPTx2._1.DSPMode_e.SAM;
                    BandwidthTrackBar.Minimum = 3000;
                    BandwidthTrackBar.Maximum = 12000;
                    rcvr.FilterFrequencyHigh = BandwidthTrackBar.Value;
                    rcvr.FilterFrequencyLow = -BandwidthTrackBar.Value;
                    tx.FilterFrequencyHigh = BandwidthTrackBar.Value;
                    tx.FilterFrequencyLow = -BandwidthTrackBar.Value;
                    break;

                case "USB":
                    CurrentMode = OperMode.USB;
                    state.DSPMode = SharpDSP2._1.DSPMode_e.USB;
                    txstate.DSPMode = SharpDSPTx2._1.DSPMode_e.USB;
                    BandwidthTrackBar.Minimum = 1200;
                    BandwidthTrackBar.Maximum = 6000;
                    rcvr.FilterFrequencyHigh = BandwidthTrackBar.Value;
                    rcvr.FilterFrequencyLow = 200.0f;
                    if (Setup_form != null)
                    {
                        tx.FilterFrequencyHigh = Properties.Settings.Default.TxBandWidth;
                    }
                    else
                    {
                        tx.FilterFrequencyHigh = BandwidthTrackBar.Value;
                    }
                    tx.FilterFrequencyLow = 200.0f;
                    break;

                case "LSB":
                    CurrentMode = OperMode.LSB;
                    state.DSPMode = SharpDSP2._1.DSPMode_e.LSB;
                    txstate.DSPMode = SharpDSPTx2._1.DSPMode_e.LSB;
                    BandwidthTrackBar.Minimum = 1200;
                    BandwidthTrackBar.Maximum = 6000;
                    rcvr.FilterFrequencyHigh = -BandwidthTrackBar.Value;
                    rcvr.FilterFrequencyLow = -200.0f;
                    if (Setup_form != null)
                    {
                        tx.FilterFrequencyHigh = -Properties.Settings.Default.TxBandWidth;
                    }
                    else
                    {
                        tx.FilterFrequencyHigh = -BandwidthTrackBar.Value;
                    }
                    tx.FilterFrequencyLow = -200.0f;
                    break;

                case "CWL":
                    CurrentMode = OperMode.CWL;
                    state.DSPMode = SharpDSP2._1.DSPMode_e.CWL;
                    txstate.DSPMode = SharpDSPTx2._1.DSPMode_e.CWL;
                    BandwidthTrackBar.Minimum = 25;
                    BandwidthTrackBar.Maximum = 1100;
                    rcvr.FilterFrequencyHigh = -(CWPitch + BandwidthCWvalue / 2);
                    rcvr.FilterFrequencyLow = -(CWPitch - BandwidthCWvalue / 2);
                    tx.FilterFrequencyHigh = -(CWPitch + BandwidthTrackBar.Value / 2);
                    tx.FilterFrequencyLow = -(CWPitch - BandwidthTrackBar.Value / 2);
                    break;

                case "CWU":
                    CurrentMode = OperMode.CWU;
                    state.DSPMode = SharpDSP2._1.DSPMode_e.CWU;
                    txstate.DSPMode = SharpDSPTx2._1.DSPMode_e.CWU;
                    BandwidthTrackBar.Minimum = 25;
                    BandwidthTrackBar.Maximum = 1100;
                    rcvr.FilterFrequencyHigh = (CWPitch + BandwidthCWvalue / 2);
                    rcvr.FilterFrequencyLow = (CWPitch - BandwidthCWvalue / 2);
                    tx.FilterFrequencyHigh = (CWPitch + BandwidthTrackBar.Value / 2);
                    tx.FilterFrequencyLow = (CWPitch - BandwidthTrackBar.Value / 2);
                    break;
            }

            labelFilterWidth.Text = BandwidthTrackBar.Value.ToString();
            AdjustFilterByMode();
        }

        // set the volume based on the VolumeTrackBar value
        public void SetVolume(object sender, EventArgs e)
        {
            // set the Receiver volume level
            setVolumeSquelched();
            set_frequency.Focus(); // return focus to the frequency slider
        }

        public void setVolumeSquelched()
        {
            // if squelch it turned on, AND 'squelched' is true, OR (MOX is set OR PTT set ) AND not full Duplex then mute audio
            if ((squelchOn && squelched) || ((MOX_set || PTT) && !Duplex))
            {
                rcvr.VolumeLeft = 0;  // send to SharpSDP
                rcvr.VolumeRight = 0;
            }
            else
            {
                // otherwise set to volume trackbar level
                // convert the track bar value to a float in the range 0 to 1, since that's what SharpDSP needs
                float v = (float)VolumeTrackBar.Value / (float)VolumeTrackBar.Maximum;
                rcvr.VolumeLeft = v;  // send to SharpSDP
                rcvr.VolumeRight = v;
           }
        }


        private void SafeUpDownValueChange(decimal newValue, NumericUpDown control)
        {
            if (newValue < control.Minimum)
                newValue = control.Minimum;
            if (newValue > control.Maximum)
                newValue = control.Maximum;

            control.Value = newValue;
        }

        private void SafeTrackBarValueChange(int newValue, TrackBar control)
        {
            if (newValue < control.Minimum)
                newValue = control.Minimum;
            if (newValue > control.Maximum)
                newValue = control.Maximum;

            control.Value = newValue;
        }

        // set the AGC Gain  based on the AGC TrackBar value
        bool noAGCTrackBarUpdate = false;
        private void AGCTrackBar_Scroll(object sender, EventArgs e)
        {
            if (noAGCTrackBarUpdate)
                return;

            if (rcvr.AGCMode == SharpDSP2._1.AGCType_e.agcOff)
            {
                rcvr.AGCFixedGainDB = AGCTrackBar.Value;    // ditto
                if (SetupFormValid())
                {
                    Setup_form.ChangeFixedGain(rcvr.AGCFixedGainDB);
                }
            }
            else
            {
                rcvr.AGCMaximumGainDB = AGCTrackBar.Value;  // TODO: make this value user adjustable and store/restore on KK.cvs
                if (SetupFormValid())
                {
                    Setup_form.ChangeMaxGain(rcvr.AGCMaximumGainDB);
                }
            }

            set_frequency.Focus(); // return focus to the frequency slider
        }

        public double AGCMaximumGainDB
        {
            get { return rcvr.AGCMaximumGainDB; }
            set
            {
                rcvr.AGCMaximumGainDB = value;
            }
        }

        public void UpdateMaxGainAndSlider(double newGain)
        {
            noAGCTrackBarUpdate = true;
            SafeTrackBarValueChange((int)newGain, AGCTrackBar);
            rcvr.AGCMaximumGainDB = AGCTrackBar.Value;
            noAGCTrackBarUpdate = false;
        }

        public double AGCFixedGainDB
        {
            get { return rcvr.AGCFixedGainDB; }
            set
            {
                rcvr.AGCFixedGainDB = value;
            }
        }

        /// <summary>
        /// used to get/set the dB level on the spectral (panadapter) display
        /// </summary>
        public double AGCThreshDB
        {
            get { return rcvr.AGCThreshDB; }
            set
            {
                rcvr.AGCThreshDB = value;
            }
        }

        public double UserAGCHangTime = 1.0;    // 1 ms
        public double UserAGCAttackTime = 10.0;
        public double UserAGCDecayTime = 10.0;

        public double AGCHangTime
        {
            get { return rcvr.AGCHangTime * 1000; }
            set
            {
                rcvr.AGCHangTime = value / 1000.0;
            }
        }

        public int AGCHangThreshold
        {
            get { return (int)(rcvr.AGCHangThreshold * 100.0); }
            set
            {
                rcvr.AGCHangThreshold = value / 100.0;
            }
        }

        /// <summary>
        /// used to get/set where the hang threshold line is on the spectral display (panadapter)
        /// </summary>
        public double AGCHangLevel
        {
            get { return rcvr.AGCHangLevel; }
            set
            {
                rcvr.AGCHangLevel = value;
            }
        }

        public double AGCAttackTime
        {
            get { return rcvr.AGCAttackTime * 1000; }
            set
            {
                rcvr.AGCAttackTime = value / 1000.0;
            }
        }

        public double AGCDecayTime
        {
            get { return rcvr.AGCDecayTime * 1000; }
            set
            {
                rcvr.AGCDecayTime = value / 1000.0;
            }
        }

        public double AGCSlope
        {
            get { return rcvr.AGCSlope; }
            set
            {
                rcvr.AGCSlope = value;
            }
        }

        // Set the AGC speed based on the user selection 
        private void AGCSpeed_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch (AGCSpeed.Text)
            {
                case "OFF":
                    {
                        rcvr.AGCMode = SharpDSP2._1.AGCType_e.agcOff;
                        rcvr.freqAGCType = "OFF";
                        rcvr.freqOutputAGCType = "OFF";
                        break;
                    }
                case "Long":
                    {
                        rcvr.AGCMode = SharpDSP2._1.AGCType_e.agcLong;
                        rcvr.freqAGCType = "Long";
                        rcvr.freqOutputAGCType = "Long";
                        break;
                    }
                case "Slow":
                    {
                        rcvr.AGCMode = SharpDSP2._1.AGCType_e.agcSlow;
                        rcvr.freqAGCType = "Slow";
                        rcvr.freqOutputAGCType = "Slow";
                        break;
                    }
                case "Med":
                    {
                        rcvr.AGCMode = SharpDSP2._1.AGCType_e.agcMedium;
                        rcvr.freqAGCType = "Med";
                        rcvr.freqOutputAGCType = "Med";
                        break;
                    }
                case "Fast":
                    {
                        rcvr.AGCMode = SharpDSP2._1.AGCType_e.agcFast;
                        rcvr.freqAGCType = "Fast";
                        rcvr.freqOutputAGCType = "Fast";
                        break;
                    }
                case "User":
                    {
                        rcvr.AGCMode = SharpDSP2._1.AGCType_e.agcUser;
                        rcvr.freqAGCType = "User";
                        rcvr.freqOutputAGCType = "User";
                        break;
                    }
            }

            bool CustomMode = (rcvr.AGCMode == SharpDSP2._1.AGCType_e.agcUser);

            bool HangEnabled = (rcvr.AGCMode != SharpDSP2._1.AGCType_e.agcMedium) && (rcvr.AGCMode != SharpDSP2._1.AGCType_e.agcFast);

            // if in custom AGC mode, set the times
            if (CustomMode)
            {
                // set the custom times.
                AGCHangTime = UserAGCHangTime;
                AGCAttackTime = UserAGCAttackTime;
                AGCDecayTime = UserAGCDecayTime;
            }

            UpdateAGCTimeDisplay();

            if (SetupFormValid())
            {
                Setup_form.CustomRXAGCEnabled = CustomMode;
            }

            // update whether AGC Hang is enabled, and pass the info to any valid setup form.
            AGCHangEnabled = HangEnabled;

            set_frequency.Focus(); // return focus to the frequency slider
        }

        private void UpdateAGCTimeDisplay()
        {
            if (SetupFormValid())
            {
                Setup_form.CustomRXAGCEnabled = (rcvr.AGCMode == SharpDSP2._1.AGCType_e.agcUser);
                Setup_form.ChangeAGCTimes();
            }
        }

        // helper function to set TX filter bandwidth
        public void SetTXFilterBandwidth()
        {
            if (ourDevice != null)
            {
                ourDevice.SetTxBandwidth(TxFilterLow, TxFilterHigh);
            }
        }

        static int BandwidthSBvalue;  //not used
        static int BandwidthCWvalue;
        // select the appropriate filter bandwidth for the Mode selected 
        public void AdjustFilterByMode()
        {
            SetTXFilterBandwidth();
            if (rcvr.WindowType != SharpDSP2._1.WindowType_e.KAISER_WINDOW)
            {
                BandwidthSBvalue = BandwidthTrackBar.Value;  //not used
                BandwidthCWvalue = BandwidthTrackBar.Value;
            }
            else if (rcvr.WindowType == SharpDSP2._1.WindowType_e.KAISER_WINDOW)
            {
                BandwidthSBvalue = BandwidthTrackBar.Value / 2;  //not used
                BandwidthCWvalue = BandwidthTrackBar.Value / 2;
            }
            switch (Mode.Text)
            {
                case "LSB":
                    rcvr.FilterFrequencyHigh = -BandwidthTrackBar.Value;
                    rcvr.FilterFrequencyLow = -200.0f;
                    if (Setup_form != null)
                    {
                        tx.FilterFrequencyHigh = -Properties.Settings.Default.TxBandWidth;
                    }
                    else
                    {
                        tx.FilterFrequencyHigh = -BandwidthTrackBar.Value;
                    }
                    tx.FilterFrequencyLow = -200.0f;
                    break;

                case "USB":
                    rcvr.FilterFrequencyHigh = BandwidthTrackBar.Value;
                    rcvr.FilterFrequencyLow = 200.0f;
                    if (Setup_form != null)
                    {
                        tx.FilterFrequencyHigh = Properties.Settings.Default.TxBandWidth;
                    }
                    else
                    {
                        tx.FilterFrequencyHigh = BandwidthTrackBar.Value;
                    }
                    tx.FilterFrequencyLow = 200.0f;
                    break;

                case "CWL":
                    rcvr.FilterFrequencyHigh = -(CWPitch + BandwidthCWvalue / 2);
                    rcvr.FilterFrequencyLow = -(CWPitch - BandwidthCWvalue / 2);
                    tx.FilterFrequencyHigh = -(CWPitch + BandwidthTrackBar.Value / 2);
                    tx.FilterFrequencyLow = -(CWPitch - BandwidthTrackBar.Value / 2);
                    break;

                case "CWU":
                    rcvr.FilterFrequencyHigh = (CWPitch + BandwidthCWvalue / 2);
                    rcvr.FilterFrequencyLow = (CWPitch - BandwidthCWvalue / 2);
                    tx.FilterFrequencyHigh = (CWPitch + BandwidthTrackBar.Value / 2);
                    tx.FilterFrequencyLow = (CWPitch - BandwidthTrackBar.Value / 2);
                    break;

                case "AM":
                    int BandwidthAMvalue = BandwidthTrackBar.Value;
                    rcvr.FilterFrequencyHigh = BandwidthAMvalue;
                    rcvr.FilterFrequencyLow = -BandwidthAMvalue;
                    tx.FilterFrequencyHigh = BandwidthAMvalue;
                    tx.FilterFrequencyLow = -BandwidthAMvalue;
                    break;

                case "SAM":
                    int BandwidthSAMvalue = BandwidthTrackBar.Value;
                    rcvr.FilterFrequencyHigh = BandwidthSAMvalue;
                    rcvr.FilterFrequencyLow = -BandwidthSAMvalue;
                    tx.FilterFrequencyHigh = BandwidthSAMvalue;
                    tx.FilterFrequencyLow = -BandwidthSAMvalue;
                    break;

                case "FM":
                    int BandwidthFMvalue = BandwidthTrackBar.Value;
                    rcvr.FilterFrequencyHigh = BandwidthFMvalue;
                    rcvr.FilterFrequencyLow = -BandwidthFMvalue;
                    tx.FilterFrequencyHigh = BandwidthFMvalue;
                    tx.FilterFrequencyLow = -BandwidthFMvalue;
                    break;
            }
        }

        // Vary the filter Bandwidth when the Bandwidth control is changed
        private void BandwidthTrackBar_Scroll(object sender, EventArgs e)
        {
            AdjustFilterByMode();
            set_frequency.Focus(); // return focus to the frequency slider
            labelFilterWidth.Text = BandwidthTrackBar.Value.ToString();
        }

        private void BandwidthTrackBar_ValueChanged(object sender, EventArgs e)
        {
            labelFilterWidth.Text = BandwidthTrackBar.Value.ToString();
        }

        public int aNFAdaptiveFilterSize = 64;
        public int ANFAdaptiveFilterSize
        {
            get { return aNFAdaptiveFilterSize; }
            set { aNFAdaptiveFilterSize = value; rcvr.InterferenceFilterAdaptiveFilterSize = value; }
        }

        public int aNFDelay = 12;
        public int ANFDelay
        {
            get { return aNFDelay; }
            set { aNFDelay = value; rcvr.InterferenceFilterDelay = value; }
        }

        public float aNFAdaptationRate = .005F;
        public float ANFAdaptationRate
        {
            get { return aNFAdaptationRate; }
            set { aNFAdaptationRate = value; rcvr.InterferenceFilterAdaptationRate = value; }
        }

        public float aNFLeakage = .01F;
        public float ANFLeakage
        {
            get { return aNFLeakage; }
            set { aNFLeakage = value; rcvr.InterferenceFilterLeakage = value; }
        }

        // set up the Automatic Notch Filter
        private void ANF_CheckedChanged(object sender, EventArgs e)
        {
            if (ANF.Checked)
            {
                rcvr.InterferenceFilterSwitchOn = true;
                rcvr.InterferenceFilterAdaptiveFilterSize = ANFAdaptiveFilterSize;
                rcvr.InterferenceFilterDelay = ANFDelay;
                rcvr.InterferenceFilterAdaptationRate = ANFAdaptationRate;
                rcvr.InterferenceFilterLeakage = ANFLeakage;
            }
            else
                rcvr.InterferenceFilterSwitchOn = false;

            set_frequency.Focus(); // return focus to the frequency slider
        }

        public int nRAdaptiveFilterSize = 64;
        public int NRAdaptiveFilterSize
        {
            get { return nRAdaptiveFilterSize; }
            set { nRAdaptiveFilterSize = value; rcvr.NoiseFilterAdaptiveFilterSize = value; }
        }

        public int nRDelay = 12;
        public int NRDelay
        {
            get { return nRDelay; }
            set { nRDelay = value; rcvr.NoiseFilterDelay = value; }
        }

        public float nRAdaptationRate = .005F;
        public float NRAdaptationRate
        {
            get { return nRAdaptationRate; }
            set { nRAdaptationRate = value; rcvr.NoiseFilterAdaptationRate = value; }
        }

        public float nRLeakage = .01F;
        public float NRLeakage
        {
            get { return nRLeakage; }
            set { nRLeakage = value; rcvr.NoiseFilterLeakage = value; }
        }

        // set up the Noise Reduction 
        private void NR_CheckedChanged(object sender, EventArgs e)
        {
            if (NR.Checked)
            {
                rcvr.NoiseFilterSwitchOn = true;
                rcvr.NoiseFilterAdaptiveFilterSize = NRAdaptiveFilterSize;
                rcvr.NoiseFilterDelay = NRDelay;
                rcvr.NoiseFilterAdaptationRate = NRAdaptationRate;
                rcvr.NoiseFilterLeakage = NRLeakage;
            }
            else
                rcvr.NoiseFilterSwitchOn = false;

            set_frequency.Focus(); // return focus to the frequency slider
        }

        // Noise Blanker1
        private void NB1_CheckedChanged(object sender, EventArgs e)
        {
            rcvr.BlockNBSwitchOn = NB1.Checked;
            set_frequency.Focus(); // return focus to the frequency slider
        }

        // Noise Blanker2
        private void NB2_CheckedChanged(object sender, EventArgs e)
        {
            rcvr.AveNBSwitchOn = NB2.Checked;
            set_frequency.Focus(); // return focus to the frequency slider
        }

        private void UpdateGraphs()
        {
            pictureBoxWaterfallBackground = null;
            pictureBoxSpectrum.Invalidate();
            pictureBoxWaterfall.Invalidate();
        }

        private void Frequency_change()
        {
            // we need to check if we are too close to the band edge to take a step 
            if (set_frequency.Value > previous_frequency_value && set_frequency.Value < (set_frequency.Maximum - step_size))
                set_frequency.Value += step_size;  // add step_size since scroll wheel has added 1 already
            else if (set_frequency.Value < previous_frequency_value && set_frequency.Value > (set_frequency.Minimum + step_size + 1))
                set_frequency.Value -= step_size; // subtract step_size since scroll wheel has subtracted 1 already
            // else use the current value

            // round frequency to step size
            int temp_frequency = set_frequency.Value; // use a temp value so we don't have to worry about the band edges.
            temp_frequency = temp_frequency / (step_size + 1);
            temp_frequency = temp_frequency * (step_size + 1);
            set_frequency.Value = temp_frequency;

            // display the new frequency and send to Ozy
            DisplayFrequency(set_frequency.Value);
            // save current value so we can compare with next value so we can determine tune direction
            previous_frequency_value = set_frequency.Value;
        }

        public float ProcGain;
        public float VOXThreshold;

        public bool received_flag = false;


        public static byte[] StrToByteArray(string str)
        {
            System.Text.UTF8Encoding encoding = new System.Text.UTF8Encoding();
            return encoding.GetBytes(str);
        }


        // when ever timer1 fires we update the bandscope
        private void timer1_Tick(object sender, EventArgs e)
        {
            // display the status of Sync and ADC overload on the 'LEDs'
            SyncLED.BackColor = (IsSynced & received_flag) ? Color.Green : Color.Red;
            received_flag = false;  // reset flag for next time
            ADCoverloadButton.BackColor = ADCOverload ? Color.Red : SystemColors.Control;

            if ((PenneyPresent || PennyLane || Hermes) && (BandSelect.Text != "GC"))  // enable Tx controls if Penny present and check 10MHz clock is selected
            {
                MOX.Enabled = true;
                TUN.Enabled = true;
                // If using Penny or PennyLane check we have a 10MHz clock selected,if not warn user and turn KK off
                if (!Hermes && !Penelope10MHz && !Atlas10MHz && !Excalibur && !Mercury10MHz)
                {
                    timer1.Enabled = false;
                    if (MessageBox.Show("Warning - no 10MHz clock selected", "Configuration Error!", MessageBoxButtons.OK, MessageBoxIcon.Error)
                        == DialogResult.OK)
                    {
                        setupToolStripMenuItem_Click(this, EventArgs.Empty);  // open setup form for user
                        OnOffButton_Click(this, EventArgs.Empty);  // turn KK off 
                    }
                }
            }
            else
            {
                MOX.Enabled = false;
                TUN.Enabled = false;
            }

            setVolumeSquelched();  // see if we need to mute the audio

#if false
            // if Hermes, display the voltage supplied to the board and user inputs
            if (Hermes)
            {
                Console.WriteLine(" Volts = \t" + SupplyVolts);
                Console.WriteLine(" IO1 = \t" + IO1);
                Console.WriteLine(" IO2 = \t" + IO2);
                Console.WriteLine(" IO3 = \t" + IO3);
            }
#endif

            if (read_ready)  // update the display if data is available 
            {
            //    read_ready = false;

                // get a wide-bandwidth sample
                if ((ourDevice != null) && (KKDevice == KKMethod.USB))

               //     if ((KKDevice == KKMethod.USB))
                    {
                        // we only call this in the USB code.  In the Ethernet code, this data is processed in EthernetDevice.cs
                        // on the data thread, because both EP6 and EP4 packets are received there.
               //         ourDevice.ProcessWideBandData(ref ourDevice.EP4buf);
                    }
                
        //        ComputeDisplay();  //w3sz moved to backgroundworker1completed when separated Compute Display from audio processing
        //        UpdateGraphs();  //w3sz

                Squelch();
                
                    pictureBoxSpectrum.Invalidate(); // causes RefreshScope to run 
                
                    // get rid of the background so that it will redraw if activated in the future
                    pictureBoxWaterfallBackground = null;
                    pictureBoxWaterfall.BackgroundImage = null;
                    pictureBoxWaterfall.Invalidate(); // causes RefreshScope to run

                // not a cross-thread call when called from the timer tick!
                // display SMeter reading.  Note that this is accumulated by the Receiver object
                // when it is invoked by Process_Data (G Byrkit, K9TRV, 15 June 2009)
                SetSMeterText(SMeterText(rcvr.SMeterAvgValue, rcvr.SMeterInstValue));

                if ((PenneyPresent || PennyLane || Hermes) && (Alex) && (ourDevice != null))
                {
                    // get power settings to local variables so any change on the data thread has no further effect
                    double fwdPower = ourDevice.AlexForwardPower;
                    double revPower = ourDevice.AlexReversePower;

                    if (fwdPower > 1.0)
                    {
                        textBoxForwardPower.Text = String.Format("{0:F1} W", fwdPower);
                    }
                    else
                    {
                        textBoxForwardPower.Text = String.Format("{0:F0} mW", fwdPower * 1000);
                    }

                    if (revPower > 1.0)
                    {
                        textBoxReversePower.Text = String.Format("{0:F1} W", revPower);
                    }
                    else
                    {
                        textBoxReversePower.Text = String.Format("{0:F0} mW", revPower * 1000);
                    }
                }
                else
                {
                    textBoxForwardPower.Text = "";
                    textBoxReversePower.Text = "";
                }

                //this is UI data sender; sends data back to radio client GUI //w3sz
                ThreadPool.QueueUserWorkItem(delegate //w3sz
                {
                byte[] FwdPwrtextstrng = StrToByteArray("FwdPwr" + textBoxForwardPower.Text); //w3sz  convert to byte array
                int FwdPwrstrngsize = FwdPwrtextstrng.Length;  //w3sz  get byte array size
                byte[] RevPwrtextstrng = StrToByteArray("RevPwr" + textBoxReversePower.Text); //w3sz  convert to byte array
                int RevPwrstrngsize = RevPwrtextstrng.Length;  //w3sz  get byte array size
                byte[] SMeterValuetextstrng = StrToByteArray("SMeter" + SMeterValue.ToString()); //w3sz  convert to byte array
                int SMeterValuestrngsize = SMeterValuetextstrng.Length;  //w3sz  get byte array size

                byte[] MOXStatusstrng = StrToByteArray("MoxSet" + MOX_set.ToString());
                int MOXStatusstringsize = MOXStatusstrng.Length;

                byte[] FilterWidthstrng = StrToByteArray("FiltWi" + labelFilterWidth.Text);
                int FilterWidthstrngsize = FilterWidthstrng.Length;


                if (chkClient == true)
                {
                    UdpClient udpServer3 = new UdpClient(); //w3sz 
                    IPEndPoint localpt3 = new IPEndPoint(IPAddress.Parse(w3szendpoint), server3port);  //w3sz power, SMeter data to client

                    try
                    {
                        udpServer3.Client.Connect(localpt3);  //w3sz changed from .Bind Jan 2024
                    }                  
		            catch (Exception ex)
		            {
		                Console.WriteLine(ex.ToString());
		            }
                    udpServer3.Send(FwdPwrtextstrng, FwdPwrstrngsize, localpt3); // send ForwardPower  //w3sz   
                    udpServer3.Send(RevPwrtextstrng, RevPwrstrngsize, localpt3); // send ReversePower  //w3sz   
                    udpServer3.Send(SMeterValuetextstrng, SMeterValuestrngsize, localpt3); // send SMeterData  //w3sz  
                    udpServer3.Send(MOXStatusstrng, MOXStatusstringsize, localpt3);  // send MOX status //w3sz
                    udpServer3.Send(FilterWidthstrng, FilterWidthstrngsize, localpt3);  // send Filter Width //w3sz
                    udpServer3.Close(); //w3sz 
                }
                });
            }

        //            Console.WriteLine("Temp MicAGCGain = \t {0} \t TempMicPeak = \t {1}", TempMicAGC, TempMicPeak);
        }

        //add mic signal level
        public void SetMicGain()
        {
            if (ourDevice != null)
            {
                ourDevice.SetMicGain();
            }
        }

        public bool squelched = false;

        // process bandscope and filter squelch.
        private void Squelch()
        {
            /* 
             * 
             * When using the filter width squelch we need to compenstate for the fact that the bandwidth is wider than
             * that when using the bandscope width. The FFT bin width (Hz) for the bandscope is 
             * 
             *      bandwidth =  sampling rate / FFT size 
             *      
             * For the filter bandwidth we can just read the filter slider.
             * Hence the  noise power will be higher in the filter bandwidth by 
             * 
             *       ratio = filter bandwidth/(sampling rate/FFT size)
             *       
             * Since we are working in dB power levels we need to take 10log of this value and apply to the signal level
             * 
             */

            // if squelch is not selected then just return
            if (!bandscopeSquelch && !filterSquelch)
            {
                Squelch_level.BackColor = SystemColors.Control;
                return;
            }
            int BandWidthCorrection;
            if (FreqSpan != 0)
            {
                BandWidthCorrection = Convert.ToInt32(10.0f * Math.Log10(BandwidthTrackBar.Value / (FreqSpan / 1024)));
            }
            else
            {
                BandWidthCorrection = Convert.ToInt32(10.0f * Math.Log10(BandwidthTrackBar.Value / (SampleRate / 1024)));
            }

            // Do Bandscope squelch
            if (bandscopeSquelch && (max_signal < Squelch_level.Value / 10))
            {
                squelched = true;
                Squelch_level.BackColor = Color.Yellow; // change the slider background to show we have muted the audio 
            }

            // Do filter squelch - this uses the average SMeter value as its level
            else if (filterSquelch && (filter_max_signal < ((Squelch_level.Value / 10) + BandWidthCorrection)))
            {
                squelched = true;
                Squelch_level.BackColor = Color.Yellow; // change the slider background to show we have muted the audio 
            }
            else  // use the user volume setting 
            {
                squelched = false;
                Squelch_level.BackColor = Color.Green; // change the slider background to show are above threshold
            }

            setVolumeSquelched();
        }
                
        public bool PreampOn = false;

        private void Preamp_CheckedChanged(object sender, EventArgs e)
        {
            PreampOn = Preamp.Checked;

            // save the preamp setting for each band
            switch (BandSelect.Text)
            {
                case "160m": Preamp_160 = PreampOn; break;
                case "80m": Preamp_80 = PreampOn; break;
                case "40m": Preamp_40 = PreampOn; break;
                case "30m": Preamp_30 = PreampOn; break;
                case "20m": Preamp_20 = PreampOn; break;
                case "17m": Preamp_17 = PreampOn; break;
                case "15m": Preamp_15 = PreampOn; break;
                case "12m": Preamp_12 = PreampOn; break;
                case "10m": Preamp_10 = PreampOn; break;
                case "6m": Preamp_6 = PreampOn; break;
                case "GC": Preamp_GC = PreampOn; break;
            }
            set_frequency.Focus(); // return focus to the frequency slider
        }

        // save the current band & frequency in a quick memory
        private void StoreFreq_Click(object sender, EventArgs e)
        {
            StoreFreq.BackColor = Color.Gray;
            QuickMemoryBand = BandSelect.Text;
            QuickMemory = set_frequency.Value;
            set_frequency.Focus(); // return focus to the frequency slider
        }

        // restore a previously stored quick band & frequency 
        private void RecallFreq_Click(object sender, EventArgs e)
        {
            RecallFreq.BackColor = Color.Gray;
            BandSelect.Text = QuickMemoryBand;
            set_frequency.Value = QuickMemory;
            Frequency_change();     // process the new frequency 
            set_frequency.Focus(); // return focus to the frequency slider

        }

        public string oldfreq = "28.150 000";

        // process key strokes from the keyboard 
        protected override void OnKeyDown(KeyEventArgs e)
        {
            if (display_freqB.Focused)
            {
                //for VFO B, split tx freq
                // setting suppressKeyPress to true does several things:
                // 1) it prevents the auto-display of the key
                // 2) sets e.Handled to true
                // this keeps the textbox from being corrupted by typed keys that are
                // not processed by this event handler, and makes sure that what is (eventually)
                // displayed is correct
                e.SuppressKeyPress = true;

                bool display = false;
                Debug.WriteLine("got a key \t" + e.KeyCode);

                // process key presses from the numeric keypad and keyboard
                switch (e.KeyCode)
                {
                    case Keys.D0:
                    case Keys.NumPad0: KeypadFrequency += "0"; keycount++; break;

                    case Keys.D1:
                    case Keys.NumPad1: KeypadFrequency += "1"; keycount++; break;

                    case Keys.D2:
                    case Keys.NumPad2: KeypadFrequency += "2"; keycount++; break;

                    case Keys.D3:
                    case Keys.NumPad3: KeypadFrequency += "3"; keycount++; break;

                    case Keys.D4:
                    case Keys.NumPad4: KeypadFrequency += "4"; keycount++; break;

                    case Keys.D5:
                    case Keys.NumPad5: KeypadFrequency += "5"; keycount++; break;

                    case Keys.D6:
                    case Keys.NumPad6: KeypadFrequency += "6"; keycount++; break;

                    case Keys.D7:
                    case Keys.NumPad7: KeypadFrequency += "7"; keycount++; break;

                    case Keys.D8:
                    case Keys.NumPad8: KeypadFrequency += "8"; keycount++; break;

                    case Keys.D9:
                    case Keys.NumPad9: KeypadFrequency += "9"; keycount++; break;
                    case Keys.Back:
                        if (keycount > 0)  // remove the last character entered
                        {
                            KeypadFrequency = KeypadFrequency.Remove(KeypadFrequency.Length - 1);
                            --keycount;
                        }
                        break;
                    case Keys.Enter:
                        if (keycount > 0)   // so we don't trigger on the main Enter Key 
                            display = true;
                        break;

                    case Keys.Decimal:
                    case Keys.OemPeriod:
                        if ((KeypadFrequency == null) || !KeypadFrequency.Contains("."))
                        {
                            if (keycount == 0)
                            {
                                // force a leading 0 if no chars present yet
                                KeypadFrequency += "0";
                                keycount++;
                            }
                            KeypadFrequency += ".";
                            keycount++;
                        }
                        break;

                    // if Up arrow increment frequency by step size
                    case Keys.Up:
                        {
                            Properties.Settings.Default.split_tx_freq = Properties.Settings.Default.split_tx_freq + step_size + 1;
                            Properties.Settings.Default.Save();
                            DisplayFrequencyB(Properties.Settings.Default.split_tx_freq);
                            KeypadFrequency = null;
                            keycount = 0;
                            set_frequency.Focus(); // return focus to the frequency slider
                            return;
                        }
                    // if Down arrow decrement frequency by step size
                    case Keys.Down:
                        {
                            Properties.Settings.Default.split_tx_freq = Properties.Settings.Default.split_tx_freq - step_size - 1;
                            Properties.Settings.Default.Save();
                            DisplayFrequencyB(Properties.Settings.Default.split_tx_freq);
                            KeypadFrequency = null;
                            keycount = 0;
                            set_frequency.Focus(); // return focus to the frequency slider
                            return;
                        }


                    // if not a valid key then exit
                    default:
                        {
                            display = false;
                            return;
                        }
                }

                double temp = 0.0d;
                long setfrequency;
                if (display)  // user has pressed enter key 
                {
                    try  // data format may be incorrect so check
                    {
                        // must use the invariant culture here, so that the '.' is properly
                        // interpreted as a decimal separator!
                        temp = (double)Convert.ToDecimal(KeypadFrequency, nfi); // convert to a float 
                    }

                    catch // format error so tidy up and return
                    {
                        Console.WriteLine("Error in display segment after temp");
                        display_freqB.Text = oldfreq;    // display the previous frequency
                        set_frequency.Focus(); // return focus to the frequency slider
                        KeypadFrequency = null;
                        keycount = 0;
                        return;
                    }
                    double temp2 = temp * 1000000.0d;  // Multiply by 10e6 to give Hz
                    setfrequency = (long)temp2; // convert to an integer
                                               // check frequency is valid
                    if ((keycount > 0) && (keycount < 10) && (setfrequency < 55000001) && (setfrequency > 0))
                    {
                        Debug.WriteLine(setfrequency);
                        // display the new frequency and send to Ozy  
                        DisplayFrequencyB(setfrequency);
                        KeypadFrequency = null;
                        keycount = 0;
                        set_frequency.Focus(); // return focus to the frequency slider
                        return;
                    }
                    else  // error, abort
                    {
                        Console.WriteLine("Now in error abort, setfrequency = {0} and keycount = {1}", setfrequency,keycount);

                        display_freqB.Text = oldfreq;   // display the previous frequency
                        set_frequency.Focus(); // return focus to the frequency slider
                        KeypadFrequency = null;
                        keycount = 0;
                        return;
                    }
                }
                else
                {
                    // display the frequency as it is entered but only the text entered on the keypad 
                    display_freqB.Text = KeypadFrequency;
                    Console.WriteLine(KeypadFrequency);
                }
            }

            else
            { 
            //for VFO A
            // setting suppressKeyPress to true does several things:
            // 1) it prevents the auto-display of the key
            // 2) sets e.Handled to true
            // this keeps the textbox from being corrupted by typed keys that are
            // not processed by this event handler, and makes sure that what is (eventually)
            // displayed is correct
            e.SuppressKeyPress = true;

            bool display = false;
            Debug.WriteLine("got a key \t" + e.KeyCode);

            // process key presses from the numeric keypad and keyboard
            switch (e.KeyCode)
            {
                case Keys.D0:
                case Keys.NumPad0: KeypadFrequency += "0"; keycount++; break;

                case Keys.D1:
                case Keys.NumPad1: KeypadFrequency += "1"; keycount++; break;

                case Keys.D2:
                case Keys.NumPad2: KeypadFrequency += "2"; keycount++; break;

                case Keys.D3:
                case Keys.NumPad3: KeypadFrequency += "3"; keycount++; break;

                case Keys.D4:
                case Keys.NumPad4: KeypadFrequency += "4"; keycount++; break;

                case Keys.D5:
                case Keys.NumPad5: KeypadFrequency += "5"; keycount++; break;

                case Keys.D6:
                case Keys.NumPad6: KeypadFrequency += "6"; keycount++; break;

                case Keys.D7:
                case Keys.NumPad7: KeypadFrequency += "7"; keycount++; break;

                case Keys.D8:
                case Keys.NumPad8: KeypadFrequency += "8"; keycount++; break;

                case Keys.D9:
                case Keys.NumPad9: KeypadFrequency += "9"; keycount++; break;
                case Keys.Back:
                    if (keycount > 0)  // remove the last character entered
                    {
                        KeypadFrequency = KeypadFrequency.Remove(KeypadFrequency.Length - 1);
                        --keycount;
                    }
                    break;
                case Keys.Enter:
                    if (keycount > 0)   // so we don't trigger on the main Enter Key 
                        display = true;
                    break;

                case Keys.Decimal:
                case Keys.OemPeriod:
                    if ((KeypadFrequency == null) || !KeypadFrequency.Contains("."))
                    {
                        if (keycount == 0)
                        {
                            // force a leading 0 if no chars present yet
                            KeypadFrequency += "0";
                            keycount++;
                        }
                        KeypadFrequency += ".";
                        keycount++;
                    }
                    break;

                // if Up arrow increment frequency by step size
                case Keys.Up: KeypadFrequency = set_frequency.Value++.ToString(); display = true; break;

                // if Down arrow decrement frequency by step size
                case Keys.Down: KeypadFrequency = set_frequency.Value--.ToString(); display = true; break;


                // if not a valid key then exit
                default:
                    {
                        display = false;
                        return;
                    }
            }

            double temp = 0.0d;
            long setfrequency;
            if (display)  // user has pressed enter key 
            {
                try  // data format may be incorrect so check
                {
                    // must use the invariant culture here, so that the '.' is properly
                    // interpreted as a decimal separator!
                    temp = (double)Convert.ToDecimal(KeypadFrequency, nfi); // convert to a float 
                }

                catch // format error so tidy up and return
                {
                    Frequency_change();     // display the previous frequency
                    set_frequency.Focus(); // return focus to the frequency slider
                    KeypadFrequency = null;
                    keycount = 0;
                    return;
                }
                double temp2 = temp * 1000000.0d;  // Multiply by 10e6 to give Hz
                setfrequency = (long)temp2; // convert to an integer
                // check frequency is valid
                if ((keycount > 0) && (keycount < 10) && (setfrequency < 55000001) && (setfrequency > 0))
                {
                    Debug.WriteLine(setfrequency);
                    BandSelect.Text = getBand(setfrequency);
                    set_frequency.Value = (int)setfrequency;
                        // display the new frequency and send to Ozy
                        Console.WriteLine("Set frequency is {0} before DisplayFrequency", setfrequency);
                    DisplayFrequency(setfrequency);
                    KeypadFrequency = null;
                    keycount = 0;
                    set_frequency.Focus(); // return focus to the frequency slider
                    return;
                }
                else  // error, abort
                {
                    Frequency_change();     // display the previous frequency
                    set_frequency.Focus(); // return focus to the frequency slider
                    KeypadFrequency = null;
                    keycount = 0;
                    return;
                }
            }
            else
            {
                // display the frequency as it is entered but only the text entered on the keypad 
                display_freq.Text = KeypadFrequency;
            }
        }
        }

        private void DisplayFrequency(long setfrequency)
        {
            // convert tune frequency to an array of bytes ready to send to Ozy
            Console.WriteLine("Set frequency is {0} on arrival in DisplayFrequency", setfrequency);
            frequency = BitConverter.GetBytes(set_frequency.Value);
            // for now set the Duplex frequency  = simplex frequency
            duplex_frequency = BitConverter.GetBytes(set_frequency.Value);
            // format the integer frequency to have a decimal point between the MHz digits.
            double temp_freq = (double)setfrequency / 1000000.0f;
            // convert to a string with 6 digits to the right of the decimal point 
            string freq = String.Format(nfi, "{0:F6}", (double)temp_freq);
            // strip off the last 3 digits, add a space then add them back 
            display_freq.Text = freq.Remove(freq.Length - 3) + ' ' + freq.Substring(freq.Length - 3, 3);
            
        }
        
        private void DisplayFrequencyB(long setfrequency)
        {
            // convert tune frequency to an array of bytes ready to send to Ozy
            Console.WriteLine("Set frequency is {0} on arrival in DisplayFrequencyB", setfrequency);
            split_tx_frequency_bytes = BitConverter.GetBytes(setfrequency);
            Properties.Settings.Default.split_tx_freq = setfrequency;
            Properties.Settings.Default.Save();
            // format the integer split txfrequency to have a decimal point between the MHz digits.
            double temp_split_tx_freq = (double)setfrequency / 1000000.0f;
            Console.WriteLine("In DisplayFrequencyB, setfrequency = {0}", setfrequency);
            Console.WriteLine("In DisplayFrequencyB, temp_split_tx_freq = {0}", temp_split_tx_freq);
            // convert to a string with 6 digits to the right of the decimal point 
            string split_tx_freq = String.Format(nfi, "{0:F6}", (double)temp_split_tx_freq);
            Console.WriteLine("In DisplayFrequencyB, split_tx_freq = {0}", split_tx_freq);
            // strip off the last 3 digits, add a space then add them back 
            display_freqB.Text = split_tx_freq.Remove(split_tx_freq.Length - 3) + ' ' + split_tx_freq.Substring(split_tx_freq.Length - 3, 3);
            oldfreq = display_freqB.Text;
        }


        // find out what band we are on when user enters frequency from keypad
        private string getBand(long frequency)
        {
            if (frequency > 1800000 && frequency < 2000000) return "160m";
            else if (frequency > 3500000 && frequency < 4000000) return "80m";
            else if (frequency > 7000000 && frequency < 7300000) return "40m";
            else if (frequency > 10100000 && frequency < 10150000) return "30m";
            else if (frequency > 14000000 && frequency < 14350000) return "20m";
            else if (frequency > 18068000 && frequency < 18168000) return "17m";
            else if (frequency > 21000000 && frequency < 21450000) return "15m";
            else if (frequency > 24880000 && frequency < 24980000) return "12m";
            else if (frequency > 28000000 && frequency < 29700000) return "10m";
            else if (frequency > 50000000 && frequency < 55000000) return "6m";
            return "GC";
        }

        public bool SetupFormValid()
        {
            // if the form is null or disposed, it is NOT valid
            if (Setup_form == null || Setup_form.IsDisposed)
                return false;
            else
                return true;
        }

        private void setupToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // only allow user to open form if not already open
            if (!SetupFormValid())
            {
                Setup_form = new SetupForm(this);
                Setup_form.TopMost = false;          // have the setup form appear over the main form.
                Setup_form.SampleRate.Text = SampleRate.ToString(); // pass the current Sampling Rate 
                Setup_form.Owner = this;
                Setup_form.Show();                  // display the Setup form
                set_frequency.Focus();              // return focus to the frequency slider
                Setup_form.Set_CWPitch.Value = CWPitch; // setup the initial value of cw pitch control on the setup form
                Setup_form.CustomRXAGCEnabled = (rcvr.AGCMode == SharpDSP2._1.AGCType_e.agcUser);
            }
            else  // form already open 
            {
                // handle setup form already being shown.  Make sure the user sees it...
                Setup_form.Show();                  // display the Setup form
                set_frequency.Focus();              // return focus to the frequency slider
            }
        }

        Image pictureBoxWaterfallBackground = null;

        private void pictureBoxWaterfall_Paint(object sender, PaintEventArgs e)
        {
            Graphics g = e.Graphics;
            RefreshWaterfall(g);
        }

        private void pictureBoxSpectrum_Paint(object sender, PaintEventArgs e)
        {
            Graphics g = e.Graphics;
            DrawSpectrumGrid(g);
            RefreshSpectrumScope(g);
        }

        private void pictureBoxSpectrum_MouseDown(object sender, MouseEventArgs e)
        {
            // check if user has clicked over squelch level control rectangle, if so set SquelchDrag to true
            if (squelch.Contains(e.X, e.Y))  // this gives true if the mouse is inside the squelch rectangle
            {
                SquelchDrag = true;
                AGCHangDrag = false;
                AGCKneeDrag = false;
                pictureBoxSpectrum.Cursor = Cursors.HSplit;
                return;  // since we don't want any other controls active along with squench
            }
            if (AGCHang.Contains(e.X, e.Y))
            {
                if (agcHangEnabled)
                {
                    AGCHangDrag = true;
                    SquelchDrag = false;
                    AGCKneeDrag = false;
                    pictureBoxSpectrum.Cursor = Cursors.HSplit;
                    return;
                }
                else
                {
                    AGCHangDrag = false;
                }
            }
            if (AGCKnee.Contains(e.X, e.Y))
            {
                AGCKneeDrag = true;
                AGCHangDrag = false;
                SquelchDrag = false;
                pictureBoxSpectrum.Cursor = Cursors.HSplit;
                return;
            }

            // check if we are adjacent to the right filter location, if so change the mouse cursor to <-> and 
            // disable mouse drag tuning of the frequency and filter shift
            if (Control.MouseButtons == MouseButtons.Left && MousePositionX > FilterRight - 5 && MousePositionX < FilterRight)
            {
                pictureBoxSpectrum.Cursor = Cursors.SizeWE; // change cursor to <->
                FrequencyDrag = false;   // turn off drag tuning of frequency and filter shift 
                BandwidthShift = false;
                HighFilter = true;      // allow drag tuning of high filter frequency
                LowFilter = false;
            }
            // check if we are adjacent to the left filter location, if so change the mouse cursor to <-> and 
            // disable mouse drag tuning of the frequency and filter shift
            else if (Control.MouseButtons == MouseButtons.Left && MousePositionX > FilterLeft && MousePositionX < FilterLeft + 5)
            {
                pictureBoxSpectrum.Cursor = Cursors.SizeWE; // change cursor to <->
                FrequencyDrag = false;      // turn off drag tuning of frequency and filter shift 
                BandwidthShift = false;
                HighFilter = false;
                LowFilter = true;           // allow drag tuning of low filter frequency
            }

            // if the mouse postion is to the left or right of the filter location then change the mouse cursor to a hand
            // and enable frequency drag. If we have right clicked then ShowCross will be true in which case set the frequency  
            // to the current mouse position.
            else if (Control.MouseButtons == MouseButtons.Left && (MousePositionX < FilterLeft || MousePositionX > FilterRight))
            {
                pictureBoxSpectrum.Cursor = Cursors.Hand; // change cursor to a hand

                FrequencyDrag = true;  // we are not inside the filter bandwidth so enable drag tuning

                // if the cross is shown then set frequency to the X location of the mouse
                if (ShowCross)
                {
                    // calculate the number of Hz each pixel represents, will be  sample rate/screen bandscope width
                    Double HzPerPixel;
                    if (FreqSpan != 0)
                    {
                        HzPerPixel = FreqSpan / pictureBoxSpectrum.Width; // how many pixels per 100Hz across the screen 
                    }
                    else
                    {
                        HzPerPixel = SampleRate / pictureBoxSpectrum.Width;
                    }


                    // calculate the  how many Hz the X mouse location represents from the left edge of the screen
                    Double offset = MousePositionX * HzPerPixel;
                    int left_edge_frequency;
                    // the frequency at the left edge of the screen will be the tuned frequency - sample rate/2
                    if (FreqSpan != 0)
                    {
                        left_edge_frequency = set_frequency.Value - Convert.ToInt32(FreqSpan / 2);
                    }
                    else
                    {
                        left_edge_frequency = set_frequency.Value - SampleRate / 2;
                    }
                    // add the offset to the left edge frequency
                    // do so in a way that does NOT generate an exception when going too low in frequency
                    if (left_edge_frequency + offset <= set_frequency.Minimum)
                    {
                        set_frequency.Value = set_frequency.Minimum;
                    }
                    else if (left_edge_frequency + offset >= set_frequency.Maximum)
                    {
                        set_frequency.Value = set_frequency.Maximum;
                    }
                    else
                    {
                        if (Mode.Text == "CWL")  // TODO: Hack so that click tune on CWL mode works - need to fix
                     //       set_frequency.Value = left_edge_frequency + offset + 1000;
                            set_frequency.Value = Convert.ToInt32(left_edge_frequency + offset + CWPitch);
                        else if (Mode.Text == "CWU")  // TODO: Hack so that click tune on CWL mode works - need to fix
                            //       set_frequency.Value = left_edge_frequency + offset + 1000;
                            set_frequency.Value = Convert.ToInt32(left_edge_frequency + offset - CWPitch);
                        else
                            set_frequency.Value = Convert.ToInt32(left_edge_frequency + offset);
                    }
                    // display the new frequency and send to Ozy
                    DisplayFrequency(set_frequency.Value);
                    set_frequency.Focus(); // return focus to the frequency slider
                }
            }
            // if the right mouse button is down then show large cross 
            else if (Control.MouseButtons == MouseButtons.Right)  // change the cursor to to a large cross
            {
                pictureBoxSpectrum.Cursor = Cursors.Cross;         // change cursor to a small cross
                // set the ShowCross flag, this is read in the DrawGrid() Method
                ShowCross = !ShowCross;                     // toggle the cross each right click
                FrequencyDrag = true;
            }

            else // we are inside the filter bandwidth so disable frequency drag tuning
            {
                FrequencyDrag = false;
                pictureBoxSpectrum.Cursor = Cursors.NoMoveHoriz;
            }
        }

        private void pictureBoxSpectrum_MouseEnter(object sender, EventArgs e)
        {
            pictureBoxSpectrum.Cursor = Cursors.Cross;
        }

        private void pictureBoxSpectrum_MouseLeave(object sender, EventArgs e)
        {
            pictureBoxSpectrum.Cursor = Cursors.Default;
            SquelchDrag = false;
            AGCHangDrag = false;
            AGCKneeDrag = false;
        }

        int mouse_delta_Y;

        private void pictureBoxSpectrum_MouseMove(object sender, MouseEventArgs e)
        {
            // get the current cursor shape so we can restore it upon leaving 
            Cursor CurrentCursor = pictureBoxSpectrum.Cursor;

            MousePositionX = e.X;   // get the mouse X and Y coordintes with respect to the bandscope
            MousePositionY = e.Y;   // these are used by the right mouse click over bandscope             
            Point MouseX = new Point();  // the mouse coordinates have a type Point
            MouseX = Control.MousePosition;  // get current mouse postion (X,Y) with respect to the screen
            // so we can drag tune outside the width of the bandscope area 
            delta = MouseOld.X - MouseX.X;  // get the difference between the last and current Mouse X coordinate
            int abs_delta = Math.Abs(delta);
            mouse_delta_Y = MouseOld.Y - MouseX.Y; // get the diffetence between the last and current Mouse Y coordinate
            int abs_mouse_delta_Y = Math.Abs(mouse_delta_Y);

            // check if the user has clicked on the squelch control rectangle, if so move the squelch threshold with the mouse
            if (SquelchDrag)
            {
                if (mouse_delta_Y < 0)  // mouse has moved down 
                {
                    if (Squelch_level.Value / 10 > (GridMin + abs_mouse_delta_Y)) // check we can make a valid move
                        Squelch_level.Value -= abs_mouse_delta_Y * (512 * 3 / pictureBoxSpectrum.Height);
                    //  512 * 3/pictureBoxSpectrum.Height is a magic number to get the line to move at a nice speed!
                }

                else if (mouse_delta_Y > 0) // mouse has moved up
                {
                    if (Squelch_level.Value / 10 < (GridMax - abs_mouse_delta_Y))  // check we can make a valid move
                        Squelch_level.Value += abs_mouse_delta_Y * (512 * 3 / pictureBoxSpectrum.Height);
                }

                // update text box
                Squelch_level_Scroll(this, EventArgs.Empty);
                MouseOld = MouseX; // save the last mouse X location for next time the mouse moves
                return;  // so other mouse controls don't operate 
            }

            if (agcHangEnabled && AGCHangDrag)
            {
                double agc_hang_point = PixelToDb(e.Y);
                // if (agc_hang_point > GridMax) agc_hang_point = GridMax;
                // if (agc_hang_point < -121.0) agc_hang_point = -121.0;
                agc_hang_point = agc_hang_point - (double)cal_offset;

                AGCHangLevel = agc_hang_point;

                int hang_threshold = AGCHangThreshold;
                if (SetupFormValid())
                    Setup_form.ChangeHangThresh(hang_threshold);

                MouseOld = MouseX; // save the last mouse X location for next time the mouse moves
                return;
            }

            if (AGCKneeDrag)
            {
                double agc_thresh_point = PixelToDb(e.Y);
                // if (agc_thresh_point > GridMax) agc_thresh_point = GridMax;
                // if (agc_thresh_point < GridMin) agc_thresh_point = GridMin;
                agc_thresh_point = agc_thresh_point - (double)cal_offset;
                // Debug.WriteLine("agc_db_point2: " + agc_db_point);

                AGCThreshDB = agc_thresh_point;

                double agc_top = AGCMaximumGainDB;
                agc_top = Math.Round(agc_top);

                SafeTrackBarValueChange((int)agc_top, AGCTrackBar);
                AGCTrackBar_Scroll(this, EventArgs.Empty);  // force AGC Threshold to accept previous saved value

                MouseOld = MouseX; // save the last mouse X location for next time the mouse moves
                return;
            }

            if (Control.MouseButtons == MouseButtons.Left && FrequencyDrag) // do this if the left mouse button is down
            {
                // if mouse has moved left then decrease frequency but first 
                // we need to check if we are too close to the band edges to move 
                if (delta < 0)
                {
                    if (FreqSpan != 0)
                    {
                        if (set_frequency.Value > (set_frequency.Minimum + Convert.ToInt32(abs_delta * (FreqSpan / pictureBoxSpectrum.Width))))
                            set_frequency.Value -= Convert.ToInt32(abs_delta * (FreqSpan / pictureBoxSpectrum.Width));
                    }
                    else
                    {
                        if (set_frequency.Value > (set_frequency.Minimum + abs_delta * SampleRate / pictureBoxSpectrum.Width))
                            set_frequency.Value -= abs_delta * SampleRate / pictureBoxSpectrum.Width;

                    }
                }
                // if mouse has moved right then decrease frequency but first
                // we need to check if we are too close to the band edges to move  
                else if (delta > 0)
                {
                    if (FreqSpan != 0)
                    {
                        if (set_frequency.Value < (set_frequency.Maximum - Convert.ToInt32(abs_delta * ((FreqSpan) / pictureBoxSpectrum.Width))))
                            set_frequency.Value += Convert.ToInt32(abs_delta * ((FreqSpan) / pictureBoxSpectrum.Width));
                    }
                    else
                        if (set_frequency.Value < (set_frequency.Maximum - abs_delta * (SampleRate / pictureBoxSpectrum.Width)))
                            set_frequency.Value += abs_delta * (SampleRate / pictureBoxSpectrum.Width);
                }

                DisplayFrequency(set_frequency.Value);

                set_frequency.Focus(); // return focus to the frequency slider
            }

            // Allow user to alter  filter  high and low frequencies using mouse.
            // If the mouse is in the region of the left edge of the filter allow it to be drag tuned
            // Placing the mouse in this region will alter the cursor, if the left mouse button is
            // down then the filter can be altered by draging the mouse left or right.
            // If the user has pressed the left mouse button whilst in this region then LowFilter will be true

            else if (MousePositionX > FilterLeft && MousePositionX < FilterLeft + 5 || LowFilter)
            {
                pictureBoxSpectrum.Cursor = Cursors.SizeWE;            // change cursor to <->
                if (Control.MouseButtons == MouseButtons.Left)
                {
                    BandwidthShift = false; // turn off bandwith shifting with mouse
                    if (delta > 0) // then we are draging to the left  and delta is positive
                    {
                        if (Mode.Text == "LSB" || Mode.Text == "CWL")
                        {
                            if (rcvr.FilterFrequencyHigh > -9999.0f)
                            {
                                rcvr.FilterFrequencyHigh -= delta * 100;
                                tx.FilterFrequencyHigh -= delta * 100;
                            }
                        }
                        else
                        {
                            if (rcvr.FilterFrequencyLow > -9999.0f)
                            {
                                rcvr.FilterFrequencyLow -= delta * 100;
                                tx.FilterFrequencyLow -= delta * 100;
                            }
                        }
                    }
                    if (delta < 0) // then we are draging to the right  and delta is negative
                    {
                        if (Mode.Text == "LSB" || Mode.Text == "CWL")
                        {
                            if (rcvr.FilterFrequencyHigh < 9999.0f)
                            {
                                rcvr.FilterFrequencyHigh -= delta * 100;
                                tx.FilterFrequencyHigh -= delta * 100;
                            }
                        }
                        else
                        {
                            if (rcvr.FilterFrequencyLow < 9999.0f)
                            {
                                rcvr.FilterFrequencyLow -= delta * 100;
                                tx.FilterFrequencyLow -= delta * 100;
                            }
                        }
                    }
                }
            }

            // If the mouse is in the region of the right edge of the filter allow it to be drag tuned
            // Placing the mouse in this region will alter the cursor, if the left mouse button is
            // down then the filter can be altered by draging the mouse left or right.
            // If the user has pressed the left mouse button whilst in this region then HighFilter will be true
            else if (MousePositionX > FilterRight - 5 && MousePositionX < FilterRight || HighFilter)
            {
                pictureBoxSpectrum.Cursor = Cursors.SizeWE;        // change cursor to <->
                if (Control.MouseButtons == MouseButtons.Left)
                {
                    BandwidthShift = false; // turn off bandwith shifting with mouse
                    if (delta > 0) // then we are draging to the left and delta is positive
                    {
                        if (Mode.Text == "LSB" || Mode.Text == "CWL") // need to invert if so
                        {
                            if (rcvr.FilterFrequencyLow > -9999.0f)
                            {
                                rcvr.FilterFrequencyLow -= delta * 100;
                                tx.FilterFrequencyLow -= delta * 100;
                            }
                        }
                        else
                        {
                            if (rcvr.FilterFrequencyHigh > -9999.0f)
                            {
                                rcvr.FilterFrequencyHigh -= delta * 100;
                                tx.FilterFrequencyHigh -= delta * 100;
                            }
                        }

                    }
                    if (delta < 0) // then we are dragging to the right and delta is negative
                    {
                        if (Mode.Text == "LSB" || Mode.Text == "CWL") // need to invert if so
                        {
                            if (rcvr.FilterFrequencyLow < 9999.0f)
                            {
                                rcvr.FilterFrequencyLow -= delta * 100;
                                tx.FilterFrequencyLow -= delta * 100;
                            }
                        }
                        else
                        {
                            if (rcvr.FilterFrequencyHigh < 9999.0f)
                            {
                                rcvr.FilterFrequencyHigh -= delta * 100;
                                tx.FilterFrequencyHigh -= delta * 100;
                            }
                        }
                    }

                }
            }

            // If the mouse is over the filter bandwith  change the cursor. If the left mouse button is also down then 
            // drag tune the filter frequency i.e passband tuning
            else if (MousePositionX > FilterLeft + 5 && MousePositionX < FilterRight - 5 || !FrequencyDrag && BandwidthShift)
            {
                pictureBoxSpectrum.Cursor = Cursors.NoMoveHoriz;

                if (Control.MouseButtons == MouseButtons.Left && !FrequencyDrag) // and if the left mouse is down move filter
                {
                    // determine how wide the current filter is
                    int FilterWidth = Math.Abs((int)(rcvr.FilterFrequencyLow - rcvr.FilterFrequencyHigh));
                    int TxFilterWidth = Math.Abs((int)(tx.FilterFrequencyHigh - tx.FilterFrequencyLow));
                    {
                        if (delta < 0)  // then we are dragging to the right and delta is negative
                        {
                            if (Mode.Text == "LSB" || Mode.Text == "CWL")
                            {
                                if (rcvr.FilterFrequencyLow < 9999.0f)  // only drag to the maximum bandwidth
                                {
                                    rcvr.FilterFrequencyLow -= delta * 100;
                                    rcvr.FilterFrequencyHigh = rcvr.FilterFrequencyLow - FilterWidth;
                                    tx.FilterFrequencyLow -= delta * 100;
                                    tx.FilterFrequencyHigh = tx.FilterFrequencyLow - TxFilterWidth;
                                }
                            }
                            else
                            {
                                if (rcvr.FilterFrequencyHigh < 9999.0f)  // only drag to the maximum bandwidth
                                {
                                    rcvr.FilterFrequencyLow -= delta * 100;
                                    rcvr.FilterFrequencyHigh = rcvr.FilterFrequencyLow + FilterWidth; // all other modes
                                    tx.FilterFrequencyLow -= delta * 100;
                                    tx.FilterFrequencyHigh = tx.FilterFrequencyLow + TxFilterWidth; // all other modes
                                }
                            }
                        }
                        if (delta > 0)  // then we are dragging to the left and delta is positive
                        {

                            if (Mode.Text == "LSB" || Mode.Text == "CWL")
                            {
                                if (rcvr.FilterFrequencyHigh > -9999.0f) // only drag to the maximum bandwidth
                                {
                                    rcvr.FilterFrequencyHigh -= delta * 100;
                                    rcvr.FilterFrequencyLow = rcvr.FilterFrequencyHigh + FilterWidth;
                                    tx.FilterFrequencyHigh -= delta * 100;
                                    tx.FilterFrequencyLow = tx.FilterFrequencyHigh + TxFilterWidth;
                                }
                            }
                            else
                            {
                                if (rcvr.FilterFrequencyLow > -9999.0f) // only drag to the maximum bandwidth
                                {
                                    rcvr.FilterFrequencyHigh -= delta * 100;
                                    rcvr.FilterFrequencyLow = rcvr.FilterFrequencyHigh - FilterWidth;
                                    tx.FilterFrequencyHigh -= delta * 100;
                                    tx.FilterFrequencyLow = tx.FilterFrequencyHigh - TxFilterWidth;
                                }
                            }
                        }
                    }
                }
            }
            else
            {
                pictureBoxSpectrum.Cursor = Cursors.Cross;  // restore cursor
            }

            MouseOld = MouseX; // save the last mouse X location for next time the mouse moves
            UpdateGraphs();
        }

        private void pictureBoxSpectrum_MouseUp(object sender, MouseEventArgs e)
        {
            pictureBoxSpectrum.Cursor = Cursors.Cross;
            FrequencyDrag = true;
            BandwidthShift = true;
            HighFilter = false;
            LowFilter = false;
            SquelchDrag = false;
            AGCHangDrag = false;
            AGCKneeDrag = false;
        }

        private void splitContainer1_SplitterMoved(object sender, SplitterEventArgs e)
        {
            UpdateGraphs();
        }

        // show the squelch level the user has set
        private void Squelch_level_Scroll(object sender, EventArgs e)
        {
            Squelch_setting.Text = (Squelch_level.Value / 10).ToString();
        }

        private bool squelchOn = false;

        private bool bandscopeSquelch = false;

        private void Bandscope_squelch_CheckedChanged(object sender, EventArgs e)
        {
            bandscopeSquelch = Bandscope_squelch.Checked;

            if (bandscopeSquelch)
            {
                if (filterSquelch)
                    Filter_squelch.Checked = false;

                squelchOn = true;
            }
            else
            {
                if (!filterSquelch)
                {
                    squelchOn = false;
                    setVolumeSquelched();
                }
            }
            set_frequency.Focus(); // return focus to the frequency slider
        }

        private bool filterSquelch = false;

        private void Filter_squelch_CheckedChanged(object sender, EventArgs e)
        {
            filterSquelch = Filter_squelch.Checked;

            if (filterSquelch)
            {
                if (bandscopeSquelch)
                    Bandscope_squelch.Checked = false;

                squelchOn = true;
            }
            else
            {
                if (!bandscopeSquelch)
                {
                    squelchOn = false;
                    setVolumeSquelched();
                }
            }

            set_frequency.Focus(); // return focus to the frequency slider
        }

        // set the drive level to Penelope 
        public void DriveLevel_Scroll(object sender, EventArgs e)
        {
            Drive.Text = DriveLevel.Value.ToString();
            // apply square law to drive level control
            DriveGain = (float)Math.Pow(DriveLevel.Value, 2) / 10000;  // DriveGain is float from 0 to 1.0
            set_frequency.Focus(); // return focus to the frequency slider
        }

        // Toggle the MOX button 
        private void MOX_Click(object sender, EventArgs e)
        {
            Console.WriteLine("MOX_Set is " + MOX_set); //newwwww
            if (KK_on) // only allow MOX to be set when radio is on 
            {
      //          if (MOX.BackColor == SystemColors.Control)
                if(MOX_set == false)
                {
                    MOX.BackColor = Color.Green;
                    MOX_set = true;
                    if (voice == true)
                    {
                        voice1 = true;
                        HPSDRDevice.runnumber = 0;  //added 2-11-2-16
                    }
                    else { voice1 = false; }
                }
                else if (MOX_set == true)
     //           else
                {
                    MOX.BackColor = SystemColors.Control;
                    MOX_set = false;
                    voice1 = false;
                    HPSDRDevice.runnumber = 0;
                    if (TUN_set)            // Force TUN off if MOX is turning off
                    {
                        TUN.BackColor = SystemColors.Control;
                        TUN_set = false;
                        DriveLevel.Value = PreviousDrive;   // restore the previous drive level
                    }
                }
            }

            DriveLevel_Scroll(this, EventArgs.Empty); // force drive level update
            setVolumeSquelched();  // check if we need to mute the receiver audio
            set_frequency.Focus(); // return focus to the frequency slider
        }

        // Toggle the TUN (Tune) button
        public int PreviousDrive;

        private void TUN_Click(object sender, EventArgs e)
        {
            if (KK_on) // only allow TUN to be set when radio is on 
            {
                if (TUN.BackColor == SystemColors.Control)
                {
                    TUN.BackColor = Color.Green;
                    TUN_set = true;
                    PreviousDrive = DriveLevel.Value;  // keep the current drive level so we can restore it
                    // reduce the drive level by the percentage set by the user on the Setup form
                    // and apply square law to the control
                    DriveLevel.Value = (int)((float)(DriveLevel.Value * (float)(TuneLevel / 100.0f)));
                 //   DriveLevel.Value = (int)((float)Math.Pow(DriveLevel.Value, 2) / 10000 * (float)(TuneLevel / 100.0f));
                    MOX_set = true;                     // Need MOX on when Tune is set 
                    MOX.BackColor = Color.Green;
                }
                else
                {
                    TUN.BackColor = SystemColors.Control;
                    TUN_set = false;
                    MOX_set = false;                   // force MOX off 
                    MOX.BackColor = SystemColors.Control;

                    if (PennyLane || Hermes)
                    {
                        // sleep if PennyLane or Hermes so that no output spik3e is observed
                        System.Threading.Thread.Sleep(500);
                    }

                    DriveLevel.Value = PreviousDrive;   // restore the previous drive level
                }
            }

            DriveLevel_Scroll(this, EventArgs.Empty); // force drive level update
            set_frequency.Focus(); // return focus to the frequency slider
        }

        public HPSDRDevice ourDevice = null;

        //especially for MethodInvoker to use from another thread.
        public void ToggleOnOffButton()
        {
            OnOffButton_Click(this, EventArgs.Empty);
        }

        // CSCore setup below
        public static WriteableBufferingSource receiveProvider2;  //this buffers receive audio to prevent clicks/dropouts
        public static ISoundOut soundOut; //receiver sound out
        public static ISoundOut SidetoneSoundOut;  //sidetone sound out
        public static VolumeSource sidetone_volume;
        public static WriteableBufferingSource mic_provider;  // this buffers mic audio from client

        public static WriteableBufferingSource testmicprovider;  // this buffers mic audio from client for wave out
        public static WriteableBufferingSource testmicprovider2;  // this buffers mic audio from client for processing

        public void OnOffButton_Click(object sender, EventArgs e)
        {
            if (WisdomComplete == true)
            {
                if (OnOffButton.BackColor == SystemColors.Control)
                {
                    OnOffButton.BackColor = Color.Green;
                    OnOffButton.Text = "ON";


                    if ((DSPchanged == true) & (OldDSPFFTSize != GlobalVar.FFTSize))
                    {
                        FFTPerCent = FFTRadioPerCent * (1 - Convert.ToDouble(Zoom_Track_Bar.Value) / 100000.0);
                        FFTHiLimit = (int)Math.Floor(GlobalVar.FFTSize * (0.5 + 0.5 * FFTPerCent));
                        FFTLowLimit = (int)Math.Floor(GlobalVar.FFTSize * (0.5 - 0.5 * FFTPerCent));
                        FFTPixDispRatio = FFTPerCent * GlobalVar.FFTSize / xWidth;
                        FFTPixDispRatioInteger = (int)Math.Floor(FFTPixDispRatio);
                        FFTSpan = (int)Math.Floor(FFTPerCent * GlobalVar.FFTSize);
                        FreqSpan = SampleRate * FFTPerCent;

                        Complex[] SignalBuffer2 = new Complex[GlobalVar.FFTSize];
                    //    float[] PowerSpectrum = new float[GlobalVar.FFTSize];
                        MessageBox.Show("You need to Quit and then Restart the Program.  Program will now close.");
                        //     this.Close();
                        Application.Exit();
                        Environment.Exit(0);
                    }
                    if (KKDevice == KKMethod.USB)
                    {
                        ourDevice = new USBDevice(this);
                        IsInSequence = true;
                    }
                    else
                    {
                        ourDevice = new EthernetDevice(this);
                    }

                    // make sure any changes made while not running are set to the new ourDevice instance
                    SetTXFilterBandwidth();

                    KK_on = true;
                    ourDevice.Start();
                   
                        if(chkLocalAudio == true)
                        {
                    StartAudio();
                        }

                    set_frequency.Focus(); // set focus to the frequency slider
                }
                else
                {
                    OnOffButton.BackColor = SystemColors.Control;
                    OnOffButton.Text = "OFF";
                    PTT = false;                                // force PTT off
                    MOX_set = false;                            // force MOX off
                    previous_OnlyTxOnPTT = OnlyTxOnPTT;         // save this status so we can restore it
                    OnlyTxOnPTT = true;                         // force always PTT off 
                    TUN.BackColor = SystemColors.Control;
                    if (TUN_set)
                    {
                        DriveLevel.Value = PreviousDrive;            // restore the previous drive level
                        DriveLevel_Scroll(this, EventArgs.Empty);   // force drive level update as TUN is on 
                    }
                    TUN_set = false;                            // force Tune off
                    MOX.BackColor = SystemColors.Control;
                    Thread.Sleep(100);                          // Delay so the PTT off is sent to Ozy
                    ourDevice.Stop();
                    ourDevice = null;
                    OnlyTxOnPTT = previous_OnlyTxOnPTT;         // restore the previous status 
                    KK_on = false;                              // set this last so PTT off gets sent
                    set_frequency.Focus(); // set focus to the frequency slider

                    //w3sz section to dispose of computer audio resources when radio turned off
                    try
                    {
                        if (soundOut != null)
                        {
                            soundOut.Stop();
                        }
  
                    }                  
		            catch (Exception ex)
		            {
		                Console.WriteLine(ex.ToString());
		            }

                    try
                        {
                            if (SidetoneSoundOut != null)
                            {
                                SidetoneSoundOut.Stop();
                            }
                        }                  
		            catch (Exception ex)
		            {
		                Console.WriteLine(ex.ToString());
		            }
                    try
                    {
                        if (soundOut != null)
                        {
                            soundOut.Dispose();
                        }
                    }                  
		            catch (Exception ex)
		            {
		                Console.WriteLine(ex.ToString());
		            }

                    try
                    {
                        if (soundOut != null)
                        {
                            soundOut = null;
                        }
                    }                  
		            catch (Exception ex)
		            {
		                Console.WriteLine(ex.ToString());
		            }
                    try
                    {
                        if (SidetoneSoundOut != null)
                        {
                            SidetoneSoundOut.Dispose();
                        }
                    }                  
		            catch (Exception ex)
		            {
		                Console.WriteLine(ex.ToString());
		            }

                    try
                    {
                        if (SidetoneSoundOut != null)
                        {
                            SidetoneSoundOut = null;
                        }
                    }                  
		            catch (Exception ex)
		            {
		                Console.WriteLine(ex.ToString());
		            }
                    //end w3sz section
                }
            }
        }

        private ISoundOut GetSoundOut()
        {
            if (WasapiOut.IsSupportedOnCurrentPlatform)
            {
                return new WasapiOut();
            }
            else
            {
                return new DirectSoundOut();
            }
        }
        SimpleMixer soundSource;

        public static ISoundOut TestWaveOut;

        public void StartAudio()
        
                    //w3sz section below to add computer audio
                    {
                      if (receiveProvider2 == null)
                    {
                            receiveProvider2 = new WriteableBufferingSource(new CSCore.WaveFormat(48000, 16, 1));
                            mic_provider = new WriteableBufferingSource(new CSCore.WaveFormat(48000, 16, 1),35280);
                            mic_provider.FillWithZeros = false;
                            testmicprovider = new WriteableBufferingSource(new CSCore.WaveFormat(48000, 16, 1));
                            testmicprovider2 = new WriteableBufferingSource(new CSCore.WaveFormat(48000, 16, 1));

                            //waveOut for mic test
                            TestWaveOut = GetSoundOut();
                            TestWaveOut.Initialize(testmicprovider);
                            TestWaveOut.Play();
                            if (Properties.Settings.Default.testMicChecked)
                            {
                                TestWaveOut.Volume = (float)Properties.Settings.Default.testMicVol / 10000;
                            }
                            else
                            {
                                TestWaveOut.Volume = 0.0f;
                            }
                            Console.WriteLine("Test Mic Sound initialized");
                            Console.WriteLine("Test Mic soundOut Status = " + TestWaveOut.PlaybackState.ToString());

                            soundSource = new SimpleMixer(1, 48000)
                            {
                                FillWithZeros = true,
                                DivideResult = true        
                            };

                            soundSource.AddSource(
                                new SineGenerator48(CWPitch, 1.0, 0)
                                .AppendSource(x => new VolumeSource(x), out sidetone_volume)
                                );
                            sidetone_volume.Volume = 0.0f;
                            soundSource.AddSource(mic_provider.ToMono().ChangeSampleRate(48000).ToSampleSource());

                            ISoundOut SidetoneSoundOut = GetSoundOut();
                            SidetoneSoundOut.Initialize(soundSource.ToWaveSource());
                            SidetoneSoundOut.Play();
                            SidetoneSoundOut.Volume = 1.0f;

                            ISoundOut soundOut = GetSoundOut();
                            soundOut.Initialize(receiveProvider2);
                            soundOut.Play();
                            soundOut.Volume = 1.0f;
                        }  
                    }

        private void MicReset_Click(object sender, EventArgs e)
        {
            soundSource.AddSource(mic_provider.ToMono().ChangeSampleRate(48000).ToSampleSource());

        }
           

        public void StopAudio()
        {
            try
            {
                if (soundOut != null)
                {
                    soundOut.Stop();
                }

            }                  
		            catch (Exception ex)
		            {
		                Console.WriteLine(ex.ToString());
		            }
            try
            {
                if (soundOut != null)
                {
                    soundOut.Dispose();
                }
            }                  
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }

            try
            {
                if (soundOut != null)
                {
                    soundOut = null;
                }
            }                  
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
            
            try
            {
                if (receiveProvider2 != null)
                {
                    receiveProvider2 = null;
                }

            }                  
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }

        public bool ProcessorOn;
        private void chkClipper_CheckedChanged(object sender, EventArgs e)
        {
            if (chkClipper.Checked)
            {
                ProcessorOn = true;
                ProcessorGain.Enabled = true;
                ProcGaindB.Enabled = true;
            }
            else  // disable the controls
            {
                ProcessorOn = false;
                ProcessorGain.Enabled = false;
                ProcGaindB.Enabled = false;
            }
            // renew the control value
            ProcessorGainChange();
        }

        public bool BassCutOn;
        private void chkBassCut_CheckedChanged(object sender, EventArgs e)
        {
            if (chkBassCut.Checked)
            {
                BassCutOn = true;
            }
            else
            {
                BassCutOn = false;
            }
        }

        public bool VOXOn;

        private void chkVOX_CheckedChanged(object sender, EventArgs e)
        {
            if (chkVOX.Checked)
            {
                VOXOn = true;
            }
            else
            {
                VOXOn = false;
            }
        }

        public bool ClipOn;
        bool ClipWasOn;

        private void HandleClip()
        {
            if (ClipOn && PTTEnable)  // only display clip LED if PTT active
            {
                ClipLED.BackColor = Color.Green;
                ClipWasOn = true;
                ClipOn = false;
            }
            else if (ClipWasOn)
            {
                ClipWasOn = false;
                ClipLED.BackColor = SystemColors.Control;
            }
        }

        // timer2 determines the length of time the Clip LED is on 
        private void timer2_Tick(object sender, EventArgs e)
        {
            HandleClip();
        }

        public bool VOXDelay;

        // timer 3 determines the VOX hang time
        private void timer3_Tick(object sender, EventArgs e)
        {
            timer3.Interval = (int)VOXHangTime.Value;
            if (VOXOn && !TUN_set)  // don't enable VOX when Tuning
            {
                if (VOXDelay)  // set when mic signal is > threshold
                {
                    MOX_set = true;
                    VOXDelay = false;
                }
                else MOX_set = false;
            }
        }

        public bool NoiseGateDelay;
        public bool NoiseGateSwitch;

        private void timer4_Tick(object sender, EventArgs e)
        {

            timer4.Interval = 100;

            if (NoiseGate)
            {
                if (NoiseGateDelay)         // set when mic signal is > threshold
                {
                    NoiseGateSwitch = true;
                    NoiseGateDelay = false;
                }
                else NoiseGateSwitch = false;
            }
        }

        public float MicGain;

        private void MicrophoneGain_Scroll(object sender, EventArgs e)
        {
            // apply square law to gain control and allow max 20dB of gain 
            MicGain = (float)Math.Pow(MicrophoneGain.Value, 2) / 1000.00f;
          //  MicGain = (float)Math.Pow(MicrophoneGain.Value, 2) / 500.0f;  
            // adjust mic gain level here W3SZ 4-28-2016
        }

        public bool NoiseGate;

        private void chkNoiseGate_CheckedChanged(object sender, EventArgs e)
        {
            if (chkNoiseGate.Checked)
            {
                NoiseGate = true;
            }
            else
            {
                NoiseGate = false;
            }
        }

        public float NoiseThreshold;

        private void NoiseGateLevel_Scroll(object sender, EventArgs e)
        {
            NoiseThreshold = 1.0f - (float)Math.Pow(NoiseGateLevel.Value, 2) / 10000.0f;
        }

        private void ProcessorGainChange()
        {
            // get the Processor gain
            ProcGain = ProcessorGain.Value / 10.0f;
            // display the Processor Gain in dB
            ProcGaindB.Text = (20 * Math.Log10(ProcGain)).ToString();
        }

        private void ProcessorGain_Scroll(object sender, EventArgs e)
        {
            ProcessorGainChange();
        }

        private void ProcessorGain_ValueChanged(object sender, EventArgs e)
        {
            ProcessorGainChange();
        }

        private void VOXLevelChange()
        {
            //get the VOX threshold, apply square law to the control 
            VOXThreshold = 1.0f - (float)Math.Pow(VOXLevel.Value, 2) / 10000.0f;
        }

        private void VOXLevel_Scroll(object sender, EventArgs e)
        {
            VOXLevelChange();
        }

        private void VOXLevel_ValueChanged(object sender, EventArgs e)
        {
            VOXLevelChange();
        }


        public List<NetworkInterface> foundNics = new List<NetworkInterface>();

        // this will help us figure out which entries in the found Metis/Hermes/Griffin board list
        // are valid by finding out which replies are on the proper port!
        // This seems necessary because, for example, Phil VK6APH seems to have his Metis board
        // show up on BOTH ethernet ports!  Likely a wiring issue for his ethernet, but
        // if we can sort that out based on which port the discovery reply came in on, and whether
        // that port is on the same subnet with the board.
        public List<NicProperties> nicProperties = new List<NicProperties>();

        public void GetNetworkInterfaces()
        {
            // creat a string that contains the name and speed of each Network adapter 
            NetworkInterface[] nics = NetworkInterface.GetAllNetworkInterfaces();

            foundNics.Clear();
            nicProperties.Clear();

            Network_interfaces = "";
            int adapterNumber = 1;

            foreach (NetworkInterface adapter in nics)
            {
                IPInterfaceProperties properties = adapter.GetIPProperties();

                // if it's not 'up' (operational), ignore it.  (Dan Quigley, 13 Aug 2011)
                if (adapter.OperationalStatus != OperationalStatus.Up)
                    continue;

                // if it's a loopback interface, ignore it!
                if (adapter.NetworkInterfaceType == NetworkInterfaceType.Loopback)
                    continue;

                // get rid of non-ethernet addresses
                if ((adapter.NetworkInterfaceType != NetworkInterfaceType.Ethernet) && (adapter.NetworkInterfaceType != NetworkInterfaceType.GigabitEthernet))
                    continue;

                Console.WriteLine("");      // a blank line
                Console.WriteLine(adapter.Description);
                Console.WriteLine(String.Empty.PadLeft(adapter.Description.Length, '='));
                Console.WriteLine("  Interface type .......................... : {0}", adapter.NetworkInterfaceType);
                Console.WriteLine("  Physical Address ........................ : {0}", adapter.GetPhysicalAddress().ToString());
                Console.WriteLine("  Is receive only.......................... : {0}", adapter.IsReceiveOnly);
                Console.WriteLine("  Multicast................................ : {0}", adapter.SupportsMulticast);
                Console.WriteLine("  Speed    ................................ : {0}", adapter.Speed);

                // list unicast addresses
                UnicastIPAddressInformationCollection c = properties.UnicastAddresses;
                foreach (UnicastIPAddressInformation a in c)
                {
                    IPAddress addr = a.Address;
                    Console.WriteLine("  Unicast Addr ............................ : {0}", addr.ToString());
                    IPAddress mask = a.IPv4Mask;
                    Console.WriteLine("  Unicast Mask ............................ : {0}", (mask == null ? "null" : mask.ToString()));

                    NicProperties np = new NicProperties();
                    np.ipv4Address = a.Address;
                    np.ipv4Mask = a.IPv4Mask;

                    nicProperties.Add(np);
                }

                // list multicast addresses
                MulticastIPAddressInformationCollection m = properties.MulticastAddresses;
                foreach (MulticastIPAddressInformation a in m)
                {
                    IPAddress addr = a.Address;
                    Console.WriteLine("  Multicast Addr .......................... : {0}", addr.ToString());
                }

                // if the length of the network adapter name is > 31 characters then trim it, if shorter then pad to 31.
                // Need to use fixed width font - Courier New
                string speed = "  " + (adapter.Speed / 1000000).ToString() + "T";
                if (adapter.Description.Length > 31)
                {
                    Network_interfaces += adapterNumber++.ToString() + ". " + adapter.Description.Remove(31) + speed + "\n";
                }
                else
                {
                    Network_interfaces += adapterNumber++.ToString() + ". " + adapter.Description.PadRight(31, ' ') + speed + "\n";
                }

                foundNics.Add(adapter);
            }

            Console.WriteLine(Network_interfaces);

            // display number of adapters on Setup form
            numberOfIPAdapters = (adapterNumber - 1).ToString();
        }

        private void deviceConfigToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // test whether running or not.  If running, warn and bail out
            if (KK_on)
            {
                MessageBox.Show("You press the ON button to stop data before changing device connectivity method", "KISS is ON!");
                return;
            }

            // possible way to choose Ethernet or USB ...
            if (SetupFormValid())
            {
                Setup_form.Close();
            }
            Setup_form = null;

            // show form to choose USB or Ethernet
            DeviceTypeForm dtf = new DeviceTypeForm(KKDevice);

            KKMethod oldDevice = KKDevice;

            DialogResult dr = dtf.ShowDialog();
            if (dr == DialogResult.OK)
            {
                KKDevice = dtf.GetChosenDeviceType();

                if (KKDevice != oldDevice)
                    ourDevice = null;
            }
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            KISSAboutBox k = new KISSAboutBox();
            k.ShowDialog();
        }

        private void Zoom100_CheckedChanged(object sender, EventArgs e)
        {
            if (Zoom100.Checked == true)
            {
                FFTRadioPerCent = 1;
                Zoom_Track_Bar.Value = 0;
                FFTHiLimit = GlobalVar.FFTSize;
                FFTLowLimit = 0;
                FFTPerCent = 1;
                FFTPixDispRatio = GlobalVar.FFTSize / xWidth;
                FFTPixDispRatioInteger = (int)Math.Floor(FFTPixDispRatio);
                FFTSpan = GlobalVar.FFTSize;
                FreqSpan = SampleRate;
            }
        }

        private void Zoom75_CheckedChanged(object sender, EventArgs e)
        {
            if (Zoom75.Checked == true)
            {
                FFTRadioPerCent = 0.75;
                Zoom_Track_Bar.Value = 0;
                FFTHiLimit = (int)Math.Floor(GlobalVar.FFTSize * (0.875));
                FFTLowLimit = (int)Math.Floor(GlobalVar.FFTSize * (0.125));
                FFTPerCent = 0.75;
                FFTPixDispRatio = 0.75 * GlobalVar.FFTSize / xWidth;
                FFTPixDispRatioInteger = (int)Math.Floor(FFTPixDispRatio);
                FFTSpan = (int)Math.Floor(FFTPerCent * GlobalVar.FFTSize);
                FreqSpan = SampleRate * 0.75;
            }
        }

        private void Zoom50_CheckedChanged(object sender, EventArgs e)
        {
            if (Zoom50.Checked == true)
            {
                FFTRadioPerCent = 0.50;
                Zoom_Track_Bar.Value = 0;
                FFTHiLimit = (int)Math.Floor(GlobalVar.FFTSize * (0.75));
                FFTLowLimit = (int)Math.Floor(GlobalVar.FFTSize * (0.25));
                FFTPerCent = 0.50;
                FFTPixDispRatio = 0.50 * GlobalVar.FFTSize / xWidth;
                FFTPixDispRatioInteger = (int)Math.Floor(FFTPixDispRatio);
                FFTSpan = (int)Math.Floor(FFTPerCent * GlobalVar.FFTSize);
                FreqSpan = SampleRate * 0.50;
            }
        }

        private void Zoom25_CheckedChanged(object sender, EventArgs e)
        {
            if (Zoom25.Checked == true)
            {
                if (GlobalVar.FFTSize == 4096)
                {
                    FFTRadioPerCent = 0.50;
                    FFTPerCent = 0.50;
                    Zoom_Track_Bar.Value = 0;
                    FFTHiLimit = (int)Math.Floor(GlobalVar.FFTSize * (0.75));
                    FFTLowLimit = (int)Math.Floor(GlobalVar.FFTSize * (0.25));
                    FFTPixDispRatio = 0.50 * GlobalVar.FFTSize / xWidth;
                    FFTPixDispRatioInteger = (int)Math.Floor(FFTPixDispRatio);
                    FFTSpan = (int)Math.Floor(FFTPerCent * GlobalVar.FFTSize);
                    FreqSpan = SampleRate * 0.50;
                }
                else
                {
                    FFTRadioPerCent = 0.25;
                    FFTPerCent = 0.25;
                    Zoom_Track_Bar.Value = 0;
                    FFTHiLimit = (int)Math.Floor(GlobalVar.FFTSize * (0.625));
                    FFTLowLimit = (int)Math.Floor(GlobalVar.FFTSize * (0.375));
                    FFTPixDispRatio = 0.25 * GlobalVar.FFTSize / xWidth;
                    FFTPixDispRatioInteger = (int)Math.Floor(FFTPixDispRatio);
                    FFTSpan = (int)Math.Floor(FFTPerCent * GlobalVar.FFTSize);
                    FreqSpan = SampleRate * 0.25;
                }
            }
        }

        private void Zoom_Track_Bar_Scroll(object sender, EventArgs e)
        {
            if (Zoom25.Checked == true)
            {
                if ((GlobalVar.FFTSize == 524288) || (GlobalVar.FFTSize == 262144))
                {
                    FFTPerCent = FFTRadioPerCent - (Convert.ToDouble(Zoom_Track_Bar.Value) * 0.95 / 100000.0);
                }
                else if (GlobalVar.FFTSize == 131072)
                {
                    FFTPerCent = FFTRadioPerCent - (Convert.ToDouble(Zoom_Track_Bar.Value) * 0.90 / 100000.0);
                }
                else if (GlobalVar.FFTSize == 65536)
                {
                    FFTPerCent = FFTRadioPerCent - (Convert.ToDouble(Zoom_Track_Bar.Value) * 0.85 / 100000.0);
                }
                else if (GlobalVar.FFTSize == 32768)
                {
                    FFTPerCent = FFTRadioPerCent - (Convert.ToDouble(Zoom_Track_Bar.Value) * 0.75 / 100000.0);
                }
                else if (GlobalVar.FFTSize == 16384)
                {
                    FFTPerCent = FFTRadioPerCent - (Convert.ToDouble(Zoom_Track_Bar.Value) * 0.45 / 100000.0);
                }
                else if (GlobalVar.FFTSize == 8192)
                {
                    FFTPerCent = FFTRadioPerCent;
                }
                else if (GlobalVar.FFTSize == 4096)
                {
                    FFTPerCent = 2 * FFTRadioPerCent;
                }
                FFTHiLimit = (int)Math.Floor(GlobalVar.FFTSize * (0.5 + 0.5 * FFTPerCent));
                FFTLowLimit = (int)Math.Floor(GlobalVar.FFTSize * (0.5 - 0.5 * FFTPerCent));
                FFTPixDispRatio = FFTPerCent * GlobalVar.FFTSize / xWidth;
                FFTPixDispRatioInteger = (int)Math.Floor(FFTPixDispRatio);
                FFTSpan = (int)Math.Floor(FFTPerCent * GlobalVar.FFTSize);
                FreqSpan = SampleRate * FFTPerCent;
            }
            else
            {
                if ((GlobalVar.FFTSize == 4096) & (Zoom50.Checked == true))
                {
                    FFTPerCent = FFTRadioPerCent;
                }
                else
                {
                    FFTPerCent = FFTRadioPerCent - (Convert.ToDouble(Zoom_Track_Bar.Value) / 100000.0);
                }
                FFTHiLimit = (int)Math.Floor(GlobalVar.FFTSize * (0.5 + 0.5 * FFTPerCent));
                FFTLowLimit = (int)Math.Floor(GlobalVar.FFTSize * (0.5 - 0.5 * FFTPerCent));
                FFTPixDispRatio = FFTPerCent * GlobalVar.FFTSize / xWidth;
                FFTPixDispRatioInteger = (int)Math.Floor(FFTPixDispRatio);
                FFTSpan = (int)Math.Floor(FFTPerCent * GlobalVar.FFTSize);
                FreqSpan = SampleRate * FFTPerCent;
            }
        }
        // Return magnitude of complex array 
        public float[] magnitude(Complex[] spectrum)
        {
            float[] pmag;
            pmag = new float[spectrum.Length];
            for (int i = 0; i < spectrum.Length; i++)
            {
               pmag[i] = Convert.ToSingle(spectrum[i].Magnitude);
            }

            return pmag;
        }

        public float magnitude(Complex spectrum)
        {
            float pmag;
            pmag = Convert.ToSingle(spectrum.Magnitude);
            return pmag;
        }



        void backgroundWorker1_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            float[] PowerSpectrum = new float[FFTSize];

            if (e.Error != null)
            {
                Console.WriteLine("BackgroundWorker1 had an exception: {0}", e.Error.Message);
            }
            else if (e.Cancelled)
            {
                Console.WriteLine("Cancelled");
            }
            else
            {
                PowerSpectrum = (float[])e.Result;
                if (PowerSpectrum != null)
                {
                    ComputeDisplay(PowerSpectrum);
                }
            }
        }


        public void backgroundWorker1_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {

        }

        public void FilterBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            FilterChange();
        }

        public void FilterChange()
        {
            switch (FilterBox.Text)
            {
                case "HANN":
    //                Console.WriteLine("FilterBox.Text = " + FilterBox.Text);
                    rcvr.WindowType = SharpDSP2._1.WindowType_e.HANNING_WINDOW;
                    break;
                case "BLACKMAN-2":
    //                Console.WriteLine("FilterBox.Text = " + FilterBox.Text);
                    rcvr.WindowType = SharpDSP2._1.WindowType_e.BLACKMAN2_WINDOW;
                break;
                case "BLACKMAN-3":
    //                Console.WriteLine("FilterBox.Text = " + FilterBox.Text);
                    rcvr.WindowType = SharpDSP2._1.WindowType_e.BLACKMAN3_WINDOW;
                    break;
                case "BLACKMAN-4":
    //                Console.WriteLine("FilterBox.Text = " + FilterBox.Text);
                    rcvr.WindowType = SharpDSP2._1.WindowType_e.BLACKMAN4_WINDOW;
                    break;
                case "BLACKMAN-HARRIS-7":
    //                Console.WriteLine("FilterBox.Text = " + FilterBox.Text);
                    rcvr.WindowType = SharpDSP2._1.WindowType_e.BLACKMAN7_WINDOW;
                    break;
                case "BLACKMAN-HARRIS-4":
    //                Console.WriteLine("FilterBox.Text = " + FilterBox.Text);
                    rcvr.WindowType = SharpDSP2._1.WindowType_e.BLACKMANHARRIS_WINDOW;
                    break;
                case "RECTANGULAR":
    //                Console.WriteLine("FilterBox.Text = " + FilterBox.Text);
                    rcvr.WindowType = SharpDSP2._1.WindowType_e.RECTANGULAR_WINDOW;
                    break;
                case "WELCH":
    //                Console.WriteLine("FilterBox.Text = " + FilterBox.Text);
                    rcvr.WindowType = SharpDSP2._1.WindowType_e.WELCH_WINDOW;
                    break;
                case "PARZEN":
    //                Console.WriteLine("FilterBox.Text = " + FilterBox.Text);
                    rcvr.WindowType = SharpDSP2._1.WindowType_e.PARZEN_WINDOW;
                    break;
                case "BARTLETT":
    //                Console.WriteLine("FilterBox.Text = " + FilterBox.Text);
                    rcvr.WindowType = SharpDSP2._1.WindowType_e.BARTLETT_WINDOW;
                    break;
                case "HAMMING":
    //                Console.WriteLine("FilterBox.Text = " + FilterBox.Text);
                    rcvr.WindowType = SharpDSP2._1.WindowType_e.HAMMING_WINDOW;
                    break;
                case "EXPONENTIAL":
    //                Console.WriteLine("FilterBox.Text = " + FilterBox.Text);
                    rcvr.WindowType = SharpDSP2._1.WindowType_e.EXPONENTIAL_WINDOW;
                    break;
                case "RIEMANN":
    //                Console.WriteLine("FilterBox.Text = " + FilterBox.Text);
                    rcvr.WindowType = SharpDSP2._1.WindowType_e.RIEMANN_WINDOW;
                    break;
                case "DOLPHCHEB":
                    //                Console.WriteLine("FilterBox.Text = " + FilterBox.Text);
                    rcvr.WindowType = SharpDSP2._1.WindowType_e.DOLPHCHEB_WINDOW;
                    break;
                case "KAISER":
                    //                Console.WriteLine("FilterBox.Text = " + FilterBox.Text);                   
                    rcvr.WindowType = SharpDSP2._1.WindowType_e.KAISER_WINDOW;
                    break;
                default:
                    Console.WriteLine("NO acceptable filter found");
                    break;

            }
            Properties.Settings.Default.FilterType = FilterBox.Text;
            Properties.Settings.Default.Save();
            AdjustFilterByMode();
     //       Console.WriteLine("New Filter Type is: " + rcvr.WindowType.ToString());
     //       Console.WriteLine("Saved filter is " + Properties.Settings.Default.FilterType);
        }

        public void alphaUpDown_ValueChanged(object sender, EventArgs e)
        {
             alpha = alphaUpDown.Value;
            Properties.Settings.Default.alpha = alphaUpDown.Value;
            Properties.Settings.Default.Save();
            if(rcvr != null) rcvr.Alpha = alpha;
        }

        private void x10Box_CheckedChanged(object sender, EventArgs e)
        {
            if (x10Box.Checked)
                {
                alphaUpDown.Increment = 1.0M;
                }
            else
            {
                alphaUpDown.Increment = 0.1M;
            }
        }

        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboBox1.Text.Length > 0)
            {
                Properties.Settings.Default.DCWAtten = int.Parse(comboBox1.Text);
                Properties.Settings.Default.Save();
                getDCW();
                try
                {
                    AdjustFilterByMode();
                }                  
	            catch (Exception ex)
	            {
	                Console.WriteLine(ex.ToString());
	            }
            }
        }

        private void splitBox_CheckedChanged(object sender, EventArgs e)
        {
            if(splitBox.Checked)
            {
                Properties.Settings.Default.split = true;
                Properties.Settings.Default.Save();
            }
            else
            {
                Properties.Settings.Default.split = false;
                Properties.Settings.Default.Save();
            }
        }

        private void AtoBBut_Click(object sender, EventArgs e)
        {
            Properties.Settings.Default.split_tx_freq = set_frequency.Value;
            Properties.Settings.Default.Save();
            DisplayFrequencyB(Properties.Settings.Default.split_tx_freq);
        }

        private void BtoABut_Click(object sender, EventArgs e)
        {
            try
            {
            set_frequency.Value = (int)Properties.Settings.Default.split_tx_freq;
            DisplayFrequency(set_frequency.Value);
        }
            catch(Exception ex)
            {
                Debug.WriteLine(ex.ToString());
            }
        }

        bool priorBassCut = false;
        bool priorMicAGC = false;
        bool priorspeechProcessor = false;
        bool priorNoiseGate = false;
        int priorBitrate = 8192;
        int priorBandwidth = OPUS_BANDWIDTH_NARROWBAND;
        string priorMode = "USB";
        int priorTxFilter = 2300;
        int priorRxFilter = 2300;
        //this saves the most recent encoder state to use for digital modes
        public OpusEncoder digitalEncoder = OpusEncoder.Create(48000, 1, FragLabs.Audio.Codecs.Opus.Application.Audio);
        //this saves the most recent encoder state to use for voice modesmic_

        public OpusEncoder voiceEncoder = OpusEncoder.Create(48000, 1, FragLabs.Audio.Codecs.Opus.Application.Voip);

        private void digiChk_CheckedChanged(object sender, EventArgs e)
        {
            if(digiChk.Checked)
            {
                priorBassCut = chkBassCut.Checked;
                chkBassCut.Checked = false;
                priorMicAGC = chkMicAGC.Checked;
                chkMicAGC.Checked = false;
                priorspeechProcessor = chkClipper.Checked;
                chkClipper.Checked = false;
                priorNoiseGate = chkNoiseGate.Checked;
                chkNoiseGate.Checked = false;
                
                priorRxFilter = BandwidthTrackBar.Value;
                priorMode = Mode.Text;
                voiceEncoder = _encoder;
                voiceEncoder.Bitrate = _encoder.Bitrate;
                voiceEncoder.Bandwidth = _encoder.Bandwidth;
                _encoder = digitalEncoder;
                _encoder.Bitrate = Properties.Settings.Default.dataBitrate;
                _encoder.Bandwidth = Properties.Settings.Default.dataBandwidth;
                Mode.Text = "USB";
                ChangeMode();
                BandwidthTrackBar.Value = Properties.Settings.Default.DigitalFilterWidth;
                BandwidthTrackBar_Scroll(this, EventArgs.Empty);
            }
            else if(!digiChk.Checked)
            {
                digitalEncoder = _encoder;
                digitalEncoder.Bitrate = _encoder.Bitrate;
                digitalEncoder.Bandwidth = _encoder.Bandwidth;
                _encoder = voiceEncoder;
                _encoder.Bitrate = Properties.Settings.Default.voiceBitrate;
                _encoder.Bandwidth = Properties.Settings.Default.voiceBandwidth;
                Properties.Settings.Default.DigitalFilterWidth = BandwidthTrackBar.Value;
                Properties.Settings.Default.Save();
                chkBassCut.Checked = priorBassCut;
                chkMicAGC.Checked = priorMicAGC;
                chkClipper.Checked = priorspeechProcessor;
                chkNoiseGate.Checked = priorNoiseGate;
                Mode.Text = priorMode;
                ChangeMode();
                BandwidthTrackBar.Value = Math.Abs(priorRxFilter);
                BandwidthTrackBar_Scroll(this, EventArgs.Empty);
            }
        }

        public float oldfagclimit = 2.0f;
        private void fAGCLimitcomboBox2_SelectedIndexChanged(object sender, EventArgs e)
        {
            float limit = 2.0f;

            bool testlimit = float.TryParse(fAGCLimitcomboBox2.Text, out limit);
            if (testlimit)
            {
                try
                {
                    rcvr.freqAGCLimit = limit;
                }                  
	            catch (Exception ex)
	            {
	                Console.WriteLine(ex.ToString());
	            }
                oldfagclimit = limit;
                Properties.Settings.Default.fagcLimit = limit;
                Properties.Settings.Default.Save();
            }
            else
            {
                fAGCLimitcomboBox2.Text = oldfagclimit.ToString();
                return;
            }

        }

        public float oldfagcknee = 0.02f;
        private void fagcKneecomboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            float knee = 0.02f;

            bool testknee = float.TryParse(fagcKneecomboBox.Text, out knee);
            if (testknee)
            {
                try
                {
                    rcvr.freqAGCKnee = knee;
                }                  
	            catch (Exception ex)
	            {
	                Console.WriteLine(ex.ToString());
	            }
                oldfagcknee = knee;
                Properties.Settings.Default.fagcKnee = knee;
                Properties.Settings.Default.Save();
            }
            else
            {
                fagcKneecomboBox.Text = oldfagcknee.ToString();
                return;
            }

        }

        public void backgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
        {
            int blocksize = GlobalVar.DSPBlockSize;
            int topsize = (blocksize * 2) - 1;
            int i, j, k;
            float two_pi = 8.0F * (float)Math.Atan(1.0F);
            float
                            a0 = 0.35875F,
                            a1 = 0.48829F,
                            a2 = 0.14128F,
                            a3 = 0.01168F;


            IntPtr pin, pout, fplan1;


            // create two unmanaged arrays, properly aligned
            pin = fftwf.malloc(FFTSize * 8);
            pout = fftwf.malloc(FFTSize * 8);


            float[] fin, fout;

            fin = new float[FFTSize * 2];
            fout = new float[FFTSize * 2];

            Complex[] ComplexTimeDomain1 = (Complex[])e.Argument;  //input to FFT
            Complex[] ComplexTimeDomain = new Complex[2*GlobalVar.DSPBlockSize]; //input to FFT
            int fftLength = ComplexTimeDomain.Length;

         //   Console.WriteLine("ComplexTImeDomain1 size is {0} and blocksize is {1} and ComplexTimeDoman size is {2}", ComplexTimeDomain1.Length, blocksize,ComplexTimeDomain.Length);


            float[] window;
            window = new float[GlobalVar.DSPBlockSize];

                // window the data
                for (i = 0; i < blocksize; i++)
                {
                   window[i] = a0 - a1 * (float)Math.Cos(two_pi * (float)(i + 0.5) / (float)blocksize)
                            + a2 * (float)Math.Cos(2.0 * two_pi * (float)(i + 0.5) / (float)blocksize)
                            - a3 * (float)Math.Cos(3.0 * two_pi * (float)(i + 0.5) / (float)blocksize);
                   ComplexTimeDomain[i] = Complex.Multiply(ComplexTimeDomain1[i], window[i]);
                }

                for (k = 0; k < FFTSize; k++)
                {
                    fin[2 * k] = (float)ComplexTimeDomain[k].Real;
                    fin[1 + 2 * k] = (float)ComplexTimeDomain[k].Imaginary;
                    fout[k] = (float)0;
                }

            float[] PowerSpectrum = new float[GlobalVar.FFTSize];

            // import wisdom and then create a plan
             string WisdomName = app_data_path + "Wisdom";
             int wisdomSuccess = 0;
             wisdomSuccess = fftwf.import_wisdom(WisdomName);
             if (wisdomSuccess == 0)
             {
                 MessageBox.Show("Unable to open Wisdom File.  File is either missing or of incorrect bitness (64 vs 32)");
                 Console.WriteLine("unable to open wisdom file at {0}", WisdomName);
             }

            fplan1 = fftwf.dft_1d(FFTSize, pin, pout, fftw_direction.Forward, fftw_flags.Patient);
            Marshal.Copy(fin, 0, pin, fin.Length);
            fftwf.execute(fplan1);
            Marshal.Copy(pout, fout, 0, fout.Length);

            for (j = 0; j < blocksize - 1; j++) // do reorder and flip here //was blocksize 
            {
                PowerSpectrum[topsize - j] = 
                   (float)(10.0 * Math.Log10(fout[2 * (j + blocksize)] * fout[2 * (j + blocksize)] + fout[2 * (j + blocksize) + 1] * fout[2 * (j + blocksize + 1)] + 1e-180));  

                PowerSpectrum[blocksize - j] =  
                    (float)(10.0 * Math.Log10(fout[2 * j] * fout[2 * j] + fout[(2 * j) + 1] * fout[(2 * j) + 1] + 1e-180));
            }

            fftwf.free(pin);
            fftwf.free(pout);
            fftwf.destroy_plan(fplan1);

            e.Result = PowerSpectrum;

        }

        //Backgroundworker2 is Receive Audio Processor
        
        const int MainAudioRingBufferSize = 32768;  //was 8192 then 32768
        SharpDSP2._1.RingBuffer MainAudioRing = new SharpDSP2._1.RingBuffer(MainAudioRingBufferSize);
        public static byte[] FinalAudioData = new byte[48000]; //w3sz
        protected byte[] AudioByte = new byte[48000];
        public static int AudioLength = 12000;
        float SampleReal = 0.0f;  //was 1.0f
        float SampleImag = 0.0f;  //was 1.0f
        protected float scaleOut = (float)Math.Pow(2, 15);  // scale factor to convert DSP output to 16-bit short int audio output sample\

        byte[] buff;
        public OpusEncoder _encoder;
        byte[] _notEncodedBuffer = new byte[0];
        int _segmentFrames;
        int _bytesPerSegment;
        public static ConcurrentQueue<byte[]> mainrec_data_queue;
        public static ConcurrentQueue<byte[]> auxrec_data_queue;
        byte[] buff2;

        private void timer5_Tick(object sender, EventArgs e)
        {
            timer5.Interval = 20;
            OnDotOrDashEvent();
        }

      
     }

    internal class FFTW
    {
        private const string library_name = "libfftw3f-3.dll";  //w3sz was libfftw3f.dll

        internal enum fftw_direction : int
        {
            Forward = -1,
            Backward = 1
        }

        internal enum fftw_flags : uint
        {
            Measure = 0,
            DestroyInput = 1,
            Unaligned = 2,
            ConserveMemory = 4,
            Exhaustive = 8,
            PreserveInput = 16,
            Patient = 32,
            Estimate = 64
        }

        [DllImport(library_name,
        EntryPoint = "fftwf_destroy_plan",
        ExactSpelling = true,
        CallingConvention = CallingConvention.Cdecl),
        System.Security.SuppressUnmanagedCodeSecurity]
        internal static extern void destroy_plan(IntPtr plan);

        [DllImport(library_name,
        EntryPoint = "fftwf_cleanup",
        ExactSpelling = true,
        CallingConvention = CallingConvention.Cdecl),
        System.Security.SuppressUnmanagedCodeSecurity]
        internal static extern void cleanup();

        [DllImport(library_name,
        EntryPoint = "fftwf_execute",
        ExactSpelling = true,
        CallingConvention = CallingConvention.Cdecl),
        System.Security.SuppressUnmanagedCodeSecurity]
        internal static extern void execute(IntPtr plan);

        [DllImport(library_name,
        EntryPoint = "fftwf_plan_dft_1d",
        ExactSpelling = true,
        CallingConvention = CallingConvention.Cdecl),
        System.Security.SuppressUnmanagedCodeSecurity]
        internal static extern IntPtr dft_1d(int n, IntPtr input, IntPtr output,
        fftw_direction direction, fftw_flags flags);
    }     

    public enum OperMode : int
    {
        AM,
        FM,
        SAM,
        USB,
        LSB,
        CWU,
        CWL
    };

    public enum OpBand : int
    {
        M160,
        M80,
        M60,
        M40,
        M30,
        M20,
        M17,
        M15,
        M12,
        M10,
        M6,
        GC,
        XVRT1,
        XVRT2
    };


    public enum DeviceType : int
    {
        Metis = 0,
        Hermes = 1,
        Griffin = 2
    }

    public class MetisHermesDevice
    {
        public DeviceType deviceType;   // which type of device (currently Metis or Hermes)
        public byte codeVersion;        // reported code version type
        public bool InUse;              // whether already in use
        public string IPAddress;        // currently, an IPV4 address
        public string MACAddress;       // a physical (MAC) address
        public IPAddress hostPortIPAddress;
    }

    public class NicProperties
    {
        public IPAddress ipv4Address;
        public IPAddress ipv4Mask;
    }

    public enum KKMethod : int
    {
        USB = 0,
        Ethernet = 1
    }

    public class SimpleMixer : ISampleSource
    {
        private readonly CSCore.WaveFormat _waveFormat;
        private readonly List<ISampleSource> _sampleSources = new List<ISampleSource>();
        private readonly object _lockObj = new object();
        private float[] _mixerBuffer;

        public bool FillWithZeros { get; set; }

        public bool DivideResult { get; set; }

        public SimpleMixer(int channelCount, int sampleRate)
        {
            if (channelCount < 1)
                throw new ArgumentOutOfRangeException("channelCount");
            if (sampleRate < 1)
                throw new ArgumentOutOfRangeException("sampleRate");

            _waveFormat = new CSCore.WaveFormat(sampleRate, 32, channelCount, AudioEncoding.IeeeFloat);
        }

        public void AddSource(ISampleSource source)
        {
            if (source == null)
                throw new ArgumentNullException("source");

            if (source.WaveFormat.Channels != WaveFormat.Channels ||
               source.WaveFormat.SampleRate != WaveFormat.SampleRate)
                throw new ArgumentException("Invalid format.", "source");

            lock (_lockObj)
            {
                if (!Contains(source))
                    _sampleSources.Add(source);
            }
        }

        public void RemoveSource(ISampleSource source)
        {
            //don't throw null ex here
            lock (_lockObj)
            {
                if (Contains(source))
                    _sampleSources.Remove(source);
            }
        }

        public bool Contains(ISampleSource source)
        {
            if (source == null)
                return false;
            return _sampleSources.Contains(source);
        }

        public int Read(float[] buffer, int offset, int count)
        {
            int numberOfStoredSamples = 0;
            try
            {

                if (count > 0 && _sampleSources.Count > 0)
                {
                    lock (_lockObj)
                    {
                        _mixerBuffer = _mixerBuffer.CheckBuffer(count);
                        List<int> numberOfReadSamples = new List<int>();
                        for (int m = _sampleSources.Count - 1; m >= 0; m--)
                        {
                            var sampleSource = _sampleSources[m];
                            int read = sampleSource.Read(_mixerBuffer, 0, count);
                            for (int i = offset, n = 0; n < read; i++, n++)
                            {
                                if (numberOfStoredSamples < i)
                                    buffer[i] = _mixerBuffer[n];
                                else
                                    buffer[i] += _mixerBuffer[n];
                            }
                            if (read > numberOfStoredSamples)
                                numberOfStoredSamples = read;

                            if (read > 0)
                                numberOfReadSamples.Add(read);
                            else
                            {
                                //raise event here
                                RemoveSource(sampleSource); //remove the input to make sure that the event gets only raised once.
                            }
                        }

                        if (DivideResult)
                        {
                            numberOfReadSamples.Sort();
                            int currentOffset = offset;
                            int remainingSources = numberOfReadSamples.Count;

                            foreach (var readSamples in numberOfReadSamples)
                            {
                                if (remainingSources == 0)
                                    break;

                                while (currentOffset < offset + readSamples)
                                {
                                    buffer[currentOffset] /= remainingSources;
                                    buffer[currentOffset] = Math.Max(-1, Math.Min(1, buffer[currentOffset]));
                                    currentOffset++;
                                }
                                remainingSources--;
                            }
                        }
                    }
                }


                if (FillWithZeros && numberOfStoredSamples != count)
                {
                    Array.Clear(
                        buffer,
                        Math.Max(offset + numberOfStoredSamples - 1, 0),
                        count - numberOfStoredSamples);

                    return count;
                }
            }                  
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }

            return numberOfStoredSamples;
        }

        public bool CanSeek { get { return false; } }

        public CSCore.WaveFormat WaveFormat
        {
            get { return _waveFormat; }
        }

        public long Position
        {
            get { return 0; }
            set
            {
                throw new NotSupportedException();
            }
        }

        public long Length
        {
            get { return 0; }
        }

        public void Dispose()
        {
            lock (_lockObj)
            {
                {
                    try
                    {
                        foreach (var sampleSource in _sampleSources)
                        {
                            if (sampleSource != null)
                                sampleSource.Dispose();
                            _sampleSources.Remove(sampleSource);
                        }
                    }                  
		            catch (Exception ex)
		            {
		                Console.WriteLine(ex.ToString());
		            }
                }
            }
        }
    }
}


